@@ -1,5 +1,6 @@
 #include "license_pbs.h" /* See here for the software license */
 #include "qsub_functions.h"
+#include "net_cache.h"
 
 /** 
  * qsub main - processing moved to another function to facilitate unit testing
@@ -12,6 +13,8 @@ int main(
   char **envp)  /* I */
 
   {
+  initialize_network_info();
+
   main_func(argc,argv,envp);
 
   return(0);
