@@ -22,6 +22,7 @@
 #define ARRAY_TOO_LARGE    -5
 #define INVALID_SLOT_LIMIT -6
 #define INITIAL_NUM_ARRAYS  50
+#define NO_JOBS_IN_ARRAY   -21
 
 #define ARRAY_FILE_SUFFIX ".AR"
 
@@ -48,7 +49,6 @@ typedef struct
    this was done without consideration to backward compatibility. */
 struct job_array_259
   {
-  list_link all_arrays;      /* node in server's linked list of all arrays */
   tlist_head request_tokens; /* head of linked list of request tokens, used 
                                 during cloning (cloning is the process of 
                                 copying the "template" job to generate all of the 
@@ -97,7 +97,6 @@ typedef struct job_array_259 job_array_259;
 
 struct job_array
   {
-  list_link all_arrays;      /* node in server's linked list of all arrays */
   tlist_head request_tokens; /* head of linked list of request tokens, used 
                                 during cloning (cloning is the process of 
                                 copying the "template" job to generate all of the 
@@ -163,8 +162,8 @@ int delete_array_range(job_array *pa, char *range);
 int delete_whole_array(job_array *pa);
 int attempt_delete(void *);
 
-int hold_array_range(job_array *,char *,attribute *);
-void hold_job(attribute *,void *);
+int hold_array_range(job_array *,char *,pbs_attribute *);
+void hold_job(pbs_attribute *,void *);
 
 int modify_array_range(job_array *,char *,svrattrl *,struct batch_request *,int);
 int modify_job(void **,svrattrl *,struct batch_request *,int, int);
