@@ -125,7 +125,6 @@ typedef unsigned long pbs_net_t;        /* for holding host addresses */
 /* sync w/PBSServerCmds[] in resmom/mom_main.c and PBSServerCmds2[] in server/node_manager.c */
 
 #define IS_NULL           0
-#define IS_HELLO          1
 #define IS_CLUSTER_ADDRS  2
 #define IS_UPDATE         3
 #define IS_STATUS         4
@@ -192,6 +191,7 @@ enum conn_type
 /* functions available in libnet.a */
 
 int add_conn(int, enum conn_type, pbs_net_t, unsigned int, unsigned int, void *(*func)(void *));
+int add_scheduler_conn(int, enum conn_type, pbs_net_t, unsigned int, unsigned int, void *(*func)(void *));
 int  client_to_svr(pbs_net_t, unsigned int, int, char *);
 void close_conn(int,int);
 pbs_net_t get_connectaddr(int,int);
@@ -230,7 +230,7 @@ struct netcounter
   int     counter;
   };
 
-int *netcounter_get();
+void netcounter_get(int netrates[]);
 #define MAXLISTENERS  3
 
 typedef struct listener_connection
