@@ -112,6 +112,9 @@
 #define ATTR_v "Variable_List"
 #define ATTR_A "Account_Name"
 #define ATTR_args "job_arguments"
+#define ATTR_reservation_id "reservation_id"
+#define ATTR_login_node_id "login_node_id"
+#define ATTR_login_prop "login_property"
 #define ATTR_M "Mail_Users"
 #define ATTR_N "Job_Name"
 #define ATTR_S "Shell_Path_List"
@@ -190,10 +193,8 @@
 #define ATTR_node_exclusive "node_exclusive"
 #define ATTR_exec_gpus   "exec_gpus"
 #define ATTR_J           "job_id"
-
-#ifdef USEJOBCREATE
 #define ATTR_pagg         "pagg_id"
-#endif /* USEJOBCREATE */
+
 
 #ifdef NVIDIA_GPUS
   /* JOB_ATR_gpu_flags */
@@ -319,11 +320,13 @@
 #ifdef MUNGE_AUTH
   #define ATTR_authusers       "authorized_users"
 #endif
-#define ATTR_minthreads        "min_threads"
-#define ATTR_maxthreads        "max_threads"
-#define ATTR_threadidleseconds "thread_idle_seconds"
+#define ATTR_minthreads          "min_threads"
+#define ATTR_maxthreads          "max_threads"
+#define ATTR_threadidleseconds   "thread_idle_seconds"
 #define ATTR_moabarraycompatible "moab_array_compatible"
-#define ATTR_nomailforce       "no_mail_force"
+#define ATTR_nomailforce             "no_mail_force"
+#define ATTR_interactivejobscanroam  "interactive_jobs_can_roam" 
+#define ATTR_crayenabled             "cray_enabled"
 /* additional node "attributes" names */
 
 #define ATTR_NODE_state            "state"
@@ -379,7 +382,6 @@
 #define ND_job_sharing   "job-sharing"
 #define ND_busy          "busy"
 #define ND_state_unknown "state-unknown"
-#define ND_timeshared    "time-shared"
 
 /* these are not state values, they describe states and are used in pbsnodes */
 /* active = job-exclusive, job-sharing, or busy */
@@ -390,7 +392,8 @@
 #define ND_up            "up"
 
 /* this specifies the type of node */
-#define ND_cluster  "cluster"
+#define ND_timeshared    "time-shared"
+#define ND_cluster       "cluster"
 
 /* queue disallowed types */
 #define Q_DT_batch              "batch"
