@@ -250,11 +250,11 @@ struct grpcache
 /*
  * Job attributes/resources are maintained in one of two ways.
  * Most of the attributes are maintained in a decoded or parsed form.
- * This allows quick access to the attribute and resource values
+ * This allows quick access to the pbs_attribute and resource values
  * when making decisions about the job (scheduling, routing, ...),
  * see ji_wattr[].
  *
- * Any attribute or resource which is not recognized on this server
+ * Any pbs_attribute or resource which is not recognized on this server
  * are kept in an "attrlist", a linked list of the "external"
  * form (attr_extern, see attribute.h).  These are maintained because
  * the job may be passed on to another server (route or qmove) that
@@ -348,7 +348,7 @@ enum job_atr
   JOB_ATR_checkpoint_time,   /* timestamp of start of last checkpoint */
   JOB_ATR_checkpoint_restart_status,   /* checkpoint restart status */
   JOB_ATR_restart_name,   /* name of checkpoint restart file */
-  JOB_ATR_fault_tolerant, /* indicates if a job should keep going if it looses a sister */
+  JOB_ATR_fault_tolerant, /* indicates if a job should keep going if it can't contact a sister while polling */
   JOB_ATR_comp_time,  /* time when job was completed */
   JOB_ATR_reported, /* tracks whether job has been reported to scheduler */
   JOB_ATR_jobtype,     /* opaque job type string */
@@ -360,14 +360,15 @@ enum job_atr
   JOB_ATR_node_exclusive,
   JOB_ATR_submit_host,  /* host which submitted the job */
   JOB_ATR_init_work_dir,    /* initial working directory */
-#ifdef USEJOBCREATE
   JOB_ATR_pagg_id,
-#endif /* USEJOBCREATE */
 #ifdef NVIDIA_GPUS
-  JOB_ATR_gpu_flags,		/* gpu flags - mode and reset flags */
+  JOB_ATR_gpu_flags,   /* gpu flags - mode and reset flags */
 #endif  /* NVIDIA_GPUS */
   JOB_ATR_job_id,
   JOB_ATR_arguments,
+  JOB_ATR_reservation_id,
+  JOB_ATR_login_node_id,
+  JOB_ATR_login_prop,
 #include "site_job_attr_enum.h"
 
   JOB_ATR_UNKN,  /* the special "unknown" type    */
@@ -578,6 +579,7 @@ struct job
   struct job_array *ji_arraystruct; /* pointer to job_array for this array */
   int               ji_is_array_template;    /* set to TRUE if this is a "template job" for a job array*/
   int               ji_have_nodes_request; /* set to TRUE if node spec uses keyword nodes */
+  int               ji_cold_restart; /* set to TRUE if this job has been loaded through a cold restart */
 #endif/* PBS_MOM */   /* END SERVER ONLY */
 
   pthread_mutex_t *ji_mutex;
@@ -655,7 +657,7 @@ struct job
    * Its presence is for rapid acces to the attributes.
    */
 
-  attribute ji_wattr[JOB_ATR_LAST]; /* decoded attributes  */
+  pbs_attribute ji_wattr[JOB_ATR_LAST]; /* decoded attributes  */
 
   int  maxAdoptedTaskId;  /* DJH 27 Feb 2002. Keep track of the task ids
                              the local mom allocates to adopted tasks; */
@@ -701,7 +703,6 @@ typedef struct job_recycler
   unsigned int     rc_next_id;
   struct all_jobs  rc_jobs;
   int              rc_iter;
-  unsigned int     rc_max_id;
   pthread_mutex_t *rc_mutex;
   } job_recycler;
 
@@ -928,7 +929,7 @@ typedef struct send_job_request
 
 #define JOB_FILE_COPY           ".JC"    /* tmp copy while updating */
 #define JOB_FILE_SUFFIX         ".JB"    /* job control file */
-#define JOB_FILE_BACKUP			".BK"	 /* job file backup */
+#define JOB_FILE_BACKUP         ".BK"    /* job file backup */
 #define JOB_SCRIPT_SUFFIX       ".SC"    /* job script file  */
 #define JOB_STDOUT_SUFFIX       ".OU"    /* job standard out */
 #define JOB_STDERR_SUFFIX       ".ER"    /* job standard error */
@@ -1044,7 +1045,7 @@ dir so that job can be restarted */
 
 extern void  add_dest(job *);
 extern void  depend_clrrdy(job *);
-extern int   depend_on_que(attribute *, void *, int);
+extern int   depend_on_que(pbs_attribute *, void *, int);
 extern int   depend_on_exec(job *);
 extern int   depend_on_term(job *);
 extern job  *find_job(char *);
@@ -1065,13 +1066,13 @@ extern int   job_save(job *, int, int);
 extern int   modify_job_attr(job *, svrattrl *, int, int *);
 extern char *prefix_std_file(job *, dynamic_string *, int);
 extern char *add_std_filename(job *, char *, int, dynamic_string *);
-extern int   set_jobexid(job *, attribute *, char *);
+extern int   set_jobexid(job *, pbs_attribute *, char *);
 extern int   site_check_user_map(job *, char *, char *, int);
 int  svr_dequejob(char *, int);
 extern int   svr_enquejob(job *, int, int);
 extern void  svr_evaljobstate(job *, int *, int *, int);
 extern void  svr_mailowner(job *, int, int, char *);
-extern void  set_resc_deflt(job *, attribute *, int);
+extern void  set_resc_deflt(job *, pbs_attribute *, int);
 extern void  set_statechar(job *);
 extern int   svr_setjobstate(job *, int, int, int);
 
