@@ -84,6 +84,7 @@
 */
 
 #include <pthread.h>
+#include <netinet/in.h> /* sockaddr_in */
 #include "resizable_array.h"
 #include "hash_table.h"
 #include "net_connect.h" /* pbs_net_t */
@@ -254,67 +255,10 @@ typedef struct node_iterator
   {
   int node_index;
   int numa_index;
+  int alps_index;
   } node_iterator;
 
 
-struct pbsnode
-  {
-  char                 *nd_name; /* node's host name */
-
-  struct pbssubn       *nd_psn; /* ptr to list of subnodes */
-
-  struct prop          *nd_first; /* first and last property */
-
-  struct prop          *nd_last;
-
-  struct prop          *nd_f_st;       /* first and last status */
-
-  struct prop          *nd_l_st;
-  u_long               *nd_addrs; /* IP addresses of host */
-
-  struct array_strings *nd_prop; /* array of properities */
-
-  struct array_strings *nd_status;
-  char                 *nd_note;  /* note set by administrator */
-  int                   nd_stream; /* stream to Mom on host */
-  enum psit             nd_flag;
-  unsigned short        nd_mom_port; /* For multi-mom-mode unique port value PBS_MOM_SERVICE_PORT*/
-  unsigned short        nd_mom_rm_port; /* For multi-mom-mode unique port value PBS_MANAGER_SERVICE_PORT */
-  short                 nd_nprops; /* number of properties */
-  short                 nd_nstatus;    /* number of status items */
-  short                 nd_nsn; /* number of VPs  */
-  short                 nd_nsnfree; /* number of VPs free */
-  short                 nd_nsnshared; /* number of VPs shared */
-  short                 nd_needed; /* number of VPs needed */
-  short                 nd_np_to_be_used; /* number of VPs marked for a job but not yet assigned */
-  unsigned short        nd_state;      /* node state (see INUSE_* #defines below) */
-  unsigned short        nd_ntype; /* node type */
-  short                 nd_order; /* order of user's request */
-  time_t                nd_warnbad;
-  time_t                nd_lastupdate; /* time of last update. */
-  unsigned short        nd_hierarchy_level;
-  unsigned char         nd_in_hierarchy; /* set to TRUE if in the hierarchy file */
-
-  short                 nd_ngpus;        /* number of gpus */ 
-  short                 nd_gpus_real;    /* gpus are real not virtual */ 
-  struct gpusubn       *nd_gpusn;        /* gpu subnodes */
-  short                 nd_ngpus_free;   /* number of free gpus */
-  short                 nd_ngpus_needed; /* number of gpus needed */
-  short                 nd_ngpus_to_be_used; /* number of gpus marked for a job but not yet assigned */
-  struct array_strings *nd_gpustatus;    /* string array of GPU status */
-  short                 nd_ngpustatus;    /* number of gpu status items */
-
-  struct pbsnode       *numa_parent;     /* pointer to the node holding this node, or NULL */
-  unsigned short        num_node_boards; /* number of numa nodes */
-  struct AvlNode       *node_boards; /* private tree of numa nodes */
-  char                 *numa_str; /* comma-delimited string of processor values */
-  char                 *gpu_str; /* comma-delimited string of the number of gpus for each nodeboard */
-
-  pthread_mutex_t      *nd_mutex; /* semaphore for accessing this node's data */
-  };
-
-
-
 
 typedef struct all_nodes
   {
@@ -326,6 +270,70 @@ typedef struct all_nodes
 
 
 
+
+struct pbsnode
+  {
+  char                 *nd_name;             /* node's host name */
+
+  struct pbssubn       *nd_psn;              /* ptr to list of subnodes */
+
+  struct prop          *nd_first;            /* first and last property */
+
+  struct prop          *nd_last;
+
+  struct prop          *nd_f_st;             /* first and last status */
+
+  struct prop          *nd_l_st;
+  u_long               *nd_addrs;            /* IP addresses of host */
+
+  struct array_strings *nd_prop;             /* array of properities */
+
+  struct array_strings *nd_status;
+  char                 *nd_note;             /* note set by administrator */
+  int                   nd_stream;           /* stream to Mom on host */
+  enum psit             nd_flag;
+  unsigned short        nd_mom_port;         /* For multi-mom-mode unique port value PBS_MOM_SERVICE_PORT*/
+  unsigned short        nd_mom_rm_port;      /* For multi-mom-mode unique port value PBS_MANAGER_SERVICE_PORT */
+  struct sockaddr_in    nd_sock_addr;        /* address information */
+  short                 nd_nprops;           /* number of properties */
+  short                 nd_nstatus;          /* number of status items */
+  short                 nd_nsn;              /* number of VPs  */
+  short                 nd_nsnfree;          /* number of VPs free */
+  short                 nd_nsnshared;        /* number of VPs shared */
+  short                 nd_needed;           /* number of VPs needed */
+  short                 nd_np_to_be_used;    /* number of VPs marked for a job but not yet assigned */
+  unsigned short        nd_state;            /* node state (see INUSE_* #defines below) */
+  unsigned short        nd_ntype;            /* node type */
+  short                 nd_order;            /* order of user's request */
+  time_t                nd_warnbad;
+  time_t                nd_lastupdate;       /* time of last update. */
+  unsigned short        nd_hierarchy_level;
+  unsigned char         nd_in_hierarchy;     /* set to TRUE if in the hierarchy file */
+
+  short                 nd_ngpus;            /* number of gpus */ 
+  short                 nd_gpus_real;        /* gpus are real not virtual */ 
+  struct gpusubn       *nd_gpusn;            /* gpu subnodes */
+  short                 nd_ngpus_free;       /* number of free gpus */
+  short                 nd_ngpus_needed;     /* number of gpus needed */
+  short                 nd_ngpus_to_be_used; /* number of gpus marked for a job but not yet assigned */
+  struct array_strings *nd_gpustatus;        /* string array of GPU status */
+  short                 nd_ngpustatus;       /* number of gpu status items */
+
+  struct pbsnode       *parent;              /* pointer to the node holding this node, or NULL */
+  unsigned short        num_node_boards;     /* number of numa nodes */
+  struct AvlNode       *node_boards;         /* private tree of numa nodes */
+  char                 *numa_str;            /* comma-delimited string of processor values */
+  char                 *gpu_str;             /* comma-delimited string of the number of gpus for each nodeboard */
+  
+  unsigned char         nd_is_alps_reporter;
+  unsigned char         nd_is_alps_login;
+  all_nodes             alps_subnodes;
+
+  pthread_mutex_t      *nd_mutex;            /* semaphore for accessing this node's data */
+  };
+
+
+
 #define INITIAL_NODE_SIZE  20
 
 
@@ -456,7 +464,7 @@ int tlist(tree *, char *, int);
 /*
  * Although at the present time a struct pbssnode doesn't have an array of
  * attributes associated with it, it is convenient for the server's req_manager
- * function to use as much of the existing attribute processing functions and
+ * function to use as much of the existing pbs_attribute processing functions and
  * philosophy as practical to carry out any node modification requests sent via
  * "qmgr".   For this reason, and for the fact that some day struct pbssnodes
  * may be redefined to possess attributes, it is convenient to pretend that
@@ -510,16 +518,13 @@ extern int    svr_clnodes;  /* number of cluster nodes */
 
 extern int    MultiMomMode; /* moms configured for multiple moms per machine */
 
-/*extern struct tree_t  *ipaddrs;*/
-
-/* extern struct tree_t  *streams; */
-
 extern int update_nodes_file(struct pbsnode *);
 
 extern void bad_node_warning(pbs_net_t, struct pbsnode *);
 
 struct pbsnode  *find_nodebyname(char *);
 int              create_partial_pbs_node(char *, unsigned long, int);
+struct pbssubn  *create_subnode(struct pbsnode *pnode);
 
 #ifdef BATCH_REQUEST_H 
 void             initialize_pbssubn(struct pbsnode *, struct pbssubn *, struct prop *);
@@ -530,7 +535,7 @@ struct pbssubn  *find_subnodebyname(char *);
 
 struct pbsnode  *find_nodebynameandaltname(char *, char *);
 void             free_prop_list(struct prop*);
-void             free_prop_attr(attribute*);
+void             free_prop_attr(pbs_attribute*);
 void             recompute_ntype_cnts();
 int              create_pbs_node(char *, svrattrl *, int, int *);
 int              mgr_set_node_attr(struct pbsnode *, attribute_def *, int, svrattrl *, int, int *, void *, int);
@@ -542,4 +547,7 @@ void             reinitialize_node_iterator(node_iterator *);
 struct prop     *init_prop(char *pname);
 #endif /* BATCH_REQUEST_H */
 
+int              initialize_pbsnode(struct pbsnode *, char *pname, u_long *pul, int ntype);
+int              hasprop(struct pbsnode *pnode, struct prop *props);
+
 #endif /* PBS_NODES_H */ 
