@@ -176,3 +176,5 @@ ATTR_maxthreads,
 ATTR_threadidleseconds,
 ATTR_moabarraycompatible,
 ATTR_nomailforce,
+ATTR_crayenabled,
+ATTR_interactivejobscanroam,
