@@ -102,9 +102,8 @@
 #define NO_ATTR_DATA    1
 #define ATTR_NOT_FOUND  -2
 /* default multi threading options */
-#define DEFAULT_MAX_THREADS    10
 #define DEFAULT_MIN_THREADS    10
-#define DEFAULT_THREAD_IDLE    -1
+#define DEFAULT_THREAD_IDLE    300
 
 enum srv_atr
   {
@@ -199,6 +198,8 @@ enum srv_atr
   SRV_ATR_threadidleseconds,
   SRV_ATR_MoabArrayCompatible,
   SRV_ATR_NoMailForce,
+  SRV_ATR_CrayEnabled,
+  SRV_ATR_InteractiveJobsCanRoam,
 
 #include "site_svr_attr_enum.h"
   /* This must be last */
@@ -228,7 +229,7 @@ struct server
   int              sv_jobstates[PBS_NUMJOBSTATE];  /* # of jobs per state */
   char             sv_jobstbuf[100];
 
-  attribute        sv_attr[SRV_ATR_LAST]; /* the server attributes      */
+  pbs_attribute    sv_attr[SRV_ATR_LAST]; /* the server attributes      */
 
   int              sv_trackmodifed; /* 1 if tracking list modified     */
   int              sv_tracksize;  /* total number of sv_track entries */
@@ -269,6 +270,9 @@ extern struct server server;
 extern int svr_recov_xml(char *, int);
 extern int svr_save(struct server *, int);
 
+int lock_sv_qs_mutex(pthread_mutex_t *sv_qs_mutex, const char *msg_string);
+int unlock_sv_qs_mutex(pthread_mutex_t *sv_qs_mutex, const char *msg_string);
+
 
 /*
 ** Macros for fast min/max.
