@@ -78,7 +78,7 @@
 */
 
 /*
- * @(#) $Id: tracking.h 2367 2008-09-03 17:17:34Z josh $
+ * @(#) $Id: tracking.h 5223 2011-10-18 20:59:09Z jrosenquist $
  */
 
 /*
@@ -98,6 +98,10 @@
 #define PBS_TRACK_MINSIZE   100 /* mininum size of buffer in records */
 #define PBS_SAVE_TRACK_TM   300 /* time interval between saves of track data */
 
+#include <time.h> /* time_t */
+#include "pbs_ifl.h" /* PBS_MAXSVRJOBID, PBS_MAXDEST */
+#include "work_task.h" /* work_task */
+
 struct tracking
   {
   time_t  tk_mtime; /* time this entry modified */
@@ -107,4 +111,4 @@ struct tracking
   char     tk_state;
   };
 
-extern void track_save A_((struct work_task *));
+extern void track_save(struct work_task *);
