@@ -133,7 +133,7 @@
 
 int diswl_(
     
-  int               stream,
+  struct tcp_chan *chan,
   dis_long_double_t value,
   unsigned          ndigs)
 
@@ -147,24 +147,18 @@ int diswl_(
   char  *ocp;
   dis_long_double_t ldval;
   char  scratch[DIS_BUFSIZ+1];
-  int (*dis_puts)(int stream, const char *string, size_t count);
-  int (*disw_commit)(int stream, int commit);
 
   assert(ndigs > 0 && ndigs <= LDBL_DIG);
-  assert(stream >= 0);
   
-  dis_puts = tcp_puts;
-  disw_commit = tcp_wcommit;
-
   memset(scratch, 0, DIS_BUFSIZ+1);
   /* Make zero a special case.  If we don't it will blow exponent  */
   /* calculation.        */
 
   if (value == 0.0L)
     {
-    retval = (*dis_puts)(stream, "+0+0", 4) < 0 ?
+    retval = tcp_puts(chan, "+0+0", 4) < 0 ?
              DIS_PROTO : DIS_SUCCESS;
-    return (((*disw_commit)(stream, retval == DIS_SUCCESS) < 0) ?
+    return ((tcp_wcommit(chan, retval == DIS_SUCCESS) < 0) ?
             DIS_NOCOMMIT : retval);
     }
 
@@ -253,13 +247,13 @@ int diswl_(
     cp = discui_(cp, ndigs, &ndigs);
 
   /* The complete coefficient integer is done.  Put it out.  */
-  retval = (*dis_puts)(stream, cp, (size_t)(ocp - cp)) < 0 ?
+  retval = tcp_puts(chan, cp, (size_t)(ocp - cp)) < 0 ?
            DIS_PROTO : DIS_SUCCESS;
 
   /* If that worked, follow with the exponent, commit, and return. */
   if (retval == DIS_SUCCESS)
-    return (diswsi(stream, expon));
+    return (diswsi(chan, expon));
 
   /* If coefficient didn't work, negative commit and return the error. */
-  return (((*disw_commit)(stream, FALSE) < 0)  ? DIS_NOCOMMIT : retval);
+  return ((tcp_wcommit(chan, FALSE) < 0)  ? DIS_NOCOMMIT : retval);
   }
