@@ -206,7 +206,7 @@ char *pbs_get_server_list(void)
       if (fgets(tmp, sizeof(tmp), fd) == NULL)
         {
         fclose(fd);
-
+        fprintf(stderr, "\n%s: server_name file\n(%s)\nis EMPTY!!!\n\n", __func__, pbs_destn_file);
         return(server_list);
         }
 
@@ -860,7 +860,6 @@ int pbs_original_connect(
   if (use_unixsock)
     {
     connection[out].ch_socket = socket(AF_UNIX, SOCK_STREAM, 0);
-    DIS_tcp_init(connection[out].ch_socket);
 
     if (connection[out].ch_socket < 0)
       {
@@ -976,7 +975,7 @@ int pbs_original_connect(
                 if_name, errno);
         if (if_name)
           free(if_name);
-        return(PBSE_SYSTEM);
+        return(PBSE_SYSTEM * -1);
         }
       }
 
@@ -1198,7 +1197,29 @@ int pbs_disconnect(
 
 
 
-
+void print_server_port_to_stderr(
+    char *s_name)
+  {
+  int rc = PBSE_NONE;
+  char *s_addr = NULL;
+  unsigned short af_family;
+  struct in_addr  hostaddr;
+  char *ip_addr = NULL;
+  int s_len = 0;
+  if ((rc = get_hostaddr_hostent_af(&rc, s_name, &af_family, &s_addr, &s_len)) == PBSE_NONE)
+    {   
+    memcpy((void *)&hostaddr, (void *)s_addr, s_len);
+    ip_addr = inet_ntoa(hostaddr);
+    fprintf(stderr, "Error communicating with %s(%s)\n", s_name, ip_addr);
+    }
+  else  
+    {
+    fprintf(stderr, "Can not resolve name for server %s. (rc = %d - %s)\n",
+        s_name,
+        rc,  
+        pbs_strerror(rc));
+    }        
+  }
 
 /**
  * This is a new version of this function that allows
@@ -1268,6 +1289,8 @@ int pbs_connect(
 
         return(connect);  /* Success, we have a connection, return it. */
         }
+      else
+        print_server_port_to_stderr(current_name);
       }
     }
 
