@@ -19,6 +19,7 @@
 #include <poll.h> /* poll functionality */
 #include "../lib/Liblog/pbs_log.h" /* log_err */
 #include "log.h" /* LOCAL_LOG_BUF_SIZE */
+#include "net_cache.h"
 
 #include "pbs_error.h" /* torque error codes */
 
@@ -432,7 +433,7 @@ int socket_wait_for_read(
   int           countdown;
   struct pollfd pfd;
 
-  countdown = pbs_tcp_timeout*10;
+  countdown = pbs_tcp_timeout * 10;
 
   pfd.fd = socket;
   pfd.events = POLLIN | POLLHUP; /* | POLLRDNORM; */
@@ -749,16 +750,27 @@ int socket_close(int socket)
 
 
 
-int get_addr_info(char *name, struct sockaddr_in *sa_info, int retry)
-  {
-  int rc = PBSE_NONE;
-  int cntr = 0;
-  struct addrinfo *addr_info;
-  struct addrinfo hints;
-  char log_buf[LOCAL_LOG_BUF_SIZE+1];
-  struct timeval start_time;
-  struct timeval end_time;
+int get_addr_info(
+    
+  char               *name,
+  struct sockaddr_in *sa_info,
+  int                 retry)
 
+  {
+  int                 rc = PBSE_NONE;
+  int                 cntr = 0;
+  struct addrinfo    *addr_info;
+  struct addrinfo     hints;
+  struct timeval      start_time;
+  struct timeval      end_time;
+  struct sockaddr_in *cached_sai;
+
+  /* retrieve from cache if possible */
+  if ((cached_sai = get_cached_addrinfo(name)) != NULL)
+    {
+    memcpy(sa_info, cached_sai, sizeof(struct sockaddr_in));
+    return(PBSE_NONE);
+    }
 
   memset(&hints, 0, sizeof(struct addrinfo));
   hints.ai_socktype = SOCK_STREAM;
@@ -767,34 +779,29 @@ int get_addr_info(char *name, struct sockaddr_in *sa_info, int retry)
   while (cntr < retry)
     {
     gettimeofday(&start_time, 0);
-    snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "%s call #%d", name, cntr);
-    /*log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_REQUEST, __func__, log_buf);*/
 
     if ((rc = getaddrinfo(name, NULL, &hints, &addr_info)) != 0)
       {
       gettimeofday(&end_time, 0);
-      snprintf(log_buf, LOCAL_LOG_BUF_SIZE,
-          "Can't get address information for [%s] - (%d-%s) [retry %d]time{%d}",
-          name, rc, gai_strerror(rc), cntr, (int)(end_time.tv_sec-start_time.tv_sec));
+
       rc = PBSE_BADHOST;
-      /*log_err(rc, __func__, log_buf);*/
       }
     else
       {
       sa_info->sin_addr = ((struct sockaddr_in *)addr_info->ai_addr)->sin_addr;
       sa_info->sin_family = addr_info->ai_family;
+      insert_addr_name_info(name, addr_info->ai_canonname, sa_info);
       freeaddrinfo(addr_info);
       gettimeofday(&end_time, 0);
-/*      snprintf(log_buf, LOCAL_LOG_BUF_SIZE,
-          "Success resolving %s. Elapsed call time {%d}",
-          name, (int)(end_time.tv_sec-start_time.tv_sec));
-      log_err(-1, __func__, log_buf);
-      */
+
       rc = PBSE_NONE;
+
       break;
       }
+
     cntr++;
     }
-  return rc;
+
+  return(rc);
   }
 
