@@ -115,11 +115,13 @@
 #include "portability.h"
 #include "server_limits.h"
 #include "net_connect.h"
+#include "net_cache.h"
 #include "log.h"
 #include "../Liblog/pbs_log.h" /* log_err */
 #include "../Liblog/log_event.h" /* log_event */
 #include "../Libifl/lib_ifl.h" /* DIS_* */
 #include "pbs_error.h" /* PBSE_NONE */
+#include "dis.h"
 
 char   local_host_name[PBS_MAXHOSTNAME + 1];
 size_t local_host_name_len = PBS_MAXHOSTNAME;
@@ -132,6 +134,9 @@ extern char local_host_name[];
 void initialize_connections_table();
 
 extern time_t time(time_t *);
+struct in_addr   net_serveraddr;
+char            *net_server_name = NULL;
+char             pbs_server_name[PBS_MAXSERVERNAME + 1];
 
 /* Global Data (I wish I could make it private to the library, sigh, but
  * C don't support that scope of control.)
@@ -154,7 +159,6 @@ pthread_mutex_t *global_sock_read_mutex = NULL;
 
 void *(*read_func[2])(void *);
 
-pthread_mutex_t *netrates_mutex = NULL;
 pthread_mutex_t *nc_list_mutex  = NULL;
 
 pbs_net_t        pbs_server_addr;
@@ -169,32 +173,28 @@ static struct netcounter nc_list[60];
 void netcounter_incr(void)
 
   {
-  time_t now, lastmin;
-  int i;
-
-  now = time(NULL);
-  lastmin = now - 60;
+  time_t now;
+  int    i;
 
   pthread_mutex_lock(nc_list_mutex);
 
+  now = time(NULL);
+
   if (nc_list[0].time == now)
     {
     nc_list[0].counter++;
     }
   else
     {
-    memmove(&nc_list[1], &nc_list[0], sizeof(struct netcounter)*59);
+    memmove(&nc_list[1], &nc_list[0], sizeof(struct netcounter) * 59);
 
     nc_list[0].time = now;
     nc_list[0].counter = 1;
 
-    for (i = 0;i < 60;i++)
+    for (i = 1; i < 60; i++)
       {
-      if (nc_list[i].time < lastmin)
-        {
-        nc_list[i].time = 0;
-        nc_list[i].counter = 0;
-        }
+      nc_list[i].time = 0;
+      nc_list[i].counter = 0;
       }
     }
 
@@ -214,20 +214,14 @@ int get_num_connections()
   }
 
 
-int *netcounter_get(void)
+void netcounter_get(
+    
+  int netrates[])
 
   {
-  static int netrates[3];
   int netsums[3] = {0, 0, 0};
   int i;
 
-  if (netrates_mutex == NULL)
-    {
-    netrates_mutex = calloc(1, sizeof(pthread_mutex_t));
-    pthread_mutex_init(netrates_mutex,NULL);
-    }
-
-  pthread_mutex_lock(netrates_mutex);
   pthread_mutex_lock(nc_list_mutex);
 
   for (i = 0;i < 5;i++)
@@ -263,8 +257,7 @@ int *netcounter_get(void)
     netrates[2] = 0;
     }
 
-  return netrates;
-  }
+  } /* END netcounter_get() */
 
 /**
  * init_network - initialize the network interface
@@ -280,9 +273,9 @@ int init_network(
   void        *(*readfunc)(void *))
 
   {
-  int   i;
+  int         i;
   static int  initialized = 0;
-  int    sock;
+  int         sock;
 
   int MaxNumDescriptors = 0;
 
@@ -307,13 +300,39 @@ int init_network(
     {
     initialize_connections_table();
 
+    if (net_server_name == NULL)
+      {
+      /* cache local server addr info */
+      struct addrinfo        *addr_info = NULL;
+      char                    namebuf[MAXLINE*2];
+
+      if (getaddrinfo(pbs_server_name, NULL, NULL, &addr_info) == 0)
+        {
+        if (getnameinfo(addr_info->ai_addr, addr_info->ai_addrlen, namebuf, sizeof(namebuf), NULL, 0, 0) == 0)
+          {
+          net_server_name = strdup(namebuf);
+          }
+
+        net_serveraddr = ((struct sockaddr_in *)addr_info->ai_addr)->sin_addr; 
+        }
+
+      if (addr_info != NULL)
+        freeaddrinfo(addr_info);
+
+      if (net_server_name == NULL)
+        net_server_name = strdup(inet_ntoa(net_serveraddr));
+      }
+
+    disiui_();
+
     for (i = 0;i < PBS_NET_MAX_CONNECTIONS;i++)
       {
       svr_conn[i].cn_mutex = (pthread_mutex_t *)calloc(1, sizeof(pthread_mutex_t));
       pthread_mutex_init(svr_conn[i].cn_mutex,NULL);
-
       pthread_mutex_lock(svr_conn[i].cn_mutex);
+      
       svr_conn[i].cn_active = Idle;
+      
       pthread_mutex_unlock(svr_conn[i].cn_mutex);
       }
     
@@ -439,8 +458,7 @@ int init_network(
 
 #endif  /* END ENABLE_UNIX_SOCKETS */
 
-    /* allocate a minute's worth of counter structs */
-
+  /* allocate a minute's worth of counter structs */
   if (nc_list_mutex == NULL)
     {
     nc_list_mutex = calloc(1, sizeof(pthread_mutex_t));
@@ -448,22 +466,83 @@ int init_network(
 
     pthread_mutex_lock(nc_list_mutex);
 
-    for (i = 0;i < 60;i++)
-      {
-      nc_list[i].time = 0;
-      nc_list[i].counter = 0;
-      }
+    memset(nc_list, 0, sizeof(nc_list));
+    nc_list[0].time = time(NULL);
 
     pthread_mutex_unlock(nc_list_mutex);
     }
 
-  return(0);
+  return(PBSE_NONE);
   }  /* END init_network() */
 
 
 
 
 
+/**
+ * check_network_port - initialize the network interface
+ * allocate a socket and bind it to the service port,
+ */
+
+int check_network_port(
+
+  unsigned int  port)
+
+  {
+  int   i;
+  int    sock;
+
+  struct sockaddr_in socname;
+
+  memset(&socname, 0, sizeof(socname));
+
+  if (port == 0)
+    {
+    return(-1);
+    }
+  else
+    {
+    sock = socket(AF_INET, SOCK_STREAM, 0);
+
+    if (sock < 0)
+      {
+      /* FAILURE */
+
+      return(-1);
+      }
+
+    i = 1;
+
+    setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
+
+    /* name that socket "in three notes" */
+
+    socname.sin_port = htons((unsigned short)port);
+
+    socname.sin_addr.s_addr = INADDR_ANY;
+
+    socname.sin_family = AF_INET;
+
+    if (bind(sock, (struct sockaddr *)&socname, sizeof(socname)) < 0)
+      {
+      /* FAILURE */
+
+      close(sock);
+
+      return(-1);
+      }
+
+    close (sock);
+
+    } /* END if (port != 0) */
+
+  return(0);
+  }  /* END check_network_port() */
+
+
+
+
+
 /*
  * wait_request - wait for a request (socket with data to read)
  * This routine does a select on the readset of sockets,
@@ -539,6 +618,7 @@ int wait_request(
           continue;
 
         /* clean up SdList and bad sd... */
+
         globalset_del_sock(i);
         } /* END for each socket in global read set */
 
@@ -582,6 +662,7 @@ int wait_request(
       else
         {
         pthread_mutex_unlock(svr_conn[i].cn_mutex);
+
         globalset_del_sock(i);
         close_conn(i, FALSE);
 
@@ -738,38 +819,49 @@ void *accept_conn(
   }  /* END accept_conn() */
 
 
+
+
 void globalset_add_sock(
-    int sock)
+
+  int sock)
+
   {
   pthread_mutex_lock(global_sock_read_mutex);
   FD_SET(sock, GlobalSocketReadSet);
   pthread_mutex_unlock(global_sock_read_mutex);
-  }
+  } /* END globalset_add_sock() */
 
+
+
+ 
 void globalset_del_sock(
-    int sock)
+
+  int sock)
+
   {
   pthread_mutex_lock(global_sock_read_mutex);
   FD_CLR(sock, GlobalSocketReadSet);
   pthread_mutex_unlock(global_sock_read_mutex);
-  }
+  } /* END globalset_del_sock() */
+
 
 
 /*
- * add_conn - add a connection to the svr_conn array.
+ * add_connection - add a connection to the svr_conn array.
  * The params addr and port are in host order.
  *
  * NOTE:  This routine cannot fail.
  */
 
-int add_conn(
+int add_connection(
 
   int            sock,    /* socket associated with connection */
   enum conn_type type,    /* type of connection */
   pbs_net_t      addr,    /* IP address of connected host */
   unsigned int   port,    /* port number (host order) on connected host */
   unsigned int   socktype, /* inet or unix */
-  void *(*func)(void *))  /* function to invoke on data rdy to read */
+  void *(*func)(void *),  /* function to invoke on data rdy to read */
+  int            add_wait_request) /* True to add into global select set */
 
   {
   if (num_connections_mutex == NULL)
@@ -782,7 +874,15 @@ int add_conn(
   num_connections++;
   pthread_mutex_unlock(num_connections_mutex);
 
-  globalset_add_sock(sock);
+  if (add_wait_request)
+    {
+    globalset_add_sock(sock);
+    }
+  else
+    {
+    /* just to make sure it's cleared */
+    globalset_del_sock(sock);
+    }
 
   pthread_mutex_lock(svr_conn[sock].cn_mutex);
 
@@ -824,10 +924,58 @@ int add_conn(
   pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
   return PBSE_NONE;
+  }  /* END add_connection() */
+
+
+
+/*
+ * add_conn - add a connection to the svr_conn array.
+ * The params addr and port are in host order.
+ *
+ * NOTE:  This routine cannot fail.
+ */
+
+int add_conn(
+
+  int            sock,    /* socket associated with connection */
+  enum conn_type type,    /* type of connection */
+  pbs_net_t      addr,    /* IP address of connected host */
+  unsigned int   port,    /* port number (host order) on connected host */
+  unsigned int   socktype, /* inet or unix */
+  void *(*func)(void *))  /* function to invoke on data rdy to read */
+
+  {
+  return(add_connection(sock, type, addr, port, socktype, func, TRUE));
   }  /* END add_conn() */
 
 
 
+/*
+ * add_scheduler_conn - add a connection to the svr_conn array.
+ * The params addr and port are in host order.
+ * This version is specific to connections the server makes to the scheduler.
+ * These connections must be outside the control of wait_request().
+ *
+ * NOTE:  This routine cannot fail.
+ */
+
+int add_scheduler_conn(
+
+  int            sock,    /* socket associated with connection */
+  enum conn_type type,    /* type of connection */
+  pbs_net_t      addr,    /* IP address of connected host */
+  unsigned int   port,    /* port number (host order) on connected host */
+  unsigned int   socktype, /* inet or unix */
+  void *(*func)(void *))  /* function to invoke on data rdy to read */
+
+  {
+  return(add_connection(sock, type, addr, port, socktype, func, FALSE));
+  }  /* END add_scheduler_conn() */
+
+
+
+
+
 /*
  * close_conn - close a network connection
  * does physical close, also marks the connection table
@@ -839,12 +987,14 @@ void close_conn(
   int has_mutex) /* I */
 
   {
-  int rc;
   char log_message[LOG_BUF_SIZE+1];
-  if ((sd < 0) || (max_connection <= sd))
+  
+  if ((sd < 0) ||
+      (max_connection <= sd))
     {
     snprintf(log_message, LOG_BUF_SIZE, "sd is invalid %d!!!", sd);
     log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_NODE,__func__,log_message);
+
     return;
     }
 
@@ -853,9 +1003,9 @@ void close_conn(
 
   if (svr_conn[sd].cn_active == Idle)
     {
-    /* apparently we did not add the socket to the connection table */
-    if (has_mutex == FALSE) 
+    if (has_mutex == FALSE)
       pthread_mutex_unlock(svr_conn[sd].cn_mutex);
+
     return;
     }
 
@@ -864,8 +1014,9 @@ void close_conn(
   if (svr_conn[sd].cn_oncl != 0)
     {
     snprintf(log_message, LOG_BUF_SIZE, "Connection %d - func %lx",
-        sd, (unsigned long)svr_conn[sd].cn_oncl);
-    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_SERVER,__func__,log_message);
+      sd, (unsigned long)svr_conn[sd].cn_oncl);
+    log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_NODE, __func__, log_message);
+
     svr_conn[sd].cn_oncl(sd);
     }
 
@@ -879,13 +1030,14 @@ void close_conn(
     globalset_del_sock(sd);
     }
 
+  close(sd);
+
   svr_conn[sd].cn_addr = 0;
   svr_conn[sd].cn_handle = -1;
   svr_conn[sd].cn_active = Idle;
   svr_conn[sd].cn_func = (void *(*)())0;
   svr_conn[sd].cn_authen = 0;
   svr_conn[sd].cn_stay_open = FALSE;
-  rc = close(sd);
     
   if (has_mutex == FALSE)
     pthread_mutex_unlock(svr_conn[sd].cn_mutex);
@@ -960,32 +1112,42 @@ pbs_net_t get_connectaddr(
     pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
   return(tmp);
-  }
+  } /* END get_connectaddr() */
+
+
+
+
+void set_localhost_name(
+    
+  char   *localhost_name,
+  size_t  len)
 
-void set_localhost_name(char *localhost_name, size_t len)
   {
   struct sockaddr sa;
   int             rc;
-
+ 
   memset(&sa, 0, sizeof(struct sockaddr));
+
   sa.sa_family = AF_INET;
   sa.sa_data[2] = 0x7F;
   sa.sa_data[5] = 1;
-  rc = getnameinfo(&sa, sizeof(sa), local_host_name, local_host_name_len, NULL, 0, 0);
-  if(rc != 0)
+
+  if ((rc = getnameinfo(&sa, sizeof(sa), local_host_name, local_host_name_len, NULL, 0, 0)) != 0)
     {
     strcpy(local_host_name, "localhost");
     strncpy(localhost_name, "localhost", len);
     }
   else
     strncpy(localhost_name, local_host_name, len);
-  }
+  } /* END set_localhost_name() */
+
+
 
-                    
 
 /*
  * get_connecthost - return name of host connected via the socket
  */
+
 int get_connecthost(
 
   int   sock,     /* I */
@@ -997,23 +1159,10 @@ int get_connecthost(
 
   struct sockaddr        *addr_info_ptr;
   struct sockaddr_in      addr_in;
-  static struct in_addr   serveraddr;
-  static char            *server_name = NULL;
+  char                   *name;
 
   addr_in.sin_family = AF_INET;
   addr_in.sin_port = 0;
-  if ((server_name == NULL) && (pbs_server_addr != 0))
-    {
-    /* cache local server addr info */
-    serveraddr.s_addr = htonl(pbs_server_addr);
-    addr_in.sin_addr = serveraddr;
-    addr_info_ptr = (struct sockaddr *)&addr_in;
-
-    if (getnameinfo(addr_info_ptr, sizeof(addr_in), namebuf, size, NULL, 0, 0) == 0)
-      server_name = strdup(namebuf);
-    else
-      server_name = strdup(inet_ntoa(serveraddr));
-    }
 
   size--;
 
@@ -1021,17 +1170,23 @@ int get_connecthost(
   addr_in.sin_addr = addr;
   addr_info_ptr = (struct sockaddr *)&addr_in;
 
-  if ((server_name != NULL) && (svr_conn[sock].cn_socktype & PBS_SOCK_UNIX))
+  if ((net_server_name != NULL) &&
+      (svr_conn[sock].cn_socktype & PBS_SOCK_UNIX))
     {
     /* lookup request is for local server */
 
-    strcpy(namebuf, server_name);
+    strcpy(namebuf, net_server_name);
     }
-  else if ((server_name != NULL) && (addr.s_addr == serveraddr.s_addr))
+  else if ((net_server_name != NULL) &&
+           (addr.s_addr == net_serveraddr.s_addr))
     {
     /* lookup request is for local server */
 
-    snprintf(namebuf, size, "%s", server_name);
+    snprintf(namebuf, size, "%s", net_server_name);
+    }
+  else if ((name = get_cached_nameinfo(&addr_in)) != NULL)
+    {
+    snprintf(namebuf, size, "%s", name);
     }
   else if (getnameinfo(addr_info_ptr, sizeof(addr_in), namebuf, size, NULL, 0, 0) != 0)
     {
@@ -1049,7 +1204,9 @@ int get_connecthost(
       }
     }
 
-  return(0);
+  /* SUCCESS */
+
+  return(PBSE_NONE);
   }  /* END get_connecthost() */
 
 
@@ -1058,7 +1215,9 @@ int get_connecthost(
 ** a staticly allocated string.
 */
 char *netaddr_pbs_net_t(
+
   pbs_net_t ipadd)
+
   {
   char  out[80];
   char *return_value;
