@@ -655,14 +655,12 @@ void free_hash(
 
 
 
-/* this will be replaced with a real hash function soon, but this works for testing */
 int get_hash(
     
   hash_table_t *ht,
-  char         *key)
+  void         *key)
 
   {
-  /* since key is a job name, just get the numbers from the front */
   size_t   len = strlen(key);
   uint32_t hash;
   uint32_t mask = 0;
@@ -689,7 +687,7 @@ void add_to_bucket(
 
   bucket **buckets,
   int      index,
-  char    *key,
+  void    *key,
   int      value)
 
   {
@@ -724,7 +722,7 @@ int add_hash(
     
   hash_table_t *ht,
   int           value,
-  char         *key)
+  void         *key)
 
   {
   int index;
@@ -748,7 +746,7 @@ int add_hash(
 
       while (b != NULL)
         {
-        new_index = get_hash(ht,b->key);
+        new_index = get_hash(ht, b->key);
 
         add_to_bucket(tmp,new_index,b->key,b->value);
 
@@ -763,7 +761,7 @@ int add_hash(
     ht->buckets = tmp;
     } /* END if (need to rehash) */
 
-  index = get_hash(ht,key);
+  index = get_hash(ht, key);
 
   add_to_bucket(ht->buckets,index,key,value);
 
@@ -783,7 +781,7 @@ int remove_hash(
 
   {
   int     rc = PBSE_NONE;
-  int     index = get_hash(ht,key);
+  int     index = get_hash(ht, key);
   bucket *b = ht->buckets[index];
   bucket *prev = NULL;
 
@@ -837,15 +835,15 @@ int remove_hash(
 int get_value_hash(
     
   hash_table_t *ht,
-  char         *key)
+  void         *key)
 
   {
   int     value = -1;
-  bucket *b = ht->buckets[get_hash(ht,key)];
+  bucket *b = ht->buckets[get_hash(ht, key)];
 
   while (b != NULL)
     {
-    if (!strcmp(b->key,key))
+    if (!strcmp(b->key, key))
       {
       value = b->value;
       break;
@@ -869,7 +867,7 @@ void change_value_hash(
   int           new_value)
 
   {
-  int index = get_hash(ht,key);
+  int index = get_hash(ht, key);
   bucket *b = ht->buckets[index];
 
   while (b != NULL)
