@@ -108,6 +108,7 @@
 #include "mom_mach.h"
 #include "mom_func.h"
 #include "resmon.h"
+#include "utils.h"
 #include "../rm_dep.h"
 
 #include <windows.h>
@@ -576,7 +577,7 @@ dep_main_loop_cycle(void)
 
 
 
-static int getsize(
+static int mm_getsize(
 
   resource *pres,  /* I */
   unsigned long *ret)   /* O */
@@ -609,7 +610,7 @@ static int getsize(
   *ret = (value << pres->rs_value.at_val.at_size.atsv_shift);
 
   return(PBSE_NONE);
-  }  /* END getsize() */
+  }  /* END mm_getsize() */
 
 
 
@@ -623,7 +624,7 @@ static int getsize(
  * decoded value of time in seconds in the unsigned long integer.
  */
 
-static int gettime(
+static int mm_gettime(
 
   resource      *pres,
   unsigned long *ret)
@@ -1070,11 +1071,11 @@ int mom_set_limits(
         {
         /* cpu time - check, if less than pcput use it */
 
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
 
         if (retval != PBSE_NONE)
           {
-          sprintf(log_buffer, "cput gettime failed in %s",id);
+          sprintf(log_buffer, "cput mm_gettime failed in %s",id);
 
           return(error(pname, retval));
           }
@@ -1088,11 +1089,11 @@ int mom_set_limits(
           {
           /* process cpu time - set */
 
-          retval = gettime(pres, &value);
+          retval = mm_gettime(pres, &value);
 
           if (retval != PBSE_NONE)
             {
-            sprintf(log_buffer, "pcput gettime failed in %s",id);
+            sprintf(log_buffer, "pcput mm_gettime failed in %s",id);
 
             return(error(pname, retval));
             }
@@ -1131,11 +1132,11 @@ int mom_set_limits(
 
       if (set_mode == SET_LIMIT_SET)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           {
-          sprintf(log_buffer, "getsize() failed for file in %s",
+          sprintf(log_buffer, "mm_getsize() failed for file in %s",
                   id);
 
           return(error(pname, retval));
@@ -1181,11 +1182,11 @@ int mom_set_limits(
         {
         /* check */
 
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           {
-          sprintf(log_buffer, "getsize() failed for vmem in %s",id);
+          sprintf(log_buffer, "mm_getsize() failed for vmem in %s",id);
 
           return(error(pname, retval));
           }
@@ -1202,11 +1203,11 @@ int mom_set_limits(
 
         if (set_mode == SET_LIMIT_SET)
           {
-          retval = getsize(pres, &value);
+          retval = mm_getsize(pres, &value);
 
           if (retval != PBSE_NONE)
             {
-            sprintf(log_buffer, "getsize() failed for pvmem in %s",
+            sprintf(log_buffer, "mm_getsize() failed for pvmem in %s",
               id);
 
             return(error(pname, retval));
@@ -1216,7 +1217,7 @@ int mom_set_limits(
             {
             log_buffer[0] = '\0';
 
-            sprintf(log_buffer, "invalid value returned by getsize() for pvmem in %s",
+            sprintf(log_buffer, "invalid value returned by mm_getsize() for pvmem in %s",
               id);
 
             return(error(pname, PBSE_BADATVAL));
@@ -1242,11 +1243,11 @@ int mom_set_limits(
 
         if (set_mode == SET_LIMIT_SET)
           {
-          retval = getsize(pres, &value);
+          retval = mm_getsize(pres, &value);
 
           if (retval != PBSE_NONE)
             {
-            sprintf(log_buffer, "getsize() failed for mem/pmem in %s",
+            sprintf(log_buffer, "mm_getsize() failed for mem/pmem in %s",
               id);
 
             return(error(pname, retval));
@@ -1293,11 +1294,11 @@ int mom_set_limits(
       {
       /* check */
 
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         {
-        sprintf(log_buffer, "gettime() failed for walltime in %s\n",
+        sprintf(log_buffer, "mm_gettime() failed for walltime in %s\n",
                 id);
 
         return(error(pname, retval));
@@ -1480,7 +1481,7 @@ mom_open_poll(void)
 
   if (proc_array == NULL)
     {
-    log_err(errno, id, "malloc");
+    log_err(errno, id, "calloc");
 
     return(PBSE_SYSTEM);
     }
@@ -1643,7 +1644,7 @@ int mom_over_limit(
 
     if ((igncput == FALSE) && (strcmp(pname, "cput") == 0))
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1659,7 +1660,7 @@ int mom_over_limit(
       }
     else if ((igncput == FALSE) && (strcmp(pname, "pcput") == 0))
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1674,7 +1675,7 @@ int mom_over_limit(
       }
     else if (strcmp(pname, "vmem") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1692,7 +1693,7 @@ int mom_over_limit(
       {
       unsigned long long valuell;
 
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1714,7 +1715,7 @@ int mom_over_limit(
       if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
         continue;
 
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -3157,7 +3158,7 @@ static char *totmem(
  * Has been used Win API.
  * Cygwin's /proc/meminfo also takes information from here
  * but does it incorrectly till now
-*/ 
+*/
 static char *availmem(
 
   struct rm_attribute *attrib)
@@ -3612,7 +3613,7 @@ char *idletime(
     return(NULL);
     }
 
-  sprintf(ret_string, "%ld", GetTickCount()/1000); 
+  sprintf(ret_string, "%ld", GetTickCount()/1000);
 
   return(ret_string);
   }  /* END idletime() */
@@ -3975,7 +3976,7 @@ static char *quota(
 
   if ((uid = (uid_t)atoi(attrib->a_value)) == 0)
     {
-    if ((pw = getpwnam(attrib->a_value)) == NULL)
+    if ((pw = getpwnam_ext(attrib->a_value)) == NULL)
       {
       sprintf(log_buffer,
               "user not found: %s", attrib->a_value);
@@ -3988,7 +3989,7 @@ static char *quota(
     }
 
   /* Cygwin doesn't yet support quota
-   * It is stub */ 
+   * It is stub */
   if (quotactl(
         QCMD(Q_GETQUOTA, USRQUOTA),
         me->mnt_fsname,
