@@ -140,7 +140,7 @@
 **  loadave  current load average
 */
 
-static char ident[] = "@(#) fujitsu/$RCSfile$ $Revision: 3230 $";
+static char ident[] = "@(#) fujitsu/$RCSfile$ $Revision: 5451 $";
 
 #ifndef TRUE
 #define FALSE 0
@@ -317,7 +317,7 @@ dep_cleanup(void)
  */
 
 static int
-getsize(resource *pres, unsigned long *ret)
+mm_getsize(resource *pres, unsigned long *ret)
   {
   unsigned long value;
 
@@ -353,7 +353,7 @@ getsize(resource *pres, unsigned long *ret)
  */
 
 static int
-gettime(resource *pres, unsigned long *ret)
+mm_gettime(resource *pres, unsigned long *ret)
   {
 
   if (pres->rs_value.at_type != ATR_TYPE_LONG)
@@ -623,7 +623,7 @@ mom_set_limits(
       if (igncput == FALSE)
         {
         /* cpu time - check, if less than pcput use it */
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -634,7 +634,7 @@ mom_set_limits(
       if (igncput == FALSE)
         {
         /* process cpu time - set */
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -650,7 +650,7 @@ mom_set_limits(
       {
       if (set_mode == SET_LIMIT_SET)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -668,7 +668,7 @@ mom_set_limits(
       {
       if (ignmem == FALSE)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -678,7 +678,7 @@ mom_set_limits(
       {
       if (ignvmem == FALSE)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -693,7 +693,7 @@ mom_set_limits(
         {
         if (set_mode == SET_LIMIT_SET)
           {
-          retval = getsize(pres, &value);
+          retval = mm_getsize(pres, &value);
 
           if (retval != PBSE_NONE)
             return (error(pname, retval));
@@ -711,7 +711,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "walltime") == 0)   /* Check */
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -800,12 +800,12 @@ mom_open_poll(void)
   char  *id = "mom_open_poll";
 
   DBPRT(("%s: entered\n", id))
-  proc_status = (prstatus_t *)malloc(sizeof(prstatus_t) * TBL_INC);
-  proc_info = (prpsinfo_t *)malloc(sizeof(prpsinfo_t) * TBL_INC);
+  proc_status = (prstatus_t *)calloc(sizeof(prstatus_t), TBL_INC);
+  proc_info = (prpsinfo_t *)calloc(sizeof(prpsinfo_t), TBL_INC);
 
   if (proc_status == NULL || proc_info == NULL)
     {
-    log_err(errno, id, "malloc");
+    log_err(errno, id, "calloc");
     return (PBSE_SYSTEM);
     }
 
@@ -922,7 +922,7 @@ mom_over_limit(job *pjob)
 
     if ((igncput == FALSE) && (strcmp(pname, "cput") == 0))
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -937,7 +937,7 @@ mom_over_limit(job *pjob)
       }
     else if ((igncput == FALSE) && (strcmp(pname, "pcput") == 0))
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -952,7 +952,7 @@ mom_over_limit(job *pjob)
       }
     else if (strcmp(pname, "vmem") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -970,7 +970,7 @@ mom_over_limit(job *pjob)
       if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
         continue;
 
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -1857,7 +1857,7 @@ ncpus(struct rm_attribute *attrib)
     }
 
   /* alloc the receiver PE table */
-  peidlist = (opr_peid_blk_t *)malloc(sizeof(opr_peid_blk_t) * ppnsize);
+  peidlist = (opr_peid_blk_t *)calloc(sizeof(opr_peid_blk_t), ppnsize);
 
   if (peidlist == NULL)
     {
