@@ -83,7 +83,7 @@
  *
  *   job_abt   abort (remove from server) a job
  *   job_alloc    allocate job struct and initialize defaults
- *   job_free   free space allocated to the job structure and its
+ *   mom_job_free   free space allocated to the job structure and its
  *    childern structures.
  *   job_purge   purge job from server
  *
@@ -134,9 +134,9 @@
 #include "net_connect.h"
 #include "portability.h"
 #include "threadpool.h"
+#include "alps_functions.h"
 #include "dis.h"
 
-
 #ifndef TRUE
 #define TRUE 1
 #define FALSE 0
@@ -161,17 +161,20 @@ static void job_init_wattr(job *);
 
 /* Global Data items */
 
-extern char tmpdir_basename[];	/* for TMPDIR */
-extern gid_t pbsgroup;
-extern uid_t pbsuser;
-extern char *msg_abt_err;
-extern char *path_jobs;
-extern char *path_spool;
-extern char *path_aux;
-extern char *msg_err_purgejob;
-extern char  server_name[];
-extern time_t time_now;
-extern int   LOGLEVEL;
+extern char   *apbasil_path;
+extern char   *apbasil_protocol;
+extern char    tmpdir_basename[];	/* for TMPDIR */
+extern gid_t   pbsgroup;
+extern uid_t   pbsuser;
+extern char   *msg_abt_err;
+extern char   *path_jobs;
+extern char   *path_spool;
+extern char   *path_aux;
+extern char   *msg_err_purgejob;
+extern char    server_name[];
+extern time_t  time_now;
+extern int     LOGLEVEL;
+extern int     is_login_node;
 
 extern tlist_head svr_newjobs;
 extern tlist_head svr_alljobs;
@@ -256,8 +259,7 @@ int remtree(
   char *dirname)
 
   {
-  static char id[] = "remtree";
-  DIR  *dir;
+  DIR           *dir;
 
   struct dirent *pdir;
   char           namebuf[MAXPATHLEN];
@@ -266,10 +268,10 @@ int remtree(
   int            rtnv = 0;
 #if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64) && defined(LARGEFILE_WORKS)
 
-  struct stat64 sb;
+  struct stat64  sb;
 #else
 
-  struct stat sb;
+  struct stat    sb;
 #endif
 
 #if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64) && defined(LARGEFILE_WORKS)
@@ -281,7 +283,7 @@ int remtree(
     {
 
     if (errno != ENOENT)
-      log_err(errno, id, "stat");
+      log_err(errno, __func__, "stat");
 
     return(-1);
     }
@@ -291,7 +293,7 @@ int remtree(
     if ((dir = opendir(dirname)) == NULL)
       {
       if (errno != ENOENT)
-        log_err(errno, id, "opendir");
+        log_err(errno, __func__, "opendir");
 
       return(-1);
       }
@@ -318,7 +320,7 @@ int remtree(
       if (lstat(namebuf, &sb) == -1)
 #endif
         {
-        log_err(errno, id, "stat");
+        log_err(errno, __func__, "stat");
 
         rtnv = -1;
 
@@ -334,7 +336,7 @@ int remtree(
         if (errno != ENOENT)
           {
           sprintf(log_buffer, "unlink failed on %s", namebuf);
-          log_err(errno, id, log_buffer);
+          log_err(errno, __func__, log_buffer);
           
           rtnv = -1;
           }
@@ -343,7 +345,7 @@ int remtree(
         {
         sprintf(log_buffer, "unlink(1) succeeded on %s", namebuf);
 
-        log_ext(-1, id, log_buffer, LOG_DEBUG);
+        log_ext(-1, __func__, log_buffer, LOG_DEBUG);
         }
       }    /* END while ((pdir = readdir(dir)) != NULL) */
 
@@ -356,7 +358,7 @@ int remtree(
         sprintf(log_buffer, "rmdir failed on %s",
                 dirname);
 
-        log_err(errno, id, log_buffer);
+        log_err(errno, __func__, log_buffer);
 
         rtnv = -1;
         }
@@ -365,13 +367,13 @@ int remtree(
       {
       sprintf(log_buffer, "rmdir succeeded on %s", dirname);
 
-      log_ext(-1, id, log_buffer, LOG_DEBUG);
+      log_ext(-1, __func__, log_buffer, LOG_DEBUG);
       }
     }
   else if (unlink(dirname) < 0)
     {
     snprintf(log_buffer,sizeof(log_buffer),"unlink failed on %s",dirname);
-    log_err(errno,id,log_buffer);
+    log_err(errno,__func__,log_buffer);
 
     rtnv = -1;
     }
@@ -379,7 +381,7 @@ int remtree(
     {
     sprintf(log_buffer, "unlink(2) succeeded on %s", dirname);
 
-    log_ext(-1, id, log_buffer, LOG_DEBUG);
+    log_ext(-1, __func__, log_buffer, LOG_DEBUG);
     }
 
   return(rtnv);
@@ -506,7 +508,7 @@ job *job_alloc(void)
 
 
 /*
- * job_free - free job structure and its various sub-structures
+ * mom_job_free - free job structure and its various sub-structures
  */
 
 void mom_job_free(
@@ -559,7 +561,7 @@ void mom_job_free(
   free((char *)pj);
 
   return;
-  }  /* END job_free() */
+  }  /* END mom_job_free() */
 
 
  /*
@@ -572,8 +574,8 @@ int job_unlink_file(
   const char *name)	 /* I */
 
   {
-  int saved_errno = 0;
-  int result = 0;
+  int   saved_errno = 0;
+  int   result = 0;
   uid_t uid = geteuid();
   gid_t gid = getegid();
 
@@ -596,7 +598,7 @@ int job_unlink_file(
   setegid(gid);
 
   errno = saved_errno;
-  return result;
+  return(result);
   }  /* END job_unlink_file() */
 
 
@@ -631,7 +633,6 @@ void *delete_job_files(
   void *vp)
 
   {
-  static char          *id = "delete_job_files";
   job_file_delete_info *jfdi = (job_file_delete_info *)vp;
   char                  namebuf[MAXPATHLEN];
   int                   rc = 0;
@@ -707,7 +708,7 @@ void *delete_job_files(
   if (unlink(namebuf) < 0)
     {
     if (errno != ENOENT)
-      log_err(errno,id,msg_err_purgejob);
+      log_err(errno,__func__,msg_err_purgejob);
     }
   else
     {
@@ -756,7 +757,7 @@ void *delete_job_files(
   if (unlink(namebuf) < 0)
     {
     if (errno != ENOENT)
-      log_err(errno,id,msg_err_purgejob);
+      log_err(errno,__func__,msg_err_purgejob);
     }
   else if (LOGLEVEL >= 6)
     {
@@ -773,20 +774,47 @@ void *delete_job_files(
 
 
 
+int release_job_reservation(
+
+  job *pjob)
+
+  {
+  int   rc = PBSE_NONE;
+  char *rsv_id;
+
+  /* release this job's reservation */
+  if ((pjob->ji_wattr[JOB_ATR_reservation_id].at_flags & ATR_VFLAG_SET) &&
+      (pjob->ji_wattr[JOB_ATR_reservation_id].at_val.at_str != NULL))
+    {
+    rsv_id = pjob->ji_wattr[JOB_ATR_reservation_id].at_val.at_str;
+
+    if ((rc = destroy_alps_reservation(rsv_id, apbasil_path, apbasil_protocol)) != PBSE_NONE)
+      {
+      snprintf(log_buffer, sizeof(log_buffer), "Couldn't release reservation for job %s",
+        pjob->ji_qs.ji_jobid);
+      log_err(-1, __func__, log_buffer);
+      }
+    }
+
+  return(rc);
+  } /* END release_job_reservation() */
+
+
+
+
 
 void job_purge(
 
   job *pjob)  /* I (modified) */
 
   {
-  static char           id[] = "job_purge";
   job_file_delete_info *jfdi;
 
   jfdi = calloc(1, sizeof(job_file_delete_info));
 
   if (jfdi == NULL)
     {
-    log_err(ENOMEM,id,"No space to allocate info for job file deletion");
+    log_err(ENOMEM,__func__,"No space to allocate info for job file deletion");
     return;
     }
 
@@ -802,6 +830,9 @@ void job_purge(
     }
 #endif  /* NVIDIA_GPUS */
 
+  if (is_login_node == TRUE)
+    release_job_reservation(pjob);
+
   /* initialize struct information */
   if (pjob->ji_flags & MOM_HAS_TMPDIR)
     {
@@ -825,8 +856,6 @@ void job_purge(
   if ((pjob->ji_wattr[JOB_ATR_checkpoint_dir].at_flags & ATR_VFLAG_SET) &&
       (pjob->ji_wattr[JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET))
     jfdi->checkpoint_dir = strdup(pjob->ji_wattr[JOB_ATR_checkpoint_dir].at_val.at_str);
-  else
-    jfdi->checkpoint_dir = NULL;
 
   jfdi->gid = pjob->ji_qs.ji_un.ji_momt.ji_exgid;
   jfdi->uid = pjob->ji_qs.ji_un.ji_momt.ji_exuid;
@@ -844,7 +873,7 @@ void job_purge(
     {
     sprintf(log_buffer,"removing job");
 
-    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buffer);
+    log_record(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
     }
 
 #if IBM_SP2==2        /* IBM SP PSSP 3.1 */
@@ -869,7 +898,7 @@ void job_purge(
 /*
  * find_job() - find job by jobid
  *
- * Search list of all server jobs for one with same job id
+ * Search list of all server jobs for one with same jobid
  * Return NULL if not found or pointer to job struct if found
  */
 
