@@ -114,6 +114,8 @@
 #include "resmon.h"
 #include "net_connect.h"
 #include "utils.h"
+#include "alps_functions.h"
+#include "tcp.h" /* tcp_chan */
 
 #ifdef _CRAY
 #include <sys/category.h>
@@ -124,6 +126,8 @@
 
 extern struct var_table vtable;      /* see start_exec.c */
 extern char           **environ;
+extern char            *apbasil_path;
+extern char            *apbasil_protocol;
 
 int reply_send_mom(struct batch_request *request);
 
@@ -141,6 +145,7 @@ extern int mkdirtree(
     mode_t mode);
 
 extern int TTmpDirName(job*, char *);
+extern int TMOMJobGetStartInfo(job *, pjobexec_t **) ;
 
 #endif /* HAVE_WORDEXP */
 
@@ -165,6 +170,10 @@ extern char            *msg_manager;
 extern time_t  time_now;
 extern int  multi_mom;
 extern int spoolasfinalname;
+#ifdef NVIDIA_GPUS
+extern int  use_nvidia_gpu;
+#endif /* NVIDIA_GPUS */
+
 /* in attr_fn_resc.c */
 
 extern char             MOMUNameMissing[];
@@ -246,8 +255,6 @@ static pid_t fork_to_user(
   char                 *EMsg)   /* I (optional,minsize=1024) */
 
   {
-  char           *id = "fork_to_user";
-
   struct group   *grpp;
   pid_t           pid;
   job            *pjob;
@@ -300,7 +307,7 @@ static pid_t fork_to_user(
       if (EMsg != NULL)
         strncpy(EMsg, log_buffer, 1024);
 
-      log_err(errno, id, log_buffer);
+      log_err(errno, __func__, log_buffer);
 
       return(-PBSE_BADUSER);
       }
@@ -324,7 +331,7 @@ static pid_t fork_to_user(
       if (EMsg != NULL)
         strncpy(EMsg, log_buffer, 1024);
 
-      log_err(errno, id, log_buffer);
+      log_err(errno, __func__, log_buffer);
 
       return(-PBSE_BADUSER);
       }
@@ -352,7 +359,7 @@ static pid_t fork_to_user(
     {
     /* FAILURE */
 
-    log_err(PBSE_UNKRESC, id, "cannot determine home directory");
+    log_err(PBSE_UNKRESC, __func__, "cannot determine home directory");
 
     if (EMsg != NULL)
       strncpy(EMsg, "cannot determine home directory", 1024);
@@ -376,7 +383,7 @@ static pid_t fork_to_user(
 
     if (LOGLEVEL >= 2)
       {
-      log_err(errno, id, log_buffer);
+      log_err(errno, __func__, log_buffer);
       }
 
     if (EMsg != NULL)
@@ -391,7 +398,7 @@ static pid_t fork_to_user(
     sprintf(log_buffer, "invalid home directory '%s' specified, not a directory",
             hdir);
 
-    log_err(PBSE_UNKRESC, id, log_buffer);
+    log_err(PBSE_UNKRESC, __func__, log_buffer);
 
     if (EMsg != NULL)
       strncpy(EMsg, log_buffer, 1024);
@@ -426,7 +433,7 @@ static pid_t fork_to_user(
             errno,
             strerror(errno));
 
-    log_err(-1, id, log_buffer);
+    log_err(-1, __func__, log_buffer);
 
     if (EMsg != NULL)
       strncpy(EMsg, log_buffer, 1024);
@@ -503,7 +510,7 @@ static pid_t fork_to_user(
               errno,
               strerror(errno));
 
-      log_err(-1, id, log_buffer);
+      log_err(-1, __func__, log_buffer);
 
       if (EMsg != NULL)
         strncpy(EMsg, log_buffer, 1024);
@@ -525,7 +532,7 @@ static pid_t fork_to_user(
               errno,
               strerror(errno));
 
-      log_err(-1, id, log_buffer);
+      log_err(-1, __func__, log_buffer);
 
       if (EMsg != NULL)
         strncpy(EMsg, log_buffer, 1024);
@@ -619,6 +626,7 @@ static int return_file(
   struct batch_request *prq;
   int                   rc = 0;
   int                   seq = 0;
+  struct tcp_chan      *chan = NULL;
 
   filename = std_file_name(pjob, which, &amt); /* amt is place holder */
 
@@ -661,26 +669,36 @@ static int return_file(
     /* prq->rq_ind.rq_jobfile.rq_size = amt; */
     /* prq->rq_ind.rq_jobfile.rq_data = buf; */
 
-    DIS_tcp_setup(sock);
-
-    if ((rc = encode_DIS_ReqHdr(sock, PBS_BATCH_MvJobFile, pbs_current_user)) ||
-        (rc = encode_DIS_JobFile(sock, seq++, buf, amt, pjob->ji_qs.ji_jobid, which)) ||
-        (rc = encode_DIS_ReqExtend(sock, NULL)))
+    if ((chan = DIS_tcp_setup(sock)) == NULL)
       {
       break;
       }
+    else if ((rc = encode_DIS_ReqHdr(chan, PBS_BATCH_MvJobFile, pbs_current_user)) ||
+             (rc = encode_DIS_JobFile(chan, seq++, buf, amt, pjob->ji_qs.ji_jobid, which)) ||
+             (rc = encode_DIS_ReqExtend(chan, NULL)))
+      {
+      DIS_tcp_cleanup(chan);
+      break;
+      }
 
-    DIS_tcp_wflush(sock);
+    DIS_tcp_wflush(chan);
 
-    if ((DIS_reply_read(sock, &prq->rq_reply) != 0) ||
+    if ((DIS_reply_read(chan, &prq->rq_reply) != 0) ||
         (prq->rq_reply.brp_code != 0))
       {
       rc = -1;
 
+      DIS_tcp_cleanup(chan);
       break;
       }
+
+    DIS_tcp_cleanup(chan);
+    chan = NULL;
     }    /* END while ((amt = read()) */
 
+  if (chan != NULL)
+    DIS_tcp_cleanup(chan);
+
   free_br(prq);
 
   close(fds);
@@ -775,8 +793,6 @@ static int told_to_cp(
   int    i;
   int    nh;
 
-  const char *id = "told_to_cp";
-
   static char newp[MAXPATHLEN + 1];
   char linkpath[MAXPATHLEN + 1];
   int max_links;
@@ -797,11 +813,7 @@ static int told_to_cp(
                   nh,
                   pcphosts[nh].cph_hosts);
 
-          log_record(
-            PBSEVENT_SYSTEM,
-            PBS_EVENTCLASS_SERVER,
-            (char *)id,
-            log_buffer);
+          log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, __func__, log_buffer);
           }
 
         i = strlen(pcphosts[nh].cph_from);
@@ -817,8 +829,7 @@ static int told_to_cp(
             snprintf(log_buffer, sizeof(log_buffer),
                      "too long string when transforming path '%s' to '%s%s'\n",
                      oldpath, pcphosts[nh].cph_to, oldpath + i);
-            log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER,
-                       (char *)id, log_buffer);
+            log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, __func__, log_buffer);
             return(0);
             }
 
@@ -837,8 +848,7 @@ static int told_to_cp(
               snprintf(log_buffer, sizeof(log_buffer),
                        "translation of symbolic link '%s' failed: %s\n",
                        newp, strerror(errno));
-              log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER,
-                         (char *)id, log_buffer);
+              log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, __func__, log_buffer);
               return(0);
               }
 
@@ -865,8 +875,7 @@ static int told_to_cp(
                        "translated symbolic link '%s:%s' to '%s:%s'; "
                        "restarting $usecp search\n",
                        host, newp, host, linkpath);
-              log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER,
-                         (char *)id, log_buffer);
+              log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, __func__, log_buffer);
               }
 
             oldpath = linkpath;
@@ -888,7 +897,7 @@ static int told_to_cp(
         log_record(
           PBSEVENT_SYSTEM,
           PBS_EVENTCLASS_SERVER,
-          (char *)id,
+          __func__,
           log_buffer);
         }
       }    /* END for (nh) */
@@ -903,7 +912,7 @@ static int told_to_cp(
             oldpath,
             nh);
 
-    log_ext(-1, (char *)id, log_buffer, LOG_DEBUG);
+    log_ext(-1, __func__, log_buffer, LOG_DEBUG);
     }
 
   return(0);
@@ -1032,8 +1041,6 @@ static int is_file_going_to_dir(
   char *destdir)
 
   {
-  char           *id = "is_file_going_to_dir";
-
 #if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64) && defined(LARGEFILE_WORKS)
 
   struct stat64 sb1;
@@ -1097,7 +1104,7 @@ static int is_file_going_to_dir(
     sprintf(log_buffer, "File %s stat failed, errno = %d",
             destdir,
             errno);
-    log_err(-1, id, log_buffer);
+    log_err(-1, __func__, log_buffer);
     }
 
   return(0);
@@ -1112,7 +1119,8 @@ void req_deletejob(
   struct batch_request *preq)  /* I */
 
   {
-  job *pjob;
+  job        *pjob;
+  pjobexec_t *TJE = NULL;
 
   pjob = find_job(preq->rq_ind.rq_delete.rq_objname);
 
@@ -1127,6 +1135,18 @@ void req_deletejob(
         "deleting job");
       }
 
+    /*
+     * We need to clear out the TJE starter slot if this job is in it.  This
+     * can occur if we qdel the job while the prologue is running.  If we
+     * don't remove ourself here then the TJE slot may remain with data that is
+     * no longer valid, yet we will reuse the slot since it only matches on the
+     * address of the pjob not on a jobid.  This can lead to unexpected crashes.
+     */
+    if (TMOMJobGetStartInfo(pjob, &TJE) == SUCCESS)
+      {
+      memset(TJE, 0, sizeof(pjobexec_t));
+      }
+
     /* assume success? */
 
     mom_deljob(pjob);
@@ -1157,7 +1177,7 @@ void mom_req_holdjob(
   int  rc;
   job *pjob;
   svrattrl *pal;
-  attribute tmph;
+  pbs_attribute tmph;
 
   /* If checkpoint supported, do it and terminate the job */
   /* otherwise, return PBSE_NOSUP    */
@@ -1232,8 +1252,6 @@ void req_gpuctrl_mom(
   struct batch_request *preq)  /* I */
 
   {
-  char   *id = "req_gpuctrl";
-
   char *mom_node;
   char *gpuid;
   int   gpumode = -1;
@@ -1260,7 +1278,27 @@ void req_gpuctrl_mom(
       gpumode,
       reset_perm,
       reset_vol);
-    log_ext(-1, id, log_buffer, LOG_INFO);
+    log_ext(-1, __func__, log_buffer, LOG_INFO);
+    }
+
+  if (!use_nvidia_gpu)
+    {
+    sprintf(
+      log_buffer,
+      "GPU control requests not active: node %s gpuid %s mode %d reset_perm %d reset_vol %d",
+      mom_node,
+      gpuid,
+      gpumode,
+      reset_perm,
+      reset_vol);
+
+    if (LOGLEVEL >= 3)
+      {
+      log_ext(-1, __func__, log_buffer, LOG_INFO);
+      }
+
+    req_reject(PBSE_NOSUP, 0, preq, NULL, NULL);
+    return;
     }
 
     /* assume success? */
@@ -1294,16 +1332,16 @@ void req_gpuctrl_mom(
     }
 #else
 
-    sprintf(log_buffer, "GPU control request not supported: node %s gpuid %s mode %d reset_perm %d reset_vol %d",
-      mom_node,
-      gpuid,
-      gpumode,
-      reset_perm,
-      reset_vol);
+  sprintf(log_buffer, "GPU control requests not supported: node %s gpuid %s mode %d reset_perm %d reset_vol %d",
+    mom_node,
+    gpuid,
+    gpumode,
+    reset_perm,
+    reset_vol);
 
   if (LOGLEVEL >= 3)
     {
-    log_ext(-1, id, log_buffer, LOG_INFO);
+    log_ext(-1, __func__, log_buffer, LOG_INFO);
     }
 
   req_reject(PBSE_NOSUP, 0, preq, NULL, NULL);
@@ -1551,16 +1589,16 @@ void req_modifyjob(
   struct batch_request *preq)  /* I */
 
   {
-  int   bad = 0;
-  int   i;
-  attribute  newattr[JOB_ATR_LAST];
-  attribute *pattr;
-  job  *pjob;
-  svrattrl *plist;
-  int   rc;
-  unsigned int momport = 0;
+  int            bad = 0;
+  int            i;
+  pbs_attribute  newattr[JOB_ATR_LAST];
+  pbs_attribute *pattr;
+  job           *pjob;
+  svrattrl      *plist;
+  int            rc;
+  unsigned int   momport = 0;
 
-  char tmpLine[1024];
+  char           tmpLine[1024];
 
   pjob = find_job(preq->rq_ind.rq_modify.rq_objname);
 
@@ -1630,7 +1668,7 @@ void req_modifyjob(
     return;
     }
 
-  /* OK, now copy the new values into the job attribute array */
+  /* OK, now copy the new values into the job pbs_attribute array */
 
   for (i = 0;i < JOB_ATR_LAST;i++)
     {
@@ -1829,7 +1867,7 @@ static void cray_susp_resum(
     {
     /* fork failed - still the main mom */
 
-    log_err(-1, id, "cannot fork child for cray suspend");
+    log_err(-1, __func__, "cannot fork child for cray suspend");
 
     req_reject(PBSE_SYSTEM, errno, preq, NULL, NULL);
 
@@ -1884,13 +1922,11 @@ int sigalltasks_sisters(
   int  signum)
 
   {
-#ifndef NDEBUG
-  char      id[] = "sigalltasks_sisters";
-#endif
-  char     *cookie;
-  eventent *ep;
-  int       i;
-  int       stream;
+  char            *cookie;
+  eventent        *ep;
+  int              i;
+  int              stream;
+  struct tcp_chan *chan = NULL;
 
   cookie = pjob->ji_wattr[JOB_ATR_Cookie].at_val.at_str;
 
@@ -1902,26 +1938,30 @@ int sigalltasks_sisters(
     if (np->hn_node == pjob->ji_nodeid) /* this is me */
       continue;
 
-    DBPRT(("%s: sending sig%d to all tasks on sister %s\n", id, signum, np->hn_host));
+    DBPRT(("%s: sending sig%d to all tasks on sister %s\n", __func__, signum, np->hn_host));
 
     ep = event_alloc(IM_SIGNAL_TASK, np, TM_NULL_EVENT, TM_NULL_TASK);
 
     if ((stream = tcp_connect_sockaddr((struct sockaddr *)&np->sock_addr,sizeof(np->sock_addr))) < 0)
         return(-1);
 
-    DIS_tcp_setup(stream);
-
-    ret = im_compose(stream, pjob->ji_qs.ji_jobid, cookie, IM_SIGNAL_TASK, ep->ee_event, TM_NULL_TASK);
-
-    if (ret == DIS_SUCCESS)
+    if ((chan = DIS_tcp_setup(stream)) == NULL)
       {
-      if ((ret = diswui(stream, pjob->ji_nodeid)) == DIS_SUCCESS)
+      }
+    else if ((ret = im_compose(chan, 
+            pjob->ji_qs.ji_jobid,
+            cookie,
+            IM_SIGNAL_TASK,
+            ep->ee_event,
+            TM_NULL_TASK)) == DIS_SUCCESS)
+      {
+      if ((ret = diswui(chan, pjob->ji_nodeid)) == DIS_SUCCESS)
         {
-        if ((ret = diswsi(stream, TM_NULL_TASK)) == DIS_SUCCESS)
+        if ((ret = diswsi(chan, TM_NULL_TASK)) == DIS_SUCCESS)
           {
-          if ((ret = diswsi(stream, signum)) == DIS_SUCCESS)
+          if ((ret = diswsi(chan, signum)) == DIS_SUCCESS)
             {
-            ret = DIS_tcp_wflush(stream);
+            ret = DIS_tcp_wflush(chan);
 
             /* NYI: add code to read reply from sister */
             }
@@ -1929,6 +1969,9 @@ int sigalltasks_sisters(
         }
       }
 
+    if (chan != NULL)
+      DIS_tcp_cleanup(chan);
+
     close(stream);
 
     if (ret != DIS_SUCCESS)
@@ -1948,8 +1991,6 @@ static void resume_suspend(
   struct batch_request *preq)
 
   {
-  static char *id = "resume_suspend";
-
   task *tp;
 
   int   stat = 0;
@@ -1962,9 +2003,8 @@ static void resume_suspend(
 
   if (LOGLEVEL >= 2)
     {
-    sprintf(log_buffer, "%s: %s job",
-            id,
-            (susp == 1) ? "suspending" : "resuming");
+    sprintf(log_buffer, "%s: %s job", __func__,
+      (susp == 1) ? "suspending" : "resuming");
 
     log_record(
       PBSEVENT_JOB,
@@ -2010,7 +2050,7 @@ static void resume_suspend(
       continue;
 
     DBPRT(("%s: inspecting %d from node %d\n",
-           id,
+           __func__,
            tp->ti_qs.ti_task,
            tp->ti_qs.ti_parentnode));
 
@@ -2045,11 +2085,12 @@ static void resume_suspend(
 
     if (LOGLEVEL >= 1)
       {
-      sprintf(log_buffer, "cannot send signal %s to tasks of job in %s (errno=%d %s) - attempt aborted",
-              (susp == 1) ? "SIGSTOP" : "SIGCONT",
-              id,
-              savederr,
-              pbs_strerror(savederr));
+      sprintf(log_buffer,
+        "cannot send signal %s to tasks of job in %s (errno=%d %s) - attempt aborted",
+        (susp == 1) ? "SIGSTOP" : "SIGCONT",
+        __func__,
+        savederr,
+        pbs_strerror(savederr));
 
       log_record(
         PBSEVENT_ERROR,
@@ -2163,7 +2204,6 @@ void req_signaljob(
   struct batch_request *preq) /* I */
 
   {
-  char            id[] = "req_signaljob";
   job            *pjob;
   int             sig;
   int             numprocs=0;
@@ -2280,7 +2320,7 @@ void req_signaljob(
     {
     /* if job is suspended, resume, and then kill - allow job to clean up on sigterm */
 
-    kill_job(pjob, SIGCONT, id, "job is suspended, resume and kill");
+    kill_job(pjob, SIGCONT, __func__, "job is suspended, resume and kill");
 
     sleep(1);
     }
@@ -2291,7 +2331,7 @@ void req_signaljob(
    * sent due to the current way that TORQUE's state machine works.
    */
 
-  numprocs = kill_job(pjob, sig, id, "killing job");
+  numprocs = kill_job(pjob, sig, __func__, "killing job");
 
   if ((numprocs == 0) && ((sig == 0)||(sig == SIGKILL)) &&
     (pjob->ji_qs.ji_substate != JOB_SUBSTATE_OBIT))
@@ -2337,10 +2377,10 @@ void encode_used(
 
   {
   unsigned long  lnum;
-  int   i;
-  attribute  *at;
-  attribute_def  *ad;
-  resource  *rs;
+  int            i;
+  pbs_attribute *at;
+  attribute_def *ad;
+  resource      *rs;
 
   at = &pjob->ji_wattr[JOB_ATR_resc_used];
   ad = &job_attr_def[JOB_ATR_resc_used];
@@ -2355,13 +2395,13 @@ void encode_used(
        rs = (resource *)GET_NEXT(rs->rs_link))
     {
     resource_def *rd = rs->rs_defin;
-    attribute     val;
+    pbs_attribute val;
     int           rc;
 
     if ((rd->rs_flags & perm) == 0)
       continue;
 
-    val = rs->rs_value; /* copy resource attribute */
+    val = rs->rs_value; /* copy resource pbs_attribute */
 
     /* count up sisterhood too */
 
@@ -2417,8 +2457,8 @@ void encode_flagged_attrs(
   tlist_head *phead)  /* O */
 
   {
-  int index;
-  attribute  *at;
+  int             index;
+  pbs_attribute  *at;
   attribute_def  *ad;
 
   for (index = 0;(int)index < JOB_ATR_LAST;++index)
@@ -2467,7 +2507,7 @@ void encode_flagged_attrs(
  * the server that tells the server the MOM's general stats (see mom_server_all_update_stat()).
  */
 
-void req_stat_job(
+int req_stat_job(
 
   struct batch_request *preq)  /* I */
 
@@ -2504,7 +2544,7 @@ void req_stat_job(
       {
       req_reject(PBSE_UNKJOBID, 0, preq, NULL, NULL);
 
-      return;
+      return(PBSE_UNKJOBID);
       }
     }
 
@@ -2543,7 +2583,7 @@ void req_stat_job(
 
   reply_send_mom(preq);
 
-  return;
+  return(PBSE_NONE);
   }  /* END req_stat_job() */
 
 
@@ -2566,8 +2606,6 @@ static int del_files(
   char                **pbadfile)  /* O */
 
   {
-  char   id[] = "del_files";
-
   int   AsUser = FALSE;
   int		UID0 = TRUE;
   int		EUID0 = TRUE;
@@ -2661,7 +2699,7 @@ static int del_files(
         {
         snprintf(log_buffer,sizeof(log_buffer),
           "%s: setgroups() for UID = %lu failed: %s",
-          id,
+          __func__,
           (unsigned long)useruid,
           strerror(errno));
 
@@ -2674,7 +2712,7 @@ static int del_files(
         {
         snprintf(log_buffer,sizeof(log_buffer),
           "%s: setgid(%lu) for UID = %lu failed: %s",
-          id,
+          __func__,
           (unsigned long)usergid,
           (unsigned long)useruid,
           strerror(errno));
@@ -2689,7 +2727,7 @@ static int del_files(
         {
         snprintf(log_buffer,sizeof(log_buffer),
           "%s: setuid(%lu) failed: %s",
-          id,
+          __func__,
           (unsigned long)useruid,
           strerror(errno));
 
@@ -2821,7 +2859,7 @@ static int del_files(
       sprintf(log_buffer, "cannot stat %s",
               path);
 
-      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_REQUEST, id, log_buffer);
+      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_REQUEST, __func__, log_buffer);
       }
 
     /*
@@ -2853,7 +2891,7 @@ static int del_files(
             errno,
             pbs_strerror(errno));
 
-          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_REQUEST, id, log_buffer);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_REQUEST, __func__, log_buffer);
 
           add_bad_list(pbadfile, log_buffer, 2);
 
@@ -2869,7 +2907,7 @@ static int del_files(
           path,
           preq->rq_ind.rq_cpyfile.rq_user);
 
-        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_FILE, id, log_buffer);
+        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_FILE, __func__, log_buffer);
 
 #endif  /* DEBUG */
         }
@@ -2888,8 +2926,6 @@ void req_rerunjob(
   struct batch_request *preq)  /* I */
 
   {
-  static char   *id = "req_rerunjob";
-
   job           *pjob;
   int            sock;
   int            rc;
@@ -2929,7 +2965,7 @@ void req_rerunjob(
    * This is acceptable because we are a child process, not pbs_mom.
    */
 
-  while ((sock = mom_open_socket_to_jobs_server(pjob, id, NULL)) < 0)
+  while ((sock = mom_open_socket_to_jobs_server(pjob, __func__, NULL)) < 0)
     {
     retrycnt++;
     if (retrycnt < 10)
@@ -2955,7 +2991,7 @@ void req_rerunjob(
     log_event(
       PBSEVENT_ERROR,
       PBS_EVENTCLASS_REQUEST,
-      id,
+      __func__,
       "cannot move output files to server");
 
     req_reject(rc, 0, preq, NULL, NULL);
@@ -2986,13 +3022,12 @@ void req_returnfiles(
   struct job  *pjob;
   int          sock;
   int          retry_attempts = 0;
-  static char *id = "req_returnfiles";
 
   pjob = find_job(preq->rq_ind.rq_returnfiles.rq_jobid);
 
   if (pjob != NULL)
     {
-    while ((sock = mom_open_socket_to_jobs_server(pjob, id, NULL)) < 0)
+    while ((sock = mom_open_socket_to_jobs_server(pjob, __func__, NULL)) < 0)
       {
       if (retry_attempts++ >= 10)
         {
@@ -3005,7 +3040,7 @@ void req_returnfiles(
         sock,
         pjob->ji_qs.ji_jobid);
       
-      log_err(-1, id, log_buffer);
+      log_err(-1, __func__, log_buffer);
       sleep(1);
       }
 
@@ -3052,7 +3087,6 @@ static int sys_copy(
   char *ag0;
   char *ag1;
   int i;
-  static char *id = "sys_copy";
   int loop;
   int rc;
 
@@ -3081,7 +3115,7 @@ static int sys_copy(
             ag2,
             ag3);
 
-    log_ext(-1, id, log_buffer, LOG_DEBUG);
+    log_ext(-1, __func__, log_buffer, LOG_DEBUG);
     }
 
   for (loop = 1;loop < 4;++loop)
@@ -3133,7 +3167,7 @@ static int sys_copy(
         sprintf(log_buffer, "can't open %s, error = %d %s",
                 rcperr, errno, pbs_strerror(errno));
 
-        log_err(errno, id, log_buffer);
+        log_err(errno, __func__, log_buffer);
 
         exit(12);
         };
@@ -3159,7 +3193,7 @@ static int sys_copy(
               errno,
               pbs_strerror(errno));
 
-      log_err(errno, id, log_buffer);
+      log_err(errno, __func__, log_buffer);
 
       exit(13); /* 13, an unlucky number */
       }    /* END else ((rc = fork()) > 0) */
@@ -3180,7 +3214,7 @@ static int sys_copy(
           rc,
           loop);
 
-  log_err(-1, id, log_buffer);
+  log_err(-1, __func__, log_buffer);
 
   return(rc);
   }  /* END sys_copy() */
@@ -3227,8 +3261,6 @@ void req_cpyfile(
   struct batch_request *preq)  /* I */
 
   {
-  static char     id[] = "req_cpyfile";
-
   char           *arg2 = NULL;
   char           *arg3 = NULL;
   int             bad_files = 0;
@@ -3306,13 +3338,14 @@ void req_cpyfile(
 
     req_reject(-rc, 0, preq, mom_host, EMsg);
 
-    if ((rc != -PBSE_SYSTEM) && (rc != -PBSE_BADUSER))
+    if ((rc != -PBSE_SYSTEM) &&
+        (rc != -PBSE_BADUSER))
       {
       sprintf(tmpLine, "fork_to_user failed with rc=%d '%s' - exiting",
         rc,
         EMsg);
 
-      log_err(errno, id, tmpLine);
+      log_err(errno, __func__, tmpLine);
 
       exit(rc);
       }
@@ -3321,7 +3354,7 @@ void req_cpyfile(
       rc,
       EMsg);
 
-    log_err(errno, id, tmpLine);
+    log_err(errno, __func__, tmpLine);
 
     return;
     }  /* END if (rc < 0) */
@@ -3427,7 +3460,7 @@ void req_cpyfile(
       sprintf(log_buffer,"alloc failed with errno=%d - returning failure",
         errno);
 
-      log_err(errno,id,log_buffer);
+      log_err(errno, __func__, log_buffer);
 
       bad_files = 1;
 
@@ -3451,7 +3484,7 @@ void req_cpyfile(
           sprintf(log_buffer,"alloc failed with errno=%d - returning failure",
             errno);
 
-          log_err(errno,id,log_buffer);
+          log_err(errno, __func__, log_buffer);
 
           bad_files = 1;
 
@@ -3490,7 +3523,7 @@ void req_cpyfile(
     sprintf(log_buffer,"alloc failed with errno=%d - returning failure",
       errno);
 
-    log_err(errno,id,log_buffer);
+    log_err(errno, __func__, log_buffer);
 
     bad_files = 1;
 
@@ -3701,7 +3734,7 @@ void req_cpyfile(
 
         if ((mkdir(needdir, 01777) == -1) && (errno != EEXIST))
           {
-          log_err(errno, id, "Failed to create jobs checkpoint directory");
+          log_err(errno, __func__, "Failed to create jobs checkpoint directory");
           }
 
         umask(saveumask);
@@ -3869,7 +3902,7 @@ nextword:
 
       add_bad_list(&bad_list, log_buffer, 2);
 
-      log_err(-1, id, log_buffer);
+      log_err(-1, __func__, log_buffer);
 
       /* copy message from rcp as well */
 
@@ -3927,7 +3960,7 @@ error:
                   localname,
                   undelname);
 
-          log_err(errno, id, log_buffer);
+          log_err(errno, __func__, log_buffer);
           }
 
 #endif /* !NO_SPOOL_OUTPUT */
@@ -3947,7 +3980,7 @@ error:
         {
         sprintf(log_buffer,"copy succeeded (%s) from (%s) to (%s)\n",
           (dir == 0)? "In" : "Out", arg2, arg3);
-        log_ext(-1, id, log_buffer, LOG_DEBUG);
+        log_ext(-1, __func__, log_buffer, LOG_DEBUG);
         }
 
       if (dir == STAGE_DIR_OUT)
@@ -3960,7 +3993,7 @@ error:
                   "stage out",
                   localname);
 
-          log_err(errno, id, log_buffer);
+          log_err(errno, __func__, log_buffer);
 
 
           add_bad_list(&bad_list, log_buffer, 2);
@@ -3988,7 +4021,7 @@ error:
         if (LOGLEVEL >= 7)
           {
           sprintf(log_buffer,"removing checkpoint file (%s)\n", localname);
-          log_ext(-1, id, log_buffer, LOG_DEBUG);
+          log_ext(-1, __func__, log_buffer, LOG_DEBUG);
           }
 
         /* have copied out, ok to remove local one */
@@ -3999,7 +4032,7 @@ error:
                   "checkpoint",
                   localname);
 
-          log_err(errno, id, log_buffer);
+          log_err(errno, __func__, log_buffer);
 
           add_bad_list(&bad_list, log_buffer, 2);
 
@@ -4030,7 +4063,7 @@ error:
     {
     reply_text(preq, PBSE_NOCOPYFILE, bad_list);
 
-    log_err(-1,id,bad_list);
+    log_err(-1, __func__, bad_list);
     }
   else
     {
@@ -4112,5 +4145,31 @@ void req_delfile(
 
 
 
+void req_delete_reservation(
+    
+  struct batch_request *request)
+
+  {
+  char *rsv_id = request->rq_extend;
+  int   rc = PBSE_NONE;
+
+  if (rsv_id != NULL)
+    {
+    if ((rc = destroy_alps_reservation(rsv_id, apbasil_path, apbasil_protocol)) != PBSE_NONE)
+      {
+      snprintf(log_buffer, sizeof(log_buffer), "Couldn't release reservation id %s",
+        rsv_id);
+      log_err(-1, __func__, log_buffer);
+      }
+    }
+
+  if (rc == PBSE_NONE)
+    reply_ack(request);
+  else
+    req_reject(-1, 0, request, NULL, log_buffer);
+  } /* END req_delete_reservation() */
+
+
+
 
 /* END requests.c */
