@@ -107,6 +107,8 @@
 #ifdef USESAVEDRESOURCES
 #include "resource.h"
 #endif
+#include "svr_func.h"
+#include "server.h"
 
 /* Local Data */
 
@@ -140,6 +142,7 @@ int acct_job(
 
   {
   int         rc;
+  long        cray_enabled = FALSE;
   int         resc_access_perm = READ_ONLY;
   char        local_buf[MAXLINE*4];
   pbs_queue  *pque;
@@ -237,7 +240,17 @@ int acct_job(
   if ((rc = append_dynamic_string(ds, " ")) != PBSE_NONE)
     return(rc);
 
-  /* now encode the job's resource_list attribute */
+  get_svr_attr_l(SRV_ATR_CrayEnabled, &cray_enabled);
+  if ((cray_enabled == TRUE) &&
+      (pjob->ji_wattr[JOB_ATR_login_node_id].at_flags & ATR_VFLAG_SET))
+    {
+    append_dynamic_string(ds, "login_node=");
+    append_dynamic_string(ds, pjob->ji_wattr[JOB_ATR_login_node_id].at_val.at_str);
+    if ((rc = append_dynamic_string(ds, " ")) != PBSE_NONE)
+      return(rc);
+    }
+
+  /* now encode the job's resource_list pbs_attribute */
   job_attr_def[JOB_ATR_resource].at_encode(
     &pjob->ji_wattr[JOB_ATR_resource],
     &attrlist,
@@ -493,12 +506,12 @@ void account_jobend(
   char *used) /* job usage information, see req_jobobit() */
 
   {
-  time_t          time_now = time(NULL);
-  dynamic_string *ds;
-  char            local_buf[MAXLINE * 4];
+  time_t              time_now = time(NULL);
+  dynamic_string     *ds;
+  char                local_buf[MAXLINE * 4];
 #ifdef USESAVEDRESOURCES
-  attribute      *pattr;
-  long            walltime_val = 0;
+  pbs_attribute      *pattr;
+  long                walltime_val = 0;
 #endif
 
   /* pack in general information about the job */
@@ -591,11 +604,10 @@ void acct_cleanup(
   long	days_to_keep)	/* Number of days to keep accounting files */
 
   {
-  char *id = "acct_cleanup";
 
   if (log_remove_old(path_acct,(days_to_keep * SECS_PER_DAY)) != 0)
     {
-    log_err(-1,id,"failure occurred when checking for old accounting logs");
+    log_err(-1, __func__, "failure occurred when checking for old accounting logs");
     }
 
   return;
