@@ -243,20 +243,23 @@ static void reissue_to_svr(
   struct work_task *pwt)
 
   {
-  time_t                time_now = time(NULL);
-  struct batch_request *preq = pwt->wt_parm1;
+  time_t         time_now = time(NULL);
+  char          *br_id = pwt->wt_parm1;
+  batch_request *preq = get_remove_batch_request(br_id);
 
   /* if not timed-out, retry send to remote server */
-
-  if (((time_now - preq->rq_time) > PBS_NET_RETRY_LIMIT) ||
-      (issue_to_svr(preq->rq_host, preq, pwt->wt_parmfunc) != PBSE_NONE))
+  if (preq != NULL)
     {
-    /* either timed-out or got hard error, tell post-function  */
-
-    pwt->wt_aux = -1; /* seen as error by post function  */
-    pwt->wt_event = -1; /* seen as connection by post func */
-
-    ((void (*)())pwt->wt_parmfunc)(pwt);
+    if (((time_now - preq->rq_time) > PBS_NET_RETRY_LIMIT) ||
+        (issue_to_svr(preq->rq_host, preq, pwt->wt_parmfunc) != PBSE_NONE))
+      {
+      /* either timed-out or got hard error, tell post-function  */
+      
+      pwt->wt_aux = -1; /* seen as error by post function  */
+      pwt->wt_event = -1; /* seen as connection by post func */
+      
+      ((void (*)())pwt->wt_parmfunc)(pwt);
+      }
     }
 
   free(pwt->wt_mutex);
@@ -320,11 +323,15 @@ int issue_to_svr(
 
     if (handle >= 0)
       {
-      if ((rc = issue_Drequest(handle, preq, replyfunc, NULL)) == PBSE_NONE)
+      if (((rc = issue_Drequest(handle, preq, replyfunc, NULL)) == PBSE_NONE) &&
+          (handle != PBS_LOCAL_CONNECTION))
         {
+        /* preq is already freed if handle == PBS_LOCAL_CONNECTION - a reply 
+         * has always been sent */
         rc = preq->rq_reply.brp_code;
         }
-      return rc;
+
+      return(rc);
       }
     else if (handle == PBS_NET_RC_RETRY)
       {
@@ -336,18 +343,21 @@ int issue_to_svr(
 
   if (do_retry)
     {
-    pwt = set_task(WORK_Timed,(long)(time_now + PBS_NET_RETRY_TIME),reissue_to_svr,preq,TRUE);
+    if (preq->rq_id == NULL)
+      get_batch_request_id(preq);
+
+    pwt = set_task(WORK_Timed, (long)(time_now + PBS_NET_RETRY_TIME), reissue_to_svr, preq->rq_id, TRUE);
 
     pwt->wt_parmfunc = replyfunc;
 
     pthread_mutex_unlock(pwt->wt_mutex);
 
-    return PBSE_NONE;
+    return(PBSE_NONE);
     }
 
   /* FAILURE */
 
-  return PBSE_INTERNAL;
+  return(PBSE_INTERNAL);
   }  /* END issue_to_svr() */
 
 
@@ -370,7 +380,11 @@ void release_req(
   struct work_task *pwt)
 
   {
-  free_br((struct batch_request *)pwt->wt_parm1);
+  batch_request *preq;
+  char          *br_id = pwt->wt_parm1;
+
+  if ((preq = get_remove_batch_request(br_id)) != NULL)
+    free_br(preq);
 
   if (pwt->wt_event != -1)
     svr_disconnect(pwt->wt_event);
@@ -421,16 +435,15 @@ int issue_Drequest(
   {
   struct attropl   *patrl;
 
-  struct work_task *ptask;
+  struct work_task *ptask = NULL;
 
   struct svrattrl  *psvratl;
   int               rc = PBSE_NONE;
   int               tmp_rc = PBSE_NONE;
   int               sock = 0;
   enum work_type    wt;
-  char             *id = "issue_Drequest";
   char              log_buf[LOCAL_LOG_BUF_SIZE];
-  struct tcp_chan *chan = NULL;
+  struct tcp_chan  *chan = NULL;
 
   if (conn == PBS_LOCAL_CONNECTION)
     {
@@ -455,32 +468,37 @@ int issue_Drequest(
       log_err(PBSE_MEM_MALLOC, __func__,
           "Could not allocate memory for socket buffer");
       close_conn(sock, FALSE);
-      return PBSE_MEM_MALLOC;
+
+      return(PBSE_MEM_MALLOC);
       }
     }
 
   if (conn == PBS_LOCAL_CONNECTION)
     {
     /* the request should be issued to ourself */
+    rc = dispatch_request(PBS_LOCAL_CONNECTION, request);
 
-    dispatch_request(PBS_LOCAL_CONNECTION, request);
-
-    return(0);
+    return(rc);
     }
 
   if (func != NULL)
     {
-    ptask = set_task(wt, (long)conn, func, (void *)request, FALSE);
+    if (request->rq_id == NULL)
+      get_batch_request_id(request);
+
+    ptask = set_task(wt, (long)conn, func, request->rq_id, FALSE);
 
     if (ppwt != NULL)
       *ppwt = ptask;
 
     if (ptask == NULL)
       {
-      log_err(errno, id, "could not set_task");
+      log_err(errno, __func__, "could not set_task");
       close_conn(sock, FALSE);
+
       if (chan != NULL)
         DIS_tcp_cleanup(chan);
+
       return(PBSE_MEM_MALLOC);
       }
     }
@@ -719,7 +737,7 @@ int issue_Drequest(
       sprintf(log_buf, msg_issuebad,
               request->rq_type);
 
-      log_err(-1, id, log_buf);
+      log_err(-1, __func__, log_buf);
 
       pthread_mutex_lock(ptask->wt_mutex);
       if (ptask->wt_being_recycled == FALSE)
@@ -731,6 +749,7 @@ int issue_Drequest(
 
       break;
     }  /* END switch (request->rq_type) */
+
   if ((tmp_rc = DIS_reply_read(chan, &request->rq_reply)) != 0)
     {
     sprintf(log_buf, "DIS_reply_read failed: %d", tmp_rc);
@@ -740,10 +759,15 @@ int issue_Drequest(
     }  
   DIS_tcp_cleanup(chan);
   svr_disconnect(conn);
+
   if (func != NULL)
     dispatch_task(ptask);
+
   return(rc);
   }  /* END issue_Drequest() */
 
+
+
+
 /* END issue_request.c */
 
