@@ -77,8 +77,8 @@
 * without reference to its choice of law rules.
 */
 /*
- * job_attr_def is the array of attribute definitions for jobs.
- * Each legal job attribute is defined here.
+ * job_attr_def is the array of pbs_attribute definitions for jobs.
+ * Each legal job pbs_attribute is defined here.
  */
 
 #include <pbs_config.h>		/* the master config generated by configure */
@@ -90,16 +90,16 @@
 
 /* Extern functions (at_action) called */
 
-extern int job_set_wait (attribute *, void *, int);
-extern int action_resc (attribute *, void *, int);
-extern int ck_checkpoint (attribute *, void *, int);
-extern int depend_on_que (attribute *, void *, int);
-extern int comp_checkpoint (attribute *, attribute *);
+extern int job_set_wait (pbs_attribute *, void *, int);
+extern int action_resc (pbs_attribute *, void *, int);
+extern int ck_checkpoint (pbs_attribute *, void *, int);
+extern int depend_on_que (pbs_attribute *, void *, int);
+extern int comp_checkpoint (pbs_attribute *, pbs_attribute *);
 
 #define ATR_DFLAG_SSET  (ATR_DFLAG_SvWR | ATR_DFLAG_SvRD)
 
 /*
- * The entries for each attribute are (see attribute.h):
+ * The entries for each pbs_attribute are (see attribute.h):
  * name,
  * decode function,
  * encode function,
@@ -1035,7 +1035,6 @@ attribute_def job_attr_def[] =
    ATR_TYPE_STR,
    PARENT_TYPE_JOB},
 
-#ifdef USEJOBCREATE
   /* JOB_ATR_pagg_id */
   { ATTR_pagg,  /* "pagg_id" */
     decode_ll,
@@ -1048,7 +1047,6 @@ attribute_def job_attr_def[] =
     ATR_TYPE_LL,
     PARENT_TYPE_JOB
   },
-#endif /* USEJOBCREATE */
 
 #ifdef NVIDIA_GPUS
   /* JOB_ATR_gpu_flags */
@@ -1065,7 +1063,7 @@ attribute_def job_attr_def[] =
 #endif  /* NVIDIA_GPUS */
 
   /* JOB_ATR_job_id */
-  {ATTR_J,      /* job_id */
+  {ATTR_J,      /* "job_id" */
    decode_str,
    encode_str,
    set_str,
@@ -1077,7 +1075,7 @@ attribute_def job_attr_def[] =
    PARENT_TYPE_JOB},
 
   /* JOB_ATR_arguments*/
-  {ATTR_args,		/* job_arguments */
+  {ATTR_args,		/* "job_arguments" */
    decode_str,
    encode_str,
    set_str,
@@ -1088,6 +1086,42 @@ attribute_def job_attr_def[] =
    ATR_TYPE_STR,
    PARENT_TYPE_JOB},
 
+  /* JOB_ATR_reservation_id */
+  {ATTR_reservation_id, /* "reservation_id" */
+   decode_str,
+   encode_str,
+   set_str,
+   comp_str,
+   free_str,
+   NULL_FUNC,
+   READ_WRITE |  ATR_DFLAG_MOM,
+   ATR_TYPE_STR,
+   PARENT_TYPE_JOB},
+
+  /* JOB_ATR_login_node_id */
+  {ATTR_login_node_id,  /* "login_node_id" */
+   decode_str,
+   encode_str,
+   set_str,
+   comp_str,
+   free_str,
+   NULL_FUNC,
+   READ_WRITE,
+   ATR_TYPE_STR,
+   PARENT_TYPE_JOB},
+
+  /* JOB_ATR_login_prop */
+  {ATTR_login_prop, /* "login_property" */
+   decode_str,
+   encode_str,
+   set_str,
+   comp_str,
+   free_str,
+   NULL_FUNC,
+   READ_WRITE,
+   ATR_TYPE_STR,
+   PARENT_TYPE_JOB},
+
   /* Site defined attributes if any, see site_job_attr_*.h  */
 #include "site_job_attr_def.h"
 
