@@ -91,7 +91,7 @@
  *   job_clone_wt work task for cloning a job
  *
  * Include private function:
- *   job_init_wattr() initialize job working attribute array to "unspecified"
+ *   job_init_wattr() initialize job working pbs_attribute array to "unspecified"
  *
  * NOTE: for multi-threaded TORQUE, all functions in here except find_job assume that
  * the caller holds any relevant mutexes
@@ -144,6 +144,7 @@
 #include "resizable_array.h"
 #include "dynamic_string.h"
 #include "svr_func.h" /* get_svr_attr_* */
+#include "track_alps_reservations.h"
 #include "req_signal.h" /* issue_signal */
 #include "issue_request.h" /* release_req */
 
@@ -199,9 +200,9 @@ void send_qsub_delmsg(
   char *text)  /* I */
 
   {
-  char      *phost;
-  attribute *pattri;
-  int        qsub_sock;
+  char          *phost;
+  pbs_attribute *pattri;
+  int            qsub_sock;
 
   phost = arst_string("PBS_O_HOST", &pjob->ji_wattr[JOB_ATR_variables]);
 
@@ -248,7 +249,6 @@ int remtree(
   char *dirname)
 
   {
-  static char    id[] = "remtree";
   DIR           *dir;
 
   struct dirent *pdir;
@@ -273,7 +273,7 @@ int remtree(
     {
 
     if (errno != ENOENT)
-      log_err(errno, id, "stat");
+      log_err(errno, __func__, "stat");
 
     return(-1);
     }
@@ -283,7 +283,7 @@ int remtree(
     if ((dir = opendir(dirname)) == NULL)
       {
       if (errno != ENOENT)
-        log_err(errno, id, "opendir");
+        log_err(errno, __func__, "opendir");
 
       return(-1);
       }
@@ -310,7 +310,7 @@ int remtree(
       if (lstat(namebuf, &sb) == -1)
 #endif
         {
-        log_err(errno, id, "stat");
+        log_err(errno, __func__, "stat");
 
         rtnv = -1;
 
@@ -327,7 +327,7 @@ int remtree(
           {
           sprintf(log_buf, "unlink failed on %s", namebuf);
 
-          log_err(errno, id, log_buf);
+          log_err(errno, __func__, log_buf);
 
           rtnv = -1;
           }
@@ -336,7 +336,7 @@ int remtree(
         {
         sprintf(log_buf, "unlink(1) succeeded on %s", namebuf);
 
-        log_ext(-1, id, log_buf, LOG_DEBUG);
+        log_ext(-1, __func__, log_buf, LOG_DEBUG);
         }
       }    /* END while ((pdir = readdir(dir)) != NULL) */
 
@@ -348,7 +348,7 @@ int remtree(
         {
         sprintf(log_buf, "rmdir failed on %s", dirname);
 
-        log_err(errno, id, log_buf);
+        log_err(errno, __func__, log_buf);
 
         rtnv = -1;
         }
@@ -357,14 +357,14 @@ int remtree(
       {
       sprintf(log_buf, "rmdir succeeded on %s", dirname);
 
-      log_ext(-1, id, log_buf, LOG_DEBUG);
+      log_ext(-1, __func__, log_buf, LOG_DEBUG);
       }
     }
   else if (unlink(dirname) < 0)
     {
     sprintf(log_buf, "unlink failed on %s", dirname);
 
-    log_err(errno, id, log_buf);
+    log_err(errno, __func__, log_buf);
 
     rtnv = -1;
     }
@@ -372,7 +372,7 @@ int remtree(
     {
     sprintf(log_buf, "unlink(2) succeeded on %s", dirname);
 
-    log_ext(-1, id, log_buf, LOG_DEBUG);
+    log_ext(-1, __func__, log_buf, LOG_DEBUG);
     }
 
   return(rtnv);
@@ -451,7 +451,7 @@ int job_abt(
     {
     svr_setjobstate(pjob, JOB_STATE_RUNNING, JOB_SUBSTATE_ABORT, FALSE);
 
-    if ((rc = issue_signal(&pjob, "SIGKILL", release_req, 0)) != 0)
+    if ((rc = issue_signal(&pjob, "SIGKILL", release_req, NULL)) != 0)
       {
       if (pjob != NULL)
         {
@@ -644,7 +644,7 @@ int conn_qsub(
 
 /*
  * job_alloc - allocate space for a job structure and initialize working
- * attribute to "unset"
+ * pbs_attribute to "unset"
  *
  * Returns: pointer to structure or null is space not available.
  *
@@ -658,7 +658,7 @@ job *job_alloc(void)
   
   if (pj == NULL)
     {
-    log_err(errno, "job_alloc", "no memory");
+    log_err(errno, __func__, "no memory");
     
     return(NULL);
     }
@@ -691,7 +691,7 @@ job *job_alloc(void)
 void job_free(
 
   job *pj,
-  int use_recycle)  /* I (modified) */
+  int  use_recycle)  /* I (modified) */
 
   {
   int               i;
@@ -706,7 +706,7 @@ void job_free(
     log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pj->ji_qs.ji_jobid,log_buf);
     }
 
-  /* remove any calloc working attribute space */
+  /* remove any calloc working pbs_attribute space */
   for (i = 0;i < JOB_ATR_LAST;i++)
     {
     job_attr_def[i].at_free(&pj->ji_wattr[i]);
@@ -739,12 +739,11 @@ void job_free(
   else
     {
     pthread_mutex_unlock(pj->ji_mutex);
-    free(pj->ji_mutex);
+    pthread_mutex_destroy(pj->ji_mutex);
     memset(pj, 254, sizeof(job));
     free(pj);
     }
 
-
   return;
   }  /* END job_free() */
 
@@ -763,31 +762,31 @@ job *job_clone(
   int        taskid)  /* I */
 
   {
-  char log_buf[LOCAL_LOG_BUF_SIZE];
+  char           log_buf[LOCAL_LOG_BUF_SIZE];
 
-  job  *pnewjob;
-  attribute tempattr;
+  job           *pnewjob;
+  pbs_attribute  tempattr;
 
-  char  *oldid;
-  char  *hostname;
-  char  *bracket;
-  char  *tmpstr;
-  char   basename[PBS_JOBBASE+1];
-  char   namebuf[MAXPATHLEN + 1];
-  char   buf[256];
-  char  *pc;
-  int    fds;
-
-  int    i;
-  int    slen;
-  int    release_mutex = FALSE;
+  char          *oldid;
+  char          *hostname;
+  char          *bracket;
+  char          *tmpstr;
+  char           basename[PBS_JOBBASE+1];
+  char           namebuf[MAXPATHLEN + 1];
+  char           buf[256];
+  char          *pc;
+  int            fds;
 
+  int            i;
+  int            slen;
+  int            release_mutex = FALSE;
 
   if (LOGLEVEL >= 7)
     {
     sprintf(log_buf, "taskid %d", taskid);
     log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
     }
+
   if (taskid > PBS_MAXJOBARRAY)
     {
     log_err(-1, __func__, "taskid out of range");
@@ -1039,7 +1038,7 @@ void *job_clone_wt(
   if (jobid == NULL)
     {
     log_err(ENOMEM, __func__, "Can't malloc");
-    return NULL;
+    return(NULL);
     }
 
   /* don't call get_jobs_array because the template job isn't part of the array */
@@ -1047,9 +1046,10 @@ void *job_clone_wt(
       ((pa = get_jobs_array(&template_job)) == NULL))
     {
     free(jobid);
+
     if (template_job != NULL)
       pthread_mutex_unlock(template_job->ji_mutex);
-    return NULL;
+    return(NULL);
     }
 
   free(jobid);
@@ -1076,6 +1076,12 @@ void *job_clone_wt(
         }
 
       svr_evaljobstate(pjobclone, &newstate, &newsub, 1);
+
+      /* do this so that  svr_setjobstate() doesn't alter sv_jobstates,
+       * these are set later in svr_enquejob() */
+      pjobclone->ji_qs.ji_state = newstate;
+      pjobclone->ji_qs.ji_substate = newsub;
+
       svr_setjobstate(pjobclone, newstate, newsub, FALSE);
 
       pjobclone->ji_wattr[JOB_ATR_qrank].at_val.at_long = ++queue_rank;
@@ -1085,7 +1091,10 @@ void *job_clone_wt(
         {
         /* XXX need more robust error handling */
         pthread_mutex_unlock(pa->ai_mutex);
-        job_purge(pjobclone);
+
+        if (rc != PBSE_JOB_RECYCLED)
+          job_purge(pjobclone);
+
         pthread_mutex_lock(pa->ai_mutex);
         continue;
         }
@@ -1105,7 +1114,8 @@ void *job_clone_wt(
       
       rn->start++;
       
-      pthread_mutex_unlock(pjobclone->ji_mutex);
+      if (prev_index != -1)
+        pthread_mutex_unlock(pjobclone->ji_mutex);
       }  /* END for (i) */
 
     if (rn->start > rn->end)
@@ -1171,14 +1181,14 @@ void *job_clone_wt(
     }
   
   pthread_mutex_unlock(pa->ai_mutex);
-  return NULL;
+  return(NULL);
   }  /* END job_clone_wt */
 
 
 
 
 /*
- * job_init_wattr - initialize job working attribute array
+ * job_init_wattr - initialize job working pbs_attribute array
  * set the types and the "unspecified value" flag
  */
 
@@ -1211,13 +1221,13 @@ struct batch_request *cpy_checkpoint(
   int                   direction)
 
   {
-  char        momfile[MAXPATHLEN+1];
-  char        serverfile[MAXPATHLEN+1];
-  char       *from = NULL;
-  char       *to = NULL;
-  char        log_buf[LOCAL_LOG_BUF_SIZE];
-  attribute  *pattr;
-  mode_t      saveumask = 0;
+  char            momfile[MAXPATHLEN+1];
+  char            serverfile[MAXPATHLEN+1];
+  char           *from = NULL;
+  char           *to = NULL;
+  char            log_buf[LOCAL_LOG_BUF_SIZE];
+  pbs_attribute  *pattr;
+  mode_t          saveumask = 0;
   
   pattr = &pjob->ji_wattr[ati];
 
@@ -1362,8 +1372,6 @@ void remove_checkpoint(
   job **pjob_ptr)  /* I */
 
   {
-  static char          *id = "remove_checkpoint";
-
   struct batch_request *preq = NULL;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
   job                  *pjob = *pjob_ptr;
@@ -1377,7 +1385,7 @@ void remove_checkpoint(
       (*pjob_ptr)->ji_wattr[JOB_ATR_checkpoint_dir].at_val.at_str,
       (*pjob_ptr)->ji_wattr[JOB_ATR_checkpoint_name].at_val.at_str);
 
-    log_ext(-1, id, log_buf, LOG_DEBUG);
+    log_ext(-1, __func__, log_buf, LOG_DEBUG);
 
     /* change the request type from copy to delete  */
 
@@ -1423,7 +1431,7 @@ void cleanup_restart_file(
 
 /*    pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHECKPOINT_COPIED; */
 
-    /* clear restart_name attribute since purging job will clean it up */
+    /* clear restart_name pbs_attribute since purging job will clean it up */
 
     pjob->ji_wattr[JOB_ATR_restart_name].at_flags &= ~ATR_VFLAG_SET;
     pjob->ji_modified = 1;
@@ -1442,7 +1450,7 @@ int record_jobinfo(
   job *pjob)
 
   {
-  attribute              *pattr;
+  pbs_attribute          *pattr;
   int                     i;
   int                     rc;
   dynamic_string         *buffer;
@@ -1490,8 +1498,8 @@ int record_jobinfo(
       {
       if (!strcmp(job_attr_def[i].at_name, "depend"))
         {
-        /* we don't want this attribute in our log -
-           The dependecies will show on the submit_args attribute */
+        /* we don't want this pbs_attribute in our log -
+           The dependecies will show on the submit_args pbs_attribute */
         continue;
         }
       
@@ -1594,19 +1602,21 @@ int job_purge(
   extern char  *msg_err_purgejob;
   time_t        time_now = time(NULL);
   long          record_job_info = FALSE;
+  long          cray_enabled = FALSE;
   char          job_id[PBS_MAXSVRJOBID+1];
   char          job_fileprefix[PBS_JOBBASE+1];
   int           job_substate;
   int           job_is_array_template;
   unsigned int  job_has_checkpoint_file;
   int           job_has_arraystruct;
-
+ 
   strcpy(job_id, pjob->ji_qs.ji_jobid);
   strcpy(job_fileprefix, pjob->ji_qs.ji_fileprefix);
   job_substate = pjob->ji_qs.ji_substate;
   job_is_array_template = pjob->ji_is_array_template;
   job_has_arraystruct = ((pjob->ji_arraystruct == NULL)?FALSE:TRUE);
   job_has_checkpoint_file = pjob->ji_wattr[JOB_ATR_checkpoint_name].at_flags;
+
   /* check to see if we are keeping a log of all jobs completed */
   get_svr_attr_l(SRV_ATR_RecordJobInfo, &record_job_info);
   if (record_job_info)
@@ -1621,12 +1631,22 @@ int job_purge(
       check_job_log_started = 1;
       }
     }
-    
+  
+  get_svr_attr_l(SRV_ATR_CrayEnabled, &cray_enabled);
+  if ((cray_enabled == TRUE) &&
+      (pjob->ji_wattr[JOB_ATR_reservation_id].at_val.at_str != NULL))
+    remove_alps_reservation(pjob->ji_wattr[JOB_ATR_reservation_id].at_val.at_str);
+
   if ((job_has_arraystruct == FALSE) || (job_is_array_template == TRUE))
-    remove_job(&array_summary,pjob);
+    if (remove_job(&array_summary,pjob) == PBSE_JOB_RECYCLED)
+      {
+      /* PBSE_JOB_RECYCLED means the job is gone */
+      return(PBSE_NONE);
+      }
 
   /* if part of job array then remove from array's job list */
-  if ((job_has_arraystruct == TRUE) && (job_is_array_template == FALSE))
+  if ((job_has_arraystruct == TRUE) &&
+      (job_is_array_template == FALSE))
     {
     /* pa->ai_mutex will come out locked after 
        the call to get_jobs_array */
@@ -1654,7 +1674,7 @@ int job_purge(
         if (LOGLEVEL >=7)
           {
           sprintf(log_buf, "unlocked ai_mutex: %s", __func__);
-          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
           }
         }
       }
@@ -1662,22 +1682,34 @@ int job_purge(
       return PBSE_JOBNOTFOUND;
     }
 
+
   if ((job_substate != JOB_SUBSTATE_TRANSIN) &&
       (job_substate != JOB_SUBSTATE_TRANSICM))
     {
+    int need_deque = !pjob->ji_cold_restart;
+
     pthread_mutex_unlock(pjob->ji_mutex);
-    rc = svr_dequejob(job_id, FALSE);
+
+    /* jobs that are being deleted after a cold restart
+     * haven't been queued */
+    if (need_deque == TRUE)
+      rc = svr_dequejob(job_id, FALSE);
+
     if (rc != PBSE_JOBNOTFOUND)
       {
-      pthread_mutex_lock(pjob->ji_mutex);
-      job_free(pjob, TRUE);
+      /* we came out of svr_dequejob with pjob locked. Our pointer is still good */
+      /* job_free will unlock the mutex for us */
+      if (pjob->ji_being_recycled == FALSE)
+        job_free(pjob, TRUE);
+      else
+        pthread_mutex_unlock(pjob->ji_mutex);
       }
     }
   else
     {
     job_free(pjob, TRUE);
     }
-
+ 
   /* remove checkpoint restart file if there is one */
   /* MUTSU - The following functions, updates a flag, saves the job and returns.
    * As we are deleting the job (and file we are saving) anyway, this is 
@@ -1690,7 +1722,8 @@ int job_purge(
     */
 
   /* delete the script file */
-  if ((job_has_arraystruct == FALSE) || (job_is_array_template == TRUE))
+  if ((job_has_arraystruct == FALSE) || 
+      (job_is_array_template == TRUE))
     {
     /* delete script file */        
     snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, job_fileprefix, JOB_SCRIPT_SUFFIX);
@@ -1704,7 +1737,7 @@ int job_purge(
       {
       sprintf(log_buf, "removed job script");
 
-      log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
+      log_record(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, job_id, log_buf);
       }
     }
 
@@ -1720,7 +1753,7 @@ int job_purge(
     {
     sprintf(log_buf, "removed job stdout");
 
-    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
+    log_record(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, job_id, log_buf);
     }
 
   /* delete any spooled stderr */
@@ -1735,13 +1768,14 @@ int job_purge(
     {
     sprintf(log_buf, "removed job stderr");
 
-    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
+    log_record(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, job_id, log_buf);
     }
 
   /* delete checkpoint file directory if there is one */
   if (job_has_checkpoint_file & ATR_VFLAG_SET)
     {
-    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_checkpoint, job_fileprefix, JOB_CHECKPOINT_SUFFIX);
+    snprintf(namebuf, sizeof(namebuf), "%s%s%s", 
+      path_checkpoint, job_fileprefix, JOB_CHECKPOINT_SUFFIX);
 
     if (remtree(namebuf) < 0)
       {
@@ -1752,17 +1786,19 @@ int job_purge(
       {
       sprintf(log_buf, "removed job checkpoint");
 
-      log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
+      log_record(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, job_id, log_buf);
       }
     }
 
   if (job_is_array_template == TRUE)
     {
-    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, job_fileprefix, JOB_FILE_TMP_SUFFIX);
+    snprintf(namebuf, sizeof(namebuf), "%s%s%s", 
+      path_jobs, job_fileprefix, JOB_FILE_TMP_SUFFIX);
     }
   else
     {
-    snprintf(namebuf, sizeof(namebuf), "%s%s%s", path_jobs, job_fileprefix, JOB_FILE_SUFFIX);
+    snprintf(namebuf, sizeof(namebuf), "%s%s%s", 
+      path_jobs, job_fileprefix, JOB_FILE_SUFFIX);
     }
 
   if (unlink(namebuf) < 0)
@@ -1774,10 +1810,10 @@ int job_purge(
     {
     sprintf(log_buf, "removed job file");
 
-    log_record(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,job_id,log_buf);
+    log_record(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, job_id, log_buf);
     }
 
-  return PBSE_NONE;
+  return(PBSE_NONE);
   }  /* END job_purge() */
 
 
@@ -1808,7 +1844,6 @@ char *get_correct_jobname(
 
   int len;
 
-  char *id = "get_correct_jobname";
   long  display_suffix = TRUE;
   char *alias = NULL;
 
@@ -1843,7 +1878,7 @@ char *get_correct_jobname(
         correct = strdup(jobid);
 
         if (correct == NULL)
-          log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+          log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
 
         return(correct);
         }
@@ -1856,7 +1891,7 @@ char *get_correct_jobname(
 
       if (correct == NULL)
         {
-        log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+        log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
         return(NULL);
         }
 
@@ -1872,7 +1907,7 @@ char *get_correct_jobname(
 
       if (correct == NULL)
         {
-        log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+        log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
         return(NULL);
         }
 
@@ -1895,7 +1930,7 @@ char *get_correct_jobname(
 
       if (correct == NULL)
         {
-        log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+        log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
         return(NULL);
         }
 
@@ -1908,7 +1943,7 @@ char *get_correct_jobname(
 
       if (correct == NULL)
         {
-        log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+        log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
         return(NULL);
         }
       }
@@ -1920,7 +1955,7 @@ char *get_correct_jobname(
 
       if (correct == NULL)
         {
-        log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+        log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
         return(NULL);
         }
 
@@ -1940,7 +1975,7 @@ char *get_correct_jobname(
 
       if (correct == NULL)
         {
-        log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+        log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
         return(NULL);
         }
 
@@ -1958,7 +1993,7 @@ char *get_correct_jobname(
 
       if (correct == NULL)
         {
-        log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+        log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
         return(NULL);
         }
 
@@ -1983,7 +2018,7 @@ char *get_correct_jobname(
 
     if (correct == NULL)
       {
-      log_err(-1,id,"ERROR:    Fatal - Cannot allocate memory\n");
+      log_err(-1, __func__, "ERROR:    Fatal - Cannot allocate memory\n");
       return(NULL);
       }
 
@@ -2001,9 +2036,10 @@ char *get_correct_jobname(
  */
 
 job *find_job_by_array(
-    struct all_jobs *aj,
-    char *job_id,
-    int need_recycled)
+    
+  struct all_jobs *aj,
+  char *job_id)
+
   {
   job *pj = NULL;
   int  i;
@@ -2021,7 +2057,7 @@ job *find_job_by_array(
   
   if (pj != NULL)
     {
-    if ((pj->ji_being_recycled == TRUE) && (need_recycled == FALSE))
+    if (pj->ji_being_recycled == TRUE)
       {
       pthread_mutex_unlock(pj->ji_mutex);
       pj = NULL;
@@ -2029,7 +2065,10 @@ job *find_job_by_array(
     }
 
   return(pj);
-  }
+  } /* END find_job_by_array() */
+
+
+
 
 /*
  * find_job() - find job by jobid
@@ -2068,7 +2107,7 @@ job *find_job(
 
   if (strstr(jobid,"[]") == NULL)
     {
-    pj = find_job_by_array(&alljobs, comp, FALSE);
+    pj = find_job_by_array(&alljobs, comp);
     }
 
   /* when remotely routing jobs, they are removed from the 
@@ -2076,7 +2115,7 @@ job *find_job(
    * Attempt to find them there if NULL
    * OR it's an array, try to find the job */
   if (pj == NULL)
-    pj = find_job_by_array(&array_summary, comp, FALSE);
+    pj = find_job_by_array(&array_summary, comp);
 
   if (at)
     *at = '@'; /* restore @server_name */
@@ -2227,7 +2266,6 @@ int insert_job_first(
   job             *pjob)
 
   {
-  static char *id = "insert_job_first";
   int          rc;
 
   pthread_mutex_lock(aj->alljobs_mutex);
@@ -2235,7 +2273,7 @@ int insert_job_first(
   if ((rc = insert_thing_after(aj->ra,pjob,ALWAYS_EMPTY_INDEX)) == -1)
     {
     rc = ENOMEM;
-    log_err(rc,id,"No memory to resize the array...SYSTEM FAILURE");
+    log_err(rc, __func__, "No memory to resize the array...SYSTEM FAILURE");
     }
   else
     {
@@ -2268,6 +2306,13 @@ int get_jobs_index(
     pthread_mutex_unlock(pjob->ji_mutex);
     pthread_mutex_lock(aj->alljobs_mutex);
     pthread_mutex_lock(pjob->ji_mutex);
+
+    if (pjob->ji_being_recycled == TRUE)
+      {
+      pthread_mutex_unlock(aj->alljobs_mutex);
+      pthread_mutex_unlock(pjob->ji_mutex);
+      return(-1);
+      }
     }
 
   index = get_value_hash(aj->ht, pjob->ji_qs.ji_jobid);
@@ -2289,11 +2334,27 @@ int has_job(
   job             *pjob)
 
   {
-  int rc;
+  int  rc;
+  char jobid[PBS_MAXSVRJOBID + 1];
 
-  pthread_mutex_lock(aj->alljobs_mutex);
+  strcpy(jobid, pjob->ji_qs.ji_jobid);
 
-  if (get_value_hash(aj->ht,pjob->ji_qs.ji_jobid) < 0)
+  if (pthread_mutex_trylock(aj->alljobs_mutex))
+    {
+    pthread_mutex_unlock(pjob->ji_mutex);
+    pthread_mutex_lock(aj->alljobs_mutex);
+    pthread_mutex_lock(pjob->ji_mutex);
+
+    if (pjob->ji_being_recycled == TRUE)
+      {
+      pthread_mutex_unlock(aj->alljobs_mutex);
+      pthread_mutex_unlock(pjob->ji_mutex);
+
+      return(PBSE_JOB_RECYCLED);
+      }
+    }
+
+  if (get_value_hash(aj->ht, pjob->ji_qs.ji_jobid) < 0)
     rc = FALSE;
   else
     rc = TRUE;
@@ -2328,6 +2389,13 @@ int  remove_job(
     pthread_mutex_unlock(pjob->ji_mutex);
     pthread_mutex_lock(aj->alljobs_mutex);
     pthread_mutex_lock(pjob->ji_mutex);
+
+    if (pjob->ji_being_recycled == TRUE)
+      {
+      pthread_mutex_unlock(aj->alljobs_mutex);
+      pthread_mutex_unlock(pjob->ji_mutex);
+      return(PBSE_JOB_RECYCLED);
+      }
     }
 
   if ((index = get_value_hash(aj->ht,pjob->ji_qs.ji_jobid)) < 0)
@@ -2354,6 +2422,8 @@ job *next_job(
 
   {
   job *pjob;
+  char log_buf[LOCAL_LOG_BUF_SIZE];
+  int  last_index = *iter;
 
   pthread_mutex_lock(aj->alljobs_mutex);
 
@@ -2367,6 +2437,13 @@ job *next_job(
 
     if (pjob->ji_being_recycled == TRUE)
       {
+      if (LOGLEVEL >= 7)
+        {
+        snprintf(log_buf, sizeof(log_buf),
+          "job at index %d is being recycled, going to the next job", last_index);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
+        }
+
       pthread_mutex_unlock(pjob->ji_mutex);
 
       pjob = next_job(aj,iter);
