@@ -208,14 +208,14 @@ int initialize_procct(
   job *pjob)
 
   { 
-  resource     *pnodesp = NULL;
-  resource_def *pnodes_def = NULL;
-  resource     *pprocsp = NULL;
-  resource_def *pprocs_def = NULL;
-  resource     *procctp = NULL;
-  resource_def *procct_def = NULL;
-  attribute    *pattr = NULL;
-  char          log_buf[LOCAL_LOG_BUF_SIZE];
+  resource      *pnodesp = NULL;
+  resource_def  *pnodes_def = NULL;
+  resource      *pprocsp = NULL;
+  resource_def  *pprocs_def = NULL;
+  resource      *procctp = NULL;
+  resource_def  *procct_def = NULL;
+  pbs_attribute *pattr = NULL;
+  char           log_buf[LOCAL_LOG_BUF_SIZE];
 
   pattr = &pjob->ji_wattr[JOB_ATR_resource];
   if (pattr == NULL)
@@ -224,6 +224,7 @@ int initialize_procct(
        by the time this function is called */
     sprintf(log_buf, "%s: Resource_List is NULL. Cannot proceed", __func__);
     log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+
     return(ROUTE_PERM_FAILURE);
     }
 
@@ -238,9 +239,10 @@ int initialize_procct(
       log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
       return(ROUTE_PERM_FAILURE);
       }
+
     pnodesp = find_resc_entry(pattr, pnodes_def);
 
-    /* Get the procs count if the procs resource attribute is set */
+    /* Get the procs count if the procs resource pbs_attribute is set */
     pprocs_def = find_resc_def(svr_resc_def, "procs", svr_resc_size);
     if (pprocs_def != NULL)
       {
@@ -260,6 +262,7 @@ int initialize_procct(
         log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
         return(ROUTE_PERM_FAILURE);
         }
+
       procctp = find_resc_entry(pattr, procct_def);
       if (procctp == NULL)
         {
@@ -269,7 +272,7 @@ int initialize_procct(
         }
       }
 
-    /* we now set pjob->procct and we also set the resource attribute procct */
+    /* we now set pjob->procct and we also set the resource pbs_attribute procct */
     procct_def = find_resc_def(svr_resc_def, "procct", svr_resc_size);
     if (procct_def == NULL)
       {
@@ -301,6 +304,7 @@ int initialize_procct(
       {
       procctp->rs_value.at_val.at_long += pprocsp->rs_value.at_val.at_long;
       }
+
     procctp->rs_value.at_flags |= ATR_VFLAG_SET;
     }
   else
@@ -321,10 +325,10 @@ int remove_procct(
   job *pjob)
 
   {
-  attribute    *pattr;
-  resource_def *pctdef;
-  resource     *pctresc;
-  char          log_buf[LOCAL_LOG_BUF_SIZE];
+  pbs_attribute *pattr;
+  resource_def  *pctdef;
+  resource      *pctresc;
+  char           log_buf[LOCAL_LOG_BUF_SIZE];
 
   pattr = &pjob->ji_wattr[JOB_ATR_resource];
   if (pattr == NULL)
@@ -336,11 +340,11 @@ int remove_procct(
     return(ROUTE_PERM_FAILURE);
     }
 
- /* unset the procct resource if it has been set */
-    pctdef = find_resc_def(svr_resc_def, "procct", svr_resc_size);
-
-    if ((pctresc = find_resc_entry(pattr, pctdef)) != NULL)
-      pctdef->rs_free(&pctresc->rs_value);
+  /* unset the procct resource if it has been set */
+  pctdef = find_resc_def(svr_resc_def, "procct", svr_resc_size);
+  
+  if ((pctresc = find_resc_entry(pattr, pctdef)) != NULL)
+    pctdef->rs_free(&pctresc->rs_value);
 
   return(PBSE_NONE);
   } /* END remove_procct */
@@ -417,7 +421,7 @@ int default_router(
       continue;
 
     /* We need to manage the procct resource which is 
-       part of the Resource_List attribute. At this point 
+       part of the Resource_List pbs_attribute. At this point 
        we need to remember what the procct value is and also
 	     make sure it is in the Resource_List before calling
 	     svr_movejob. See ROUTE_RETRY to see what is done
@@ -433,11 +437,11 @@ int default_router(
 
       case ROUTE_PERM_FAILURE: /* permanent failure */
 
-        rc = remove_procct(jobp);
-        if(rc != PBSE_NONE)
+        if ((rc = remove_procct(jobp)) != PBSE_NONE)
           {
           return(rc);
           }
+
         add_dest(jobp);
 
         break;
