@@ -123,7 +123,10 @@
 #include "queue_recycler.h" /* queue_recycler */
 #include "svr_task.h" /* initialize_task_recycler */
 #include "svr_func.h" /* get_svr_attr_* */
+#include "login_nodes.h"
+#include "track_alps_reservations.h"
 #include "job_func.h" /* job_purge */
+#include "net_cache.h"
 
 /*#ifndef SIGKILL*/
 /* there is some weird stuff in gcc include files signal.h & sys/params.h */
@@ -202,6 +205,9 @@ dynamic_string         *hierarchy_holder;
 hello_container         hellos;
 hello_container         failures;
 
+reservation_holder      alps_reservations;
+batch_request_holder    brh;
+
 extern pthread_mutex_t *acctfile_mutex;
 pthread_mutex_t        *scheduler_sock_jobct_mutex;
 extern int              scheduler_sock;
@@ -210,7 +216,6 @@ extern pthread_mutex_t *svr_do_schedule_mutex;
 extern pthread_mutex_t *listener_command_mutex;
 extern pthread_mutex_t *node_state_mutex;
 extern pthread_mutex_t *check_tasks_mutex;
-extern pthread_mutex_t *retry_routing_mutex;
 
 extern int a_opt_init;
 
@@ -234,7 +239,6 @@ void          on_job_exit(struct work_task *);
 
 void  init_abt_job(job *);
 char *build_path(char *, char *, char *);
-void  catch_child(int);
 void  catch_abort(int);
 void  change_logs(int);
 void  change_log_level(int);
@@ -332,16 +336,13 @@ int DArrayAppend(
       Array->AppendIndex = 0;
       return(FAILURE);
       }
+
     memcpy(tmp, Array->Data, sizeof(Array->Data[0]) * Array->Length);
     free(Array->Data);
     Array->Data = tmp;
     Array->Length = newLength;
     }
 
-  /*
-  assert(Array->AppendIndex >= 0);
-  assert(Array->AppendIndex < Array->Length);
-   */
   Array->Data[Array->AppendIndex++] = Item;
   return(SUCCESS);
   } /* END DArrayAppend */
@@ -398,19 +399,19 @@ void  update_default_np()
 void add_server_names_to_acl_hosts(void)
 
   {
-  int        n; 
-  int        list_len; 
-  int        rc;
+  int            n; 
+  int            list_len; 
+  int            rc;
 
-  char      *server_list_ptr;
-  char      *tp;
-  char       buffer[PBS_MAXSERVERNAME+1];
-  attribute  temp;
+  char          *server_list_ptr;
+  char          *tp;
+  char           buffer[PBS_MAXSERVERNAME+1];
+  pbs_attribute  temp;
 
-  attribute *patr = &server.sv_attr[SRV_ATR_acl_hosts];
+  pbs_attribute *patr = &server.sv_attr[SRV_ATR_acl_hosts];
 
   memset(buffer, 0, PBS_MAXSERVERNAME+1);
-  memset(&temp, 0, sizeof(attribute));
+  memset(&temp, 0, sizeof(pbs_attribute));
 
   server_list_ptr = pbs_get_server_list();
   list_len = csv_length(server_list_ptr);
@@ -475,7 +476,7 @@ dynamic_string *make_default_hierarchy()
       snprintf(buf, sizeof(buf), ":%d", (int)pnode->nd_mom_rm_port);
       append_dynamic_string(level_ds, buf);
       }
-    
+
     pnode->nd_hierarchy_level = 0;
 
     unlock_node(pnode, __func__, NULL, LOGLEVEL);
@@ -506,9 +507,13 @@ int can_resolve_hostname(
   if ((colon = strchr(hostname, ':')) != NULL)
     *colon = '\0';
 
-  if (getaddrinfo(hostname, NULL, NULL, &addr_info) == 0)
-    {
+  if (get_cached_addrinfo(hostname) != NULL)
     can_resolve = TRUE;
+  else if (getaddrinfo(hostname, NULL, NULL, &addr_info) == 0)
+    {
+    struct sockaddr_in *sai = (struct sockaddr_in *)addr_info->ai_addr;
+    can_resolve = TRUE;
+    insert_addr_name_info(hostname, addr_info->ai_canonname, sai);
     freeaddrinfo(addr_info);
     }
 
@@ -528,11 +533,12 @@ void check_if_in_nodes_file(
   int   level_index)
 
   {
-  char             log_buf[LOCAL_LOG_BUF_SIZE];
-  struct pbsnode  *pnode;
-  char            *colon;
-  struct addrinfo *addr_info;
-  unsigned long    ipaddr;
+  char                log_buf[LOCAL_LOG_BUF_SIZE];
+  struct pbsnode     *pnode;
+  char               *colon;
+  struct addrinfo    *addr_info;
+  struct sockaddr_in *sai;
+  unsigned long       ipaddr;
 
   if ((colon = strchr(hostname, ':')) != NULL)
     *colon = '\0';
@@ -544,8 +550,19 @@ void check_if_in_nodes_file(
       hostname);
     log_err(-1, __func__, log_buf);
 
-    getaddrinfo(hostname, NULL, NULL, &addr_info);
-    ipaddr = ntohl(((struct sockaddr_in *)addr_info->ai_addr)->sin_addr.s_addr);
+    if ((sai = get_cached_addrinfo(hostname)) == NULL)
+      {
+      getaddrinfo(hostname, NULL, NULL, &addr_info);
+      sai = (struct sockaddr_in *)addr_info->ai_addr;
+      ipaddr = ntohl(sai->sin_addr.s_addr);
+
+      insert_addr_name_info(hostname, addr_info->ai_canonname, sai);
+
+      freeaddrinfo(addr_info);
+      }
+    else
+      ipaddr = ntohl(sai->sin_addr.s_addr);
+
     create_partial_pbs_node(hostname, ipaddr, ATR_DFLAG_MGRD | ATR_DFLAG_MGWR);
     pnode = find_nodebyname(hostname);
     }
@@ -903,84 +920,14 @@ int get_default_threads()
 
 
 
-
-/*
- * This file contains the functions to initialize the PBS Batch Server.
- * The code is called once when the server is brought up.
- */
-
-int pbsd_init(
-
-  int type)  /* type of initialization   */
+int setup_limits()
 
   {
-  int               baselen = 0;
-  char              basen[MAXPATHLEN+1];
-
-  struct dirent    *pdirent;
-  DIR              *dir;
-  int               fd;
-  int               had;
-  int               i;
-  char             *job_suffix = JOB_FILE_SUFFIX;
-  int               job_suf_len = strlen(job_suffix);
-  int               array_suf_len = strlen(ARRAY_FILE_SUFFIX);
-  int               logtype;
-  char             *new_tag = ".new";
-  job              *pjob = NULL;
-  pbs_queue        *pque = NULL;
-  char             *psuffix;
-  int               rc;
-  int               Index;
-  int               iter;
-  long              min_threads;
-  long              max_threads;
-  long              thread_idle_time = DEFAULT_THREAD_IDLE;
-  int               job_count = 0; /* Count of recovered jobs */
-
-  struct stat       statbuf;
-  char             *suffix_slash = "/";
-
-  struct sigaction  act;
-
-  struct sigaction  oact;
-
-  job_array        *pa;
-  char              log_buf[LOCAL_LOG_BUF_SIZE];
-  time_t            time_now = time(NULL);
-
-#if !defined(DEBUG) && !defined(NO_SECURITY_CHECK)
-  char   EMsg[1024];
-#endif /* not DEBUG and not NO_SECURITY_CHECK */
-
-  min_threads = get_default_threads();
-  max_threads = min_threads;
-
-  memset(&hints, 0, sizeof(hints));
-  hints.ai_flags = AI_CANONNAME;
-
-  /* The following is code to reduce security risks */
-
-  if (setup_env(PBS_ENVIRON) == -1)
-    {
-    return(-1);
-    }
-
-  i = getgid();
-
-  /* secure suppl. groups */
-  if (setgroups(1,(gid_t *)&i) != 0)
-    {
-    snprintf(log_buf, sizeof(log_buf),
-      "Unable to drop secondary groups. Some MAC framework is active?\n");
-    log_err(errno, __func__, log_buf);
-    snprintf(log_buf, sizeof(log_buf),
-      "setgroups(group = %lu) failed: %s\n",
-      (unsigned long)i, strerror(errno));
-    log_err(errno, __func__, log_buf);
-
-    return(-1);
-    }
+#ifndef DEBUG
+#ifndef _CRAY
+  struct rlimit rlimit;
+#endif
+#endif
 
 #ifndef DEBUG
 #ifdef _CRAY
@@ -993,34 +940,32 @@ int pbsd_init(
   limit(C_JOB,      0, L_MEM  , 0);
   limit(C_JOBPROCS, 0, L_MEM  , 0);
 #else /* not  _CRAY */
-    {
-
-    struct rlimit rlimit;
-
-    rlimit.rlim_cur = RLIM_INFINITY;
-    rlimit.rlim_max = RLIM_INFINITY;
-    setrlimit(RLIMIT_CPU,   &rlimit);
-    setrlimit(RLIMIT_FSIZE, &rlimit);
-    setrlimit(RLIMIT_DATA,  &rlimit);
-    setrlimit(RLIMIT_STACK, &rlimit);
+  rlimit.rlim_cur = RLIM_INFINITY;
+  rlimit.rlim_max = RLIM_INFINITY;
+  setrlimit(RLIMIT_CPU,   &rlimit);
+  setrlimit(RLIMIT_FSIZE, &rlimit);
+  setrlimit(RLIMIT_DATA,  &rlimit);
+  setrlimit(RLIMIT_STACK, &rlimit);
 #ifdef RLIMIT_RSS
-    setrlimit(RLIMIT_RSS,   &rlimit);
+  setrlimit(RLIMIT_RSS,   &rlimit);
 #endif /* RLIMIT_RSS */
 #ifdef RLIMIT_VMEM
-    setrlimit(RLIMIT_VMEM,  &rlimit);
+  setrlimit(RLIMIT_VMEM,  &rlimit);
 #endif /* RLIMIT_VMEM */
-    }
 #endif /* not _CRAY */
 #endif /* DEBUG */
   
-  /* setup the threadpool for use */
-  get_svr_attr_l(SRV_ATR_minthreads, &min_threads);
-  get_svr_attr_l(SRV_ATR_maxthreads, &max_threads);
-  get_svr_attr_l(SRV_ATR_threadidleseconds, &thread_idle_time);
-  
-  initialize_threadpool(&request_pool,min_threads,max_threads,thread_idle_time);
+  return(PBSE_NONE);
+  } /* END setup_limits() */
 
-  /* 1. set up to catch or ignore various signals */
+
+
+
+int setup_signal_handling()
+
+  {
+  struct sigaction  act;
+  struct sigaction  oact;
 
   sigemptyset(&act.sa_mask);
 
@@ -1095,14 +1040,8 @@ int pbsd_init(
     sigaction(SIGSYS,  &act, NULL);
     }
 
-#ifdef NO_SIGCHLD
   act.sa_handler = SIG_DFL;
 
-#else
-  act.sa_handler = catch_child;
-
-#endif
-
   if (sigaction(SIGCHLD, &act, &oact) != 0)
     {
     log_err(errno, __func__, "sigaction for CHLD");
@@ -1135,43 +1074,53 @@ int pbsd_init(
     return(2);
     }
 
-  /* 2. set up the various paths and other global variables we need */
+  return(PBSE_NONE);
+  } /* END setup_signal_handling() */
+
+
+
+
+int initialize_paths()
+
+  {
+  int          rc = PBSE_NONE;
+  char        *suffix_slash = "/";
+  char        *new_tag = ".new";
+  struct stat  statbuf;
+  char         log_buf[LOCAL_LOG_BUF_SIZE];
+#if !defined(DEBUG) && !defined(NO_SECURITY_CHECK)
+  char         EMsg[1024];
+#endif /* not DEBUG and not NO_SECURITY_CHECK */
 
   if (path_priv == NULL)
-    {
-    path_priv      = build_path(path_home, PBS_SVR_PRIVATE, suffix_slash);
-    }
+    path_priv        = build_path(path_home, PBS_SVR_PRIVATE, suffix_slash);
 
-  path_arrays    = build_path(path_priv, PBS_ARRAYDIR, suffix_slash);
-  path_spool     = build_path(path_home, PBS_SPOOLDIR, suffix_slash);
-  path_queues    = build_path(path_priv, PBS_QUEDIR,   suffix_slash);
-  path_jobs      = build_path(path_priv, PBS_JOBDIR,   suffix_slash);
-  path_credentials = build_path(path_priv, PBS_CREDENTIALDIR, suffix_slash);
-  path_acct  = build_path(path_priv, PBS_ACCT,     suffix_slash);
+  path_arrays        = build_path(path_priv, PBS_ARRAYDIR, suffix_slash);
+  path_spool         = build_path(path_home, PBS_SPOOLDIR, suffix_slash);
+  path_queues        = build_path(path_priv, PBS_QUEDIR,   suffix_slash);
+  path_jobs          = build_path(path_priv, PBS_JOBDIR,   suffix_slash);
+  path_credentials   = build_path(path_priv, PBS_CREDENTIALDIR, suffix_slash);
+  path_acct          = build_path(path_priv, PBS_ACCT,     suffix_slash);
 
   if (path_svrdb == NULL)
-    {
-    path_svrdb     = build_path(path_priv, PBS_SERVERDB, NULL);
-    }
+    path_svrdb       = build_path(path_priv, PBS_SERVERDB, NULL);
 
-  path_svrdb_new = build_path(path_priv, PBS_SERVERDB, new_tag);
-  path_svrlog = build_path(path_home, PBS_LOGFILES, suffix_slash);
-  path_jobinfo_log = build_path(path_home, PBS_JOBINFOLOGDIR, suffix_slash);
-  path_track  = build_path(path_priv, PBS_TRACKING, NULL);
-  path_nodes  = build_path(path_priv, NODE_DESCRIP, NULL);
-  path_nodes_new = build_path(path_priv, NODE_DESCRIP, new_tag);
-  path_nodestate = build_path(path_priv, NODE_STATUS,  NULL);
-  path_nodenote  = build_path(path_priv, NODE_NOTE,    NULL);
-  path_nodenote_new = build_path(path_priv, NODE_NOTE, new_tag);
+  path_svrdb_new     = build_path(path_priv, PBS_SERVERDB, new_tag);
+  path_svrlog        = build_path(path_home, PBS_LOGFILES, suffix_slash);
+  path_jobinfo_log   = build_path(path_home, PBS_JOBINFOLOGDIR, suffix_slash);
+  path_track         = build_path(path_priv, PBS_TRACKING, NULL);
+  path_nodes         = build_path(path_priv, NODE_DESCRIP, NULL);
+  path_nodes_new     = build_path(path_priv, NODE_DESCRIP, new_tag);
+  path_nodestate     = build_path(path_priv, NODE_STATUS,  NULL);
+  path_nodenote      = build_path(path_priv, NODE_NOTE,    NULL);
+  path_nodenote_new  = build_path(path_priv, NODE_NOTE, new_tag);
   path_mom_hierarchy = build_path(path_priv, PBS_MOM_HIERARCHY, NULL);
 
-
 #ifdef SERVER_CHKPTDIR
   /* need to make sure path ends with a '/' */
-
   if (*(SERVER_CHKPTDIR + strlen(SERVER_CHKPTDIR) - 1)  == '/')
     {
-    path_checkpoint = strdup(SERVER_CHKPTDIR);
+    path_checkpoint  = strdup(SERVER_CHKPTDIR);
     }
   else
     {
@@ -1181,7 +1130,7 @@ int pbsd_init(
     }
 
 #else
-  path_checkpoint     = build_path(path_home, PBS_CHKPTDIR, suffix_slash);
+  path_checkpoint    = build_path(path_home, PBS_CHKPTDIR, suffix_slash);
 #endif
 
   /* check existance amd make sure it is a directory */
@@ -1217,8 +1166,7 @@ int pbsd_init(
 
   if (svr_resc_def == NULL)
     {
-    rc = init_resc_defs();
-    if (rc != 0)
+    if ((rc = init_resc_defs()) != PBSE_NONE)
       {
       log_err(rc, __func__, msg_init_baddb);
       
@@ -1235,22 +1183,28 @@ int pbsd_init(
   rc |= chk_file_sec(path_credentials,  1, 0, S_IWGRP | S_IWOTH, 0, EMsg);
   rc |= chk_file_sec(PBS_ENVIRON, 0, 0, S_IWGRP | S_IWOTH, 1, EMsg);
 
-  if (rc != 0)
+  if (rc != PBSE_NONE)
     {
     return(3);
     }
-
 #endif /* not DEBUG and not NO_SECURITY_CHECK */
 
+  return(rc);
+  } /* END initialize_paths() */
+
+
+
+int initialize_data_structures_and_mutexes()
+
+  {
+  long cray_enabled = FALSE;
+
   svr_do_schedule_mutex = calloc(1, sizeof(pthread_mutex_t));
   pthread_mutex_init(svr_do_schedule_mutex, NULL);
 
   check_tasks_mutex = calloc(1, sizeof(pthread_mutex_t));
   pthread_mutex_init(check_tasks_mutex, NULL);
 
-  retry_routing_mutex = calloc(1, sizeof(pthread_mutex_t));
-  pthread_mutex_init(retry_routing_mutex, NULL);
-
   listener_command_mutex = calloc(1, sizeof(pthread_mutex_t));
   pthread_mutex_init(listener_command_mutex, NULL);
 
@@ -1270,6 +1224,7 @@ int pbsd_init(
    * they can be called by a signal handler */
 
   initialize_recycler();
+  initialize_batch_request_holder();
 
   initialize_all_tasks_array(&task_list_timed);
   initialize_all_tasks_array(&task_list_event);
@@ -1288,11 +1243,31 @@ int pbsd_init(
 
   initialize_allques_array(&svr_queues);
 
-  time_now = time((time_t *)0);
+  get_svr_attr_l(SRV_ATR_CrayEnabled, &cray_enabled);
+  if (cray_enabled == TRUE)
+    {
+    initialize_login_holder();
+    initialize_alps_reservations();
+    }
+  
+  acctfile_mutex = calloc(1, sizeof(pthread_mutex_t));
+  pthread_mutex_init(acctfile_mutex, NULL);
+
+  return(PBSE_NONE);
+  } /* END initialize_data_structures_and_mutexes() */
+
+
+
+int setup_server_attrs(
+    
+  int type)
+
+  {
+  int i;
+  int rc = PBSE_NONE;
 
-  /* 3. Set default server attibutes values */
   pthread_mutex_lock(server.sv_attr_mutex);
-  for (i = 0;i < SRV_ATR_LAST;i++)
+  for (i = 0; i < SRV_ATR_LAST; i++)
     clear_attr(&server.sv_attr[i], &svr_attr_def[i]);
 
   server.sv_attr[SRV_ATR_scheduler_iteration].at_val.at_long =  PBS_SCHEDULE_CYCLE;
@@ -1326,20 +1301,17 @@ int pbsd_init(
   server.sv_attr[SRV_ATR_MoabArrayCompatible].at_val.at_long = TRUE;
   server.sv_attr[SRV_ATR_MoabArrayCompatible].at_flags = ATR_VFLAG_SET;
 
-  /* 4. force logging of all types */
-
+  /* force logging of all types */
   server.sv_attr[SRV_ATR_log_events].at_val.at_long = PBSEVENT_MASK;
   server.sv_attr[SRV_ATR_log_events].at_flags = ATR_VFLAG_SET;
 
-  /* 5. If not a "create" initialization, recover server db */
-
+  /* If not a "create" initialization, recover server db */
   rc = chk_save_file(path_svrdb);
 
   if (type != RECOV_CREATE)
     {
     /* Open the server database (save file) and read it in */
-
-    if ((rc != 0) || ((rc = svr_recov_xml(path_svrdb, FALSE)) == -1)) 
+    if ((rc != PBSE_NONE) || ((rc = svr_recov_xml(path_svrdb, FALSE)) == -1)) 
       {
       log_err(rc, __func__, msg_init_baddb);
 
@@ -1354,19 +1326,9 @@ int pbsd_init(
     }
   else
     {
-    if (rc == 0)
+    if (rc == PBSE_NONE)
       {
       /* path_svrdb exists */
-
-      /* will exit on (n)o response */
-
-/*      if (TForceUpdate == 0)
-        {
-        need_y_response(type);
-        }*/
-
-      /* (y)es response received */
-
       rm_files(path_priv);
 
       pthread_mutex_unlock(server.sv_attr_mutex);
@@ -1375,18 +1337,16 @@ int pbsd_init(
       }
     }
 
-  svr_attr_def[SRV_ATR_version].at_decode(
+  rc = PBSE_NONE;
 
+  svr_attr_def[SRV_ATR_version].at_decode(
     &server.sv_attr[SRV_ATR_version],
     0,
     0,
     PACKAGE_VERSION,
     0);
 
-  /* 6. open accounting file and job log file if logging is set */
-  acctfile_mutex = calloc(1, sizeof(pthread_mutex_t));
-  pthread_mutex_init(acctfile_mutex, NULL);
-
+  /* open accounting file and job log file if logging is set */
   if (acct_open(acct_file) != 0)
     {
     pthread_mutex_unlock(server.sv_attr_mutex);
@@ -1397,7 +1357,7 @@ int pbsd_init(
     {
     rc = job_log_open(job_log_file, path_jobinfo_log);
 
-    if (rc != 0)
+    if (rc != PBSE_NONE)
       {
       fprintf(stderr, "Could not open job_logs \n");
       pthread_mutex_unlock(server.sv_attr_mutex);
@@ -1405,23 +1365,27 @@ int pbsd_init(
       }
     }
 
-
-  /* 7. Set up other server and global variables */
-
+  /* set up other server and global variables */
   if (a_opt_init != -1)
     {
     /* a_option was set, overrides saved value of scheduling attr */
-
     server.sv_attr[SRV_ATR_scheduling].at_val.at_long = a_opt_init;
     server.sv_attr[SRV_ATR_scheduling].at_flags |= ATR_VFLAG_SET;
     }
       
   pthread_mutex_unlock(server.sv_attr_mutex);
 
-  /* Open and read in node list if one exists */
+  return(rc);
+  } /* END setup_server_attrs() */
+
+
+
+int initialize_nodes()
+
+  {
   initialize_all_nodes_array(&allnodes);
 
-  if ((rc = setup_nodes()) == -1)
+  if (setup_nodes() == -1)
     {
     return(-1);
     }
@@ -1429,10 +1393,24 @@ int pbsd_init(
   add_server_names_to_acl_hosts();
   update_default_np();
 
-  /*
-   * 8. If not a "create" initialization, recover queues.
-   *    If a create, remove any queues that might be there.
-   */
+  return(PBSE_NONE);
+  } /* END initialize_nodes() */
+
+
+
+
+int handle_queue_recovery(
+    
+  int type)
+
+  {
+  int               rc = PBSE_NONE;
+  struct dirent    *pdirent;
+  DIR              *dir;
+  int               had;
+  pbs_queue        *pque = NULL;
+  char              log_buf[LOCAL_LOG_BUF_SIZE];
+  int               logtype;
 
   if (chdir(path_queues) != 0)
     {
@@ -1443,10 +1421,7 @@ int pbsd_init(
     return(-1);
     }
 
-  pthread_mutex_lock(server.sv_qs_mutex);
-
   had = server.sv_qs.sv_numque;
-
   server.sv_qs.sv_numque = 0;
 
   dir = opendir(".");
@@ -1455,7 +1430,8 @@ int pbsd_init(
     {
     log_err(-1, __func__, msg_init_noqueues);
 
-    pthread_mutex_unlock(server.sv_qs_mutex);
+    sprintf(log_buf, "%s:1", __func__);
+    unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
 
     return(-1);
     }
@@ -1465,18 +1441,15 @@ int pbsd_init(
     if (pdirent->d_name == NULL)
       {
       /* invalid name returned */
-
       continue;
       }
 
     if (chk_save_file(pdirent->d_name) == 0)
       {
       /* recover queue */
-
       if ((pque = que_recov_xml(pdirent->d_name)) != NULL)
         {
         /* que_recov increments sv_numque */
-
         sprintf(log_buf, msg_init_recovque, pque->qu_qs.qu_name);
 
         log_event(
@@ -1487,8 +1460,7 @@ int pbsd_init(
 
         if (pque->qu_attr[QE_ATR_ResourceAssn].at_flags & ATR_VFLAG_SET)
           {
-          que_attr_def[QE_ATR_ResourceAssn].at_free(
-            &pque->qu_attr[QE_ATR_ResourceAssn]);
+          que_attr_def[QE_ATR_ResourceAssn].at_free(&pque->qu_attr[QE_ATR_ResourceAssn]);
           }
 
         unlock_queue(pque, __func__, NULL, LOGLEVEL);
@@ -1507,12 +1479,25 @@ int pbsd_init(
 
   log_event(logtype, PBS_EVENTCLASS_SERVER, msg_daemonname, log_buf);
 
-  /*
-   * 9. If not "create" or "clean" recovery, recover the jobs.
-   *    If a a create or clean recovery, delete any jobs.
-   */
+  return(rc);
+  } /* END handle_queue_recovery() */
 
-  /* 9.a, recover job array info */
+
+
+
+int handle_array_recovery(
+    
+  int type)
+
+  {
+  char              log_buf[LOCAL_LOG_BUF_SIZE];
+  struct dirent    *pdirent;
+  DIR              *dir;
+  int               rc = PBSE_NONE;
+  job_array        *pa = NULL;
+  int               baselen = 0;
+  int               array_suf_len = strlen(ARRAY_FILE_SUFFIX);
+  char             *psuffix;
 
   if (chdir(path_arrays) != 0)
     {
@@ -1520,7 +1505,8 @@ int pbsd_init(
 
     log_err(errno, __func__, log_buf);
 
-    pthread_mutex_unlock(server.sv_qs_mutex);
+    sprintf(log_buf, "%s:2", __func__);
+    unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
 
     return(-1);
     }
@@ -1529,14 +1515,13 @@ int pbsd_init(
 
   while ((pdirent = readdir(dir)) != NULL)
     {
-
-    if (chk_save_file(pdirent->d_name) == 0)
+    if (chk_save_file(pdirent->d_name) == PBSE_NONE)
       {
       /* if not create or clean recovery, recover arrays */
 
-      if ((type != RECOV_CREATE) && (type != RECOV_COLD))
+      if ((type != RECOV_CREATE) && 
+          (type != RECOV_COLD))
         {
-
         /* skip files without the proper suffix */
         baselen = strlen(pdirent->d_name) - array_suf_len;
 
@@ -1545,42 +1530,65 @@ int pbsd_init(
         if (strcmp(psuffix, ARRAY_FILE_SUFFIX))
           continue;
 
-        rc = array_recov(pdirent->d_name, &pa);
-
-        if (rc != PBSE_NONE)
+        if ((rc = array_recov(pdirent->d_name, &pa)) != PBSE_NONE)
           {
           sprintf(log_buf,
-            "could not recover array-struct from file %s--skipping. "
-            "job array can not be recovered.",
+            "could not recover array-struct from file %s--skipping. job array can not be recovered.",
             pdirent->d_name);
 
           log_err(errno, __func__, log_buf);
-          pthread_mutex_unlock(server.sv_qs_mutex);
+
+          sprintf(log_buf, "%s:3", __func__);
+          unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
+
           return(rc);
           }
 
         pa->jobs_recovered = 0;
 
         pthread_mutex_unlock(pa->ai_mutex);
+
         if (LOGLEVEL >= 7)
           {
           sprintf(log_buf, "%s: unlocking ai_mutex", __func__);
           log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
           }
-                    
         }
       else
         {
         unlink(pdirent->d_name);
         }
-
       }
-
     }
 
   closedir(dir);
 
-  /* 9.b,  recover jobs */
+  return(rc);
+  } /* handle_array_recovery() */
+
+
+
+
+int handle_job_recovery(
+
+  int type)
+
+  {
+  char              log_buf[LOCAL_LOG_BUF_SIZE];
+  struct dirent    *pdirent;
+  DIR              *dir;
+  int               had;
+  int               rc = PBSE_NONE;
+  job              *pjob;
+  int               logtype;
+  int               baselen = 0;
+  char             *psuffix;
+  int               job_count = 0; /* Count of recovered jobs */
+  char             *job_suffix = JOB_FILE_SUFFIX;
+  int               job_suf_len = strlen(job_suffix);
+  char              basen[MAXPATHLEN+1];
+  int               Index;
+  int               iter = -1;
 
   if (chdir(path_jobs) != 0)
     {
@@ -1588,7 +1596,8 @@ int pbsd_init(
 
     log_err(errno, __func__, log_buf);
 
-    pthread_mutex_unlock(server.sv_qs_mutex);
+    sprintf(log_buf, "%s:1", __func__);
+    unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
 
     return(-1);
     }
@@ -1596,7 +1605,8 @@ int pbsd_init(
   had = server.sv_qs.sv_numjobs;
 
   server.sv_qs.sv_numjobs = 0;
-  pthread_mutex_unlock(server.sv_qs_mutex);
+  sprintf(log_buf, "%s:2", __func__);
+  unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
 
   dir = opendir(".");
 
@@ -1634,7 +1644,6 @@ int pbsd_init(
       if (chk_save_file(pdirent->d_name) == 0)
         {
         /* recover the jobs */
-
         baselen = strlen(pdirent->d_name) - job_suf_len;
 
         psuffix = pdirent->d_name + baselen;
@@ -1650,6 +1659,9 @@ int pbsd_init(
               log_err(ENOMEM,"main","out of memory reloading jobs");
               exit(-1);
               }
+
+            if (type == RECOV_COLD)
+              pjob->ji_cold_restart = TRUE;
             
             pthread_mutex_unlock(pjob->ji_mutex);
             }
@@ -1657,7 +1669,6 @@ int pbsd_init(
           continue;
           }
 
-
         if (strcmp(psuffix, job_suffix))
           continue;
 
@@ -1666,11 +1677,13 @@ int pbsd_init(
 
           if (DArrayAppend(&Array,pjob) == FAILURE)
             {
-            log_err(ENOMEM,"main","out of memory reloading jobs");
+            log_err(ENOMEM, "main", "out of memory reloading jobs");
             exit(-1);
-
             }
 
+          if (type == RECOV_COLD)
+            pjob->ji_cold_restart = TRUE;
+
           pthread_mutex_unlock(pjob->ji_mutex);
           }
         else
@@ -1697,7 +1710,7 @@ int pbsd_init(
     snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "%d total files read from disk", job_count);
     log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_SERVER, msg_daemonname, log_buf);
     closedir(dir);
-    qsort(Array.Data,Array.AppendIndex,sizeof(Array.Data[0]),SortPrioAscend);
+    qsort(Array.Data, Array.AppendIndex, sizeof(Array.Data[0]), SortPrioAscend);
 
     for (Index = 0; Index < Array.AppendIndex; Index++)
       {
@@ -1745,7 +1758,8 @@ int pbsd_init(
       }
 
     DArrayFree(&Array);
-    pthread_mutex_lock(server.sv_qs_mutex);
+    sprintf(log_buf, "%s:1", __func__);
+    lock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
 
     if ((had != server.sv_qs.sv_numjobs) &&
         (type != RECOV_CREATE) &&
@@ -1759,11 +1773,11 @@ int pbsd_init(
       }
 
     sprintf(log_buf, msg_init_exptjobs, had, server.sv_qs.sv_numjobs);
-    pthread_mutex_unlock(server.sv_qs_mutex);
+    sprintf(log_buf, "%s:3", __func__);
+    unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
     log_event(logtype,PBS_EVENTCLASS_SERVER,msg_daemonname,log_buf);
     }  /* END else */
 
-
   /* If queue_rank has gone negative, renumber all jobs and reset rank */
   if (queue_rank < 0)
     {
@@ -1771,7 +1785,7 @@ int pbsd_init(
 
     queue_rank = 0;
 
-    while ((pjob = next_job(&alljobs,&iter)) != NULL)
+    while ((pjob = next_job(&alljobs, &iter)) != NULL)
       {
       pjob->ji_wattr[JOB_ATR_qrank].at_val.at_long = ++queue_rank;
       
@@ -1781,16 +1795,27 @@ int pbsd_init(
       }
     }
 
-  /* finish setting up array structs
-     look for empty arrays and delete them
-     also look for arrays that weren't fully built and setup a work task to
-     continue the cloning process*/
-  iter = -1;
+  return(rc);
+  } /* END handle_job_recovery() */
+
+
+
+
+int cleanup_recovered_arrays()
+
+  {
+  job_array *pa;
+  job       *pjob;
+  int        iter = -1;
+  int        rc = PBSE_NONE;
+  char       log_buf[LOCAL_LOG_BUF_SIZE];
 
   while ((pa = next_array(&iter)) != NULL)
     {
     int job_template_exists = FALSE;
+
     pthread_mutex_lock(pa->ai_mutex);
+
     if (LOGLEVEL >= 7)
       {
       sprintf(log_buf, "%s: locked ai_mutex", __func__);
@@ -1803,9 +1828,21 @@ int pbsd_init(
       pthread_mutex_unlock(pjob->ji_mutex);
       }
 
+    /* if no jobs were recovered, delete this array */
+    if (pa->jobs_recovered == 0)
+      {
+      if ((pjob = find_job(pa->ai_qs.parent_id)) != NULL)
+        job_purge(pjob);
+
+      array_delete(pa);
+
+      /* move on to the next array */
+      continue;
+      }
+
     /* see if we need to upgrade the array version. */
     /* We will upgrade from version 3 or later */
-    if(pa->ai_qs.struct_version == 3)
+    if (pa->ai_qs.struct_version == 3)
       {
       pa->ai_qs.struct_version = ARRAY_QS_STRUCT_VERSION;
       pa->ai_qs.num_purged = pa->ai_qs.num_jobs - pa->jobs_recovered;
@@ -1820,7 +1857,6 @@ int pbsd_init(
       if (job_template_exists == FALSE)
         {
         int        i;
-        job_array *temp;
 
         for (i = 0; i < pa->ai_qs.array_size; i++)
           {
@@ -1835,9 +1871,7 @@ int pbsd_init(
             }
           }
 
-        temp = (job_array*)GET_NEXT(pa->all_arrays);
         array_delete(pa);
-        pa = temp;
         continue;
         }
       else
@@ -1852,9 +1886,7 @@ int pbsd_init(
     else if ((pa->ai_qs.jobs_done == pa->ai_qs.num_jobs) && 
              (job_template_exists == FALSE))
       {
-      job_array *temp = (job_array*)GET_NEXT(pa->all_arrays);
       array_delete(pa);
-      pa = temp;
       continue;
       }
 
@@ -1867,24 +1899,44 @@ int pbsd_init(
     pthread_mutex_unlock(pa->ai_mutex);
     } /* END for each array */
 
-
-  /* Put us back in the Server's Private directory */
-
-  if (chdir(path_priv) != 0)
-    {
-    sprintf(log_buf, msg_init_chdir, path_priv);
-
-    log_err(-1, __func__, log_buf);
-
-    return(3);
-    }
+  return(rc);
+  } /* END cleanup_recovered_arrays() */
 
 
-  /* 10. Open and read in tracking records */
 
-  fd = open(path_track, O_RDONLY | O_CREAT, 0600);
 
-  if (fd < 0)
+int handle_job_and_array_recovery(
+
+  int type)
+
+  {
+  int rc;
+
+  if ((rc = handle_array_recovery(type)) != PBSE_NONE)
+    return(rc);
+  else if ((rc = handle_job_recovery(type)) != PBSE_NONE)
+    return(rc);
+  else
+    rc = cleanup_recovered_arrays();
+
+  return(rc);
+  } /* END handle_job_and_array_recovery() */
+
+
+
+
+int handle_tracking_records()
+
+  {
+  int          fd;
+  int          rc = PBSE_NONE;
+  int          i;
+  struct stat  statbuf;
+#if !defined(DEBUG) && !defined(NO_SECURITY_CHECK)
+  char         EMsg[1024];
+#endif /* not DEBUG and not NO_SECURITY_CHECK */
+
+  if ((fd = open(path_track, O_RDONLY | O_CREAT, 0600)) < 0)
     {
     log_err(errno, __func__, "unable to open tracking file");
 
@@ -1892,7 +1944,6 @@ int pbsd_init(
     }
 
 #if !defined(DEBUG) && !defined(NO_SECURITY_CHECK)
-
   if (chk_file_sec(path_track, 0, 0, S_IWGRP | S_IWOTH, 0, EMsg) != 0)
     {
     return(-1);
@@ -1907,30 +1958,24 @@ int pbsd_init(
     return(-1);
     }
 
-  i = (statbuf.st_size + sizeof(struct tracking) - 1) / sizeof(struct tracking);
+  server.sv_tracksize = (statbuf.st_size + sizeof(struct tracking) - 1) / sizeof(struct tracking);
 
-  if (i < PBS_TRACK_MINSIZE)
+  if (server.sv_tracksize < PBS_TRACK_MINSIZE)
     server.sv_tracksize = PBS_TRACK_MINSIZE;
-  else
-    server.sv_tracksize = i;
 
-  if ((server.sv_track = (struct tracking *)calloc(server.sv_tracksize,sizeof(struct tracking))) == NULL)
+  if ((server.sv_track = calloc(server.sv_tracksize, sizeof(struct tracking))) == NULL)
     {
     /* FAILURE - cannot alloc memory */
-
-    log_err(errno,"pbs_init","calloc failure");
+    log_err(errno, "pbs_init", "calloc failure");
 
     return(-1);
     }
 
-  for (i = 0;i < server.sv_tracksize;i++)
+  for (i = 0; i < server.sv_tracksize; i++)
     (server.sv_track + i)->tk_mtime = 0;
 
   /* NOTE:  tracking file records are optional */
-
-  i = read(fd, (char *)server.sv_track, server.sv_tracksize * sizeof(struct tracking));
-
-  if (i < 0)
+  if (read(fd, (char *)server.sv_track, server.sv_tracksize * sizeof(struct tracking)) < 0)
     {
     log_err(errno, "pbs_init", "unable to read tracksize from tracking file");
     }
@@ -1940,7 +1985,119 @@ int pbsd_init(
   server.sv_trackmodifed = 0;
   
   /* set work task to periodically save the tracking records */
-  set_task(WORK_Timed, (long)(time_now + PBS_SAVE_TRACK_TM), track_save, NULL, FALSE);
+  set_task(WORK_Timed, (long)(time(NULL) + PBS_SAVE_TRACK_TM), track_save, NULL, FALSE);
+
+  return(rc);
+  } /* END handle_tracking_records() */
+
+
+
+
+void setup_threadpool()
+
+  {
+  long              min_threads;
+  long              max_threads;
+  long              thread_idle_time = DEFAULT_THREAD_IDLE;
+  
+  min_threads = get_default_threads();
+  max_threads = min_threads * 10;
+  
+  /* setup the threadpool for use */
+  get_svr_attr_l(SRV_ATR_minthreads, &min_threads);
+  get_svr_attr_l(SRV_ATR_maxthreads, &max_threads);
+  get_svr_attr_l(SRV_ATR_threadidleseconds, &thread_idle_time);
+  
+  initialize_threadpool(&request_pool, min_threads, max_threads, thread_idle_time);
+  } /* END setup_threadpool() */
+
+
+
+
+/*
+ * This file contains the functions to initialize the PBS Batch Server.
+ * The code is called once when the server is brought up.
+ */
+
+int pbsd_init(
+
+  int type)  /* type of initialization   */
+
+  {
+  int               ret = PBSE_NONE;
+  gid_t             gid;
+  char              log_buf[LOCAL_LOG_BUF_SIZE];
+
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_flags = AI_CANONNAME;
+
+  /* The following is code to reduce security risks */
+  if (setup_env(PBS_ENVIRON) == -1)
+    {
+    return(-1);
+    }
+
+  gid = getgid();
+
+  /* secure suppl. groups */
+  if (setgroups(1, &gid) != 0)
+    {
+    snprintf(log_buf, sizeof(log_buf),
+      "Unable to drop secondary groups. Some MAC framework is active?\n");
+    log_err(errno, __func__, log_buf);
+    snprintf(log_buf, sizeof(log_buf),
+      "setgroups(group = %lu) failed: %s\n",
+      (unsigned long)gid, strerror(errno));
+    log_err(errno, __func__, log_buf);
+
+    return(-1);
+    }
+
+  setup_threadpool();
+
+  setup_limits();
+
+  initialize_network_info();
+
+  /* 1. set up to catch or ignore various signals */
+  if ((ret = setup_signal_handling()) != PBSE_NONE)
+    return(ret);
+
+  /* 2. set up the various paths and other global variables we need */
+  if ((ret = initialize_paths()) != PBSE_NONE)
+    return(ret);
+
+  initialize_data_structures_and_mutexes();
+
+  /* 3. Set default server attibutes values */
+  if ((ret = setup_server_attrs(type)) != PBSE_NONE)
+    return(ret);
+
+  /* Open and read in node list if one exists */
+  if ((ret = initialize_nodes()) != PBSE_NONE)
+    return(ret);
+
+  /* the functions we're calling assume this mutex is locked */
+  sprintf(log_buf, "%s:1", __func__);
+  lock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
+
+  if ((ret = handle_queue_recovery(type)) != PBSE_NONE)
+    return(ret);
+
+  if ((ret = handle_job_and_array_recovery(type)) != PBSE_NONE)
+    return(ret);
+
+  /* Put us back in the Server's Private directory */
+  if (chdir(path_priv) != 0)
+    {
+    sprintf(log_buf, msg_init_chdir, path_priv);
+
+    log_err(-1, __func__, log_buf);
+
+    return(3);
+    }
+
+  handle_tracking_records();
 
   /* read the hierarchy file */
   if ((hierarchy_holder = prepare_mom_hierarchy()) == NULL)
@@ -1956,7 +2113,7 @@ int pbsd_init(
   start_request_pool();
 
   /* SUCCESS */
-  return(0);
+  return(PBSE_NONE);
   }  /* END pbsd_init() */
 
 
@@ -2039,7 +2196,7 @@ int pbsd_init_job(
 
   pjob->ji_momhandle = -1;
 
-  /* update at_server attribute in case name changed */
+  /* update at_server pbs_attribute in case name changed */
 
   job_attr_def[JOB_ATR_at_server].at_free(
     &pjob->ji_wattr[JOB_ATR_at_server]);
@@ -2315,8 +2472,10 @@ void pbsd_init_reque(
 
   {
   char logbuf[265];
-  int newstate;
-  int newsubstate;
+  int  newstate;
+  int  newsubstate;
+  int  rc;
+  char log_buf[LOCAL_LOG_BUF_SIZE];
 
   sprintf(logbuf, msg_init_substate,
           pjob->ji_qs.ji_substate);
@@ -2336,8 +2495,9 @@ void pbsd_init_reque(
     set_statechar(pjob);
     }
 
-  pthread_mutex_lock(server.sv_qs_mutex);
-  if (svr_enquejob(pjob, TRUE, -1) == PBSE_NONE)
+  sprintf(log_buf, "%s:1", __func__);
+  lock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
+  if ((rc = svr_enquejob(pjob, TRUE, -1)) == PBSE_NONE)
     {
     strcat(logbuf, msg_init_queued);
     strcat(logbuf, pjob->ji_qs.ji_queue);
@@ -2351,20 +2511,28 @@ void pbsd_init_reque(
   else
     {
     /* Oops, this should never happen */
+    if (rc != PBSE_JOB_RECYCLED)
+      {
+      sprintf(logbuf, "%s; job %s queue %s",
+        msg_err_noqueue,
+        pjob->ji_qs.ji_jobid,
+        pjob->ji_qs.ji_queue);
+    
+      log_err(-1, "pbsd_init_reque", logbuf);
+      }
 
-    sprintf(logbuf, "%s; job %s queue %s",
-            msg_err_noqueue,
-            pjob->ji_qs.ji_jobid,
-            pjob->ji_qs.ji_queue);
+    unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
 
-    log_err(-1, "pbsd_init_reque", logbuf);
+    if (rc != PBSE_JOB_RECYCLED)
+      job_abt(&pjob, logbuf);
 
-    job_abt(&pjob, logbuf);
+    lock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
 
     /* NOTE:  pjob freed but dangling pointer remains */
     }
-  pthread_mutex_unlock(server.sv_qs_mutex);
-  return;
+
+  sprintf(log_buf, "%s:1", __func__);
+  unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
   }  /* END pbsd_init_reque() */
 
 
@@ -2418,72 +2586,6 @@ void catch_abort(
 
 
 
-/*
- * catch_child() - the signal handler for  SIGCHLD.
- *
- * Collect child status and add to work list entry for that child.
- * The list entry is marked as immediate to show the child is gone and
- */
-
-void catch_child(
-
-  int sig)
-
-  {
-  pid_t        pid;
-  int          statloc;
-/*  char         log_buf[LOCAL_LOG_BUF_SIZE];*/
-
-  while (1)
-    {
-    if (((pid = waitpid(-1, &statloc, WNOHANG)) == -1) &&
-        (errno != EINTR))
-      {
-/*      if ((LOGLEVEL >= 7) && (errno != ECHILD))
-        {
-        log_err(errno, id, "waitpid failed");
-        }*/
-
-      return;
-      }
-
-    if (pid == 0)
-      {
-      return;
-      }
-
-/*    if (LOGLEVEL >= 7)
-      {
-      sprintf(log_buf, "caught SIGCHLD for pid %d", pid);
-
-      log_record(
-        PBSEVENT_SYSTEM | PBSEVENT_FORCE,
-        PBS_EVENTCLASS_SERVER,
-        msg_daemonname,
-        log_buf);
-      }*/
-    }    /* END while (1) */
-
-  return;
-  }  /* END catch_child() */
-
-
-/*
- * check_children() - Check for child proccess that have exited
- *
- */
-
-#ifdef NO_SIGCHLD
-void check_children(void)
-
-  {
-  catch_child(0);
-
-  return;
-  } /* END check_children() */
-
-#endif
-
 /*
  * changs_logs - signal handler for SIGHUP
  * Causes the accounting file and log file to be closed and reopened.
@@ -2522,7 +2624,7 @@ void change_logs(
  * Increases log level if SIGUSR1 is received.
  * Decreases log level if SIGUSR2 is received.
  * Variable plogenv tells us whether or not PBSLOGLEVEL was specified
- * If it was not then we will update the server log level attribute
+ * If it was not then we will update the server log level pbs_attribute
  * which allows qmgr to see the current log level value
  */
 
