@@ -132,13 +132,13 @@
 #include "svr_func.h" /* get_svr_attr_* */
 #include "../lib/Libifl/lib_ifl.h" /* get_port_from_server_name_file */
 #include "node_manager.h" /* svr_is_request */
+#include "net_connect.h" /* set_localhost_name */
 #include "../lib/Libnet/lib_net.h" /* start_listener_addrinfo */
 #include "process_request.h" /* process_request */
 #include "net_connect.h" /* set_localhost_name */
 #include "tcp.h" /* tcp_chan */
 
 #define TASK_CHECK_INTERVAL    10
-#define RETRY_ROUTING_INTERVAL 45
 #define HELLO_WAIT_TIME        600
 #define TSERVER_HA_CHECK_TIME  1  /* 1 second sleep time between checks on the lock file for high availability */
 
@@ -158,10 +158,8 @@ extern void acct_close(void);
 extern int  svr_startjob(job *, struct batch_request *, char *, char *);
 extern int RPPConfigure(int, int);
 extern void acct_cleanup(long);
-#ifdef NO_SIGCHLD
-extern void check_children();
-#endif
 void stream_eof(int, u_long, uint16_t, int);
+extern void scheduler_close();
 
 #ifndef MAX_PATH_LEN
 #define MAX_PATH_LEN 256
@@ -185,9 +183,8 @@ static void lock_out_ha();
 /* external data items */
 
 extern hello_container failures;
-extern int    svr_chngNodesfile;
-extern int    svr_totnodes;
-extern AvlTree streams;
+extern int             svr_chngNodesfile;
+extern int             svr_totnodes;
 extern struct all_jobs alljobs;
 
 /* External Functions */
@@ -203,12 +200,12 @@ int mutex_unlock (mutex_t *);
 int get_file_info (char *,unsigned long *,long *,bool_t *,bool_t *);
 int get_full_path (char *,char *,int);
 int svr_restart();
-void          restore_attr_default (struct attribute *);
+void          restore_attr_default (struct pbs_attribute *);
 
 /* Global Data Items */
 
-time_t                  last_routing_retry_time = 0;
 time_t                  last_task_check_time = 0;
+int                     disable_timeout_check = FALSE;
 int                     lockfds = -1;
 int                     ForceCreation = FALSE;
 int                     high_availability_mode = FALSE;
@@ -252,12 +249,14 @@ listener_connection     listener_conns[MAXLISTENERS];
 int                     queue_rank = 0;
 int                     a_opt_init = -1;
 int                     wait_for_moms_hierarchy = FALSE;
+
+int                     route_retry_interval = 10; /* time in seconds to check routing queues */
 /* HA global data items */
 long                    HALockCheckTime = 0;
 long                    HALockUpdateTime = 0;
 char                    HALockFile[MAXPATHLEN+1];
 char                    OriginalPath[MAXPATHLEN+1];
-mutex_t                 EUIDMutex; /* prevents thread from trying to lock the file 
+mutex_t                 EUIDMutex; /* prevents thread from trying to lock the file
                                       from a different euid */
 int                     HALockFD;
 
@@ -271,7 +270,6 @@ char                    server_name[PBS_MAXSERVERNAME + 1]; /* host_name[:servic
 int                     svr_do_schedule = SCH_SCHEDULE_NULL;
 pthread_mutex_t        *svr_do_schedule_mutex;
 pthread_mutex_t        *check_tasks_mutex;
-pthread_mutex_t        *retry_routing_mutex;
 extern all_queues       svr_queues;
 extern int              listener_command;
 extern hello_container  hellos;
@@ -296,8 +294,7 @@ int                     allow_any_mom = FALSE;
 int                     array_259_upgrade = FALSE;
 
 
-/* Globals to thread the accept port */
-pthread_t      accept_thread_id = -1;
+
 
 
 char server_localhost[PBS_MAXHOSTNAME + 1];
@@ -380,21 +377,24 @@ static void need_y_response(
 
 
 int process_pbs_server_port(
-  int sock)
+     
+  int sock,
+  int is_scheduler_port)
+ 
   {
   int          proto_type;
   int          rc = PBSE_NONE;
   int          version;
   char         log_buf[LOCAL_LOG_BUF_SIZE];
   struct tcp_chan *chan = NULL;
-  
+   
   if ((chan = DIS_tcp_setup(sock)) == NULL)
     {
     }
 
   proto_type = disrui_peek(chan,&rc);
   
-  switch(proto_type)
+  switch (proto_type)
     {
     case PBS_BATCH_PROT_TYPE:
       
@@ -404,7 +404,7 @@ int process_pbs_server_port(
       
     case IS_PROTOCOL:
 
-      version = disrsi(chan,&rc);
+      version = disrsi(chan, &rc);
       
       if (rc != DIS_SUCCESS)
         {
@@ -413,7 +413,7 @@ int process_pbs_server_port(
         break;
         }
       
-      rc = svr_is_request(chan,version);
+      rc = svr_is_request(chan, version);
       
       break;
 
@@ -428,59 +428,106 @@ int process_pbs_server_port(
         addr = (struct sockaddr_in *)&s_addr;
 
         if (proto_type == 0)
-          snprintf(log_buf, sizeof(log_buf),
-              "Socket (%d) close detected from %s", sock, netaddr(addr));
-        else 
+          {
+          /* 
+           * Don't log error if close is on scheduler port.  Scheduler is
+           * responsible for closing the connection
+           */
+          if (!is_scheduler_port)
+            {
+            snprintf(log_buf, sizeof(log_buf),
+                "Socket (%d) close detected from %s", sock, netaddr(addr));
+            log_err(-1, __func__, log_buf);
+            }
+          }
+        else
+          {
           snprintf(log_buf,sizeof(log_buf),
               "Socket (%d) Unknown protocol %d from %s", sock, proto_type, netaddr(addr));
+          log_err(-1, __func__, log_buf);
+          }
 
-        log_err(-1, __func__, log_buf);
         }
 
-      /*close_conn(sock, FALSE);*/
       rc = PBSE_SOCKET_DATA;
 
       break;
       }
     }
+
   if (chan != NULL)
     DIS_tcp_cleanup(chan);
 
-  return rc;
+  return(rc);
   }  /* END process_pbs_server_port() */
 
-/* process_pbs_server_port_scheduler
+
+
+
+/* 
+ * process_pbs_server_port_scheduler
  * This function is a wrapper for process_pbs_server_port
  * whose signature has been changed to make torques
  * accept process fully multithreaded. 
- * This will be run from wait request after being scheduled
- * by contact_sched
+ * This will be run from contact_sched
  */
-void *process_pbs_server_port_scheduler(void *new_sock)
-  {
-  int sock = *(int *)new_sock;
 
-  process_pbs_server_port(sock);
-  return(NULL);
+void process_pbs_server_port_scheduler(
+    
+  int *new_sock)
+
+  {
+  int rc = PBSE_NONE;
+  int sock = *new_sock;
+
+  while ((rc != PBSE_SOCKET_DATA) && (rc != PBSE_SOCKET_INFORMATION)
+        && (rc != PBSE_INTERNAL) && (rc != PBSE_SYSTEM)  && (rc != PBSE_SOCKET_CLOSE))
+    {
+    netcounter_incr();
+    rc = process_pbs_server_port(sock, TRUE);
+    }
+
+  /* 
+   * Socket should have been closed by scheduler, except in error cases,
+   * but we still need to call close_conn() to clean up connections.
+   */
+
+  close_conn(sock, FALSE);
+
+  scheduler_close();
+  return;
   }
 
-void *start_process_pbs_server_port(void *new_sock)
+
+
+
+void *start_process_pbs_server_port(
+    
+  void *new_sock)
+
   {
   int sock = *(int *)new_sock;
   int rc = PBSE_NONE;
-
+ 
   free(new_sock);
-  while ((rc != PBSE_SOCKET_DATA) && (rc != PBSE_SOCKET_INFORMATION) 
-        && (rc != PBSE_INTERNAL) && (rc != PBSE_SYSTEM)  && (rc != PBSE_SOCKET_CLOSE))
+
+  while ((rc != PBSE_SOCKET_DATA) &&
+         (rc != PBSE_SOCKET_INFORMATION) &&
+         (rc != PBSE_INTERNAL) &&
+         (rc != PBSE_SYSTEM) &&
+         (rc != PBSE_SOCKET_CLOSE))
     {
-    rc = process_pbs_server_port(sock);
+    netcounter_incr();
+
+    rc = process_pbs_server_port(sock, FALSE);
     }
+
   close_conn(sock, FALSE);
+
   /* Thread exit */
   return(NULL);
   }
-
-
+ 
 
 
 void clear_listeners(void)   /* I */
@@ -567,7 +614,7 @@ int PBSShowUsage(
  */
 
 void parse_command_line(
-    
+
   int   argc,
   char *argv[])
 
@@ -598,7 +645,7 @@ void parse_command_line(
     { "create", RECOV_CREATE },
     { "", RECOV_Invalid }
     };
-  
+
   ForceCreation = FALSE;
 
   while ((c = getopt(argc, argv, "A:a:cd:DefhH:L:l:mM:p:R:S:t:uv-:")) != -1)
@@ -966,6 +1013,10 @@ void parse_command_line(
   }
 
 
+/* Globals to thread the accept port */
+pthread_t      accept_thread_id = -1;
+pthread_t      route_retry_thread_id = -1;
+
 
 /*
  * check_tasks - look for the next work task to perform:
@@ -1013,7 +1064,7 @@ void *check_tasks()
     listener_command = SCH_SCHEDULE_TIME;
     pthread_mutex_unlock(listener_command_mutex);
     }
-  
+
   pthread_mutex_unlock(check_tasks_mutex);
 
   return(NULL);
@@ -1035,7 +1086,7 @@ static int start_hot_jobs(void)
   job *pjob;
 
   int  iter = -1;
-  
+
   while ((pjob = next_job(&alljobs,&iter)) != NULL)
     {
 
@@ -1052,7 +1103,7 @@ static int start_hot_jobs(void)
 
       ct++;
       }
-    
+
     pthread_mutex_unlock(pjob->ji_mutex);
     }
 
@@ -1087,24 +1138,18 @@ void *handle_queue_routing_retries(
   {
   pbs_queue *pque;
   int        iter = -1;
-  time_t     time_now = time(NULL);
 
-  pthread_mutex_lock(retry_routing_mutex);
-
-  if (time_now - last_routing_retry_time > RETRY_ROUTING_INTERVAL)
+  while(1)
     {
-    last_routing_retry_time = time_now;
-    
+    sleep(route_retry_interval);
     while ((pque = next_queue(&svr_queues, &iter)) != NULL)
       {
       if (pque->qu_qs.qu_type == QTYPE_RoutePush)
         queue_route(pque);
-      
+
       unlock_queue(pque, __func__, NULL, 0);
       }
     }
-  
-  pthread_mutex_unlock(retry_routing_mutex);
 
   return(NULL);
   } /* END handle_queue_routing_retries() */
@@ -1126,6 +1171,7 @@ void *handle_scheduler_contact(
 
 
 void *start_accept_listener()
+
   {
   char server_name_trimmed[PBS_MAXSERVERNAME + 1];
   char *colon_pos = NULL;
@@ -1135,11 +1181,14 @@ void *start_accept_listener()
   else
     strncpy(server_name_trimmed, server_name, colon_pos - server_name);
   start_listener_addrinfo(server_name_trimmed, pbs_server_port_dis, start_process_pbs_server_port);
-  return NULL;
-  }
+  return(NULL);
+  } /* END start_accept_listener() */
+
+
 
 
 void start_accept_thread()
+
   {
   pthread_attr_t accept_attr;
   if ((pthread_attr_init(&accept_attr)) != 0)
@@ -1154,19 +1203,59 @@ void start_accept_thread()
     }
   else if ((pthread_create(&accept_thread_id, &accept_attr, start_accept_listener, NULL)) != 0)
     {
+    accept_thread_id = -1;
     perror("could not start listener for pbs_server");
     log_err(-1, msg_daemonname, "Failed to start listener for pbs_server");
     }
-  }
+  } /* END start_accept_thread() */
+
+
+
+void start_routing_retry_thread()
+
+  {
+  pthread_attr_t routing_attr;
+  if ((pthread_attr_init(&routing_attr)) != 0)
+    {
+    perror("pthread_attr_init failed. Could not start accept thread");
+    log_err(-1, msg_daemonname,"pthread_attr_init failed. Could not start accept thread");
+    }
+  else if ((pthread_attr_setdetachstate(&routing_attr, PTHREAD_CREATE_DETACHED) != 0))
+    {
+    perror("pthread_attr_setdetatchedstate failed. Could not start accept thread");
+    log_err(-1, msg_daemonname,"pthread_attr_setdetachedstate failed. Could not start accept thread");
+    }
+  else if ((pthread_create(&route_retry_thread_id, &routing_attr, handle_queue_routing_retries, NULL)) != 0)
+    {
+    perror("could not start listener for pbs_server");
+    log_err(-1, msg_daemonname, "Failed to start listener for pbs_server");
+    }
+  } /* END start_routing_retry_thread() */
+
+
 
 
 void monitor_accept_thread()
   {
-  if (pthread_kill(accept_thread_id, 0) == ESRCH)
+  if (accept_thread_id == (pthread_t)-1)
+    start_accept_thread();
+  else if (pthread_kill(accept_thread_id, 0) == ESRCH)
     {
     start_accept_thread();
     }
-  }
+  } /* END monitor_accept_thread() */
+
+
+
+
+void monitor_route_retry_thread()
+  {
+  if (pthread_kill(route_retry_thread_id, 0) == ESRCH)
+    {
+    start_routing_retry_thread();
+    }
+  } /* END monitor_route_retry_thread() */
+
 
 
 
@@ -1178,7 +1267,6 @@ void main_loop(void)
   time_t        waittime = 5;
   job          *pjob;
   int           iter;
-  time_t        last_jobstat_time;
   long          when = 0;
   long          timeout = 0;
   long          log = 0;
@@ -1195,7 +1283,6 @@ void main_loop(void)
   void check_job_log(struct work_task *);
   void check_acct_log(struct work_task *);
 
-  last_jobstat_time = time_now;
   server.sv_started = time_now; /* time server started */
 
 
@@ -1205,13 +1292,6 @@ void main_loop(void)
 
   log_event(PBSEVENT_SYSTEM | PBSEVENT_FORCE,PBS_EVENTCLASS_SERVER,msg_daemonname,log_buf);
 
-#ifdef NO_SIGCHLD
-  log_record(
-    PBSEVENT_SYSTEM | PBSEVENT_FORCE,
-    PBS_EVENTCLASS_SERVER,
-    msg_daemonname,
-    "Server NOT using signal handler for SIGCHLD");
-#endif
   /* do not check nodes immediately as they will initially be marked
      down unless they have already reported in */
 
@@ -1269,13 +1349,15 @@ void main_loop(void)
     try_hellos = time_now + HELLO_WAIT_TIME;
 
   start_accept_thread();
+  start_routing_retry_thread();
 
   while (state != SV_STATE_DOWN)
     {
     /* first process any task whose time delay has expired */
-    last_jobstat_time = time_now = time(NULL);
+    time_now = time(NULL);
 
     monitor_accept_thread();
+    monitor_route_retry_thread();
 
     if (try_hellos <= time_now)
       send_any_hellos_needed();
@@ -1283,19 +1365,22 @@ void main_loop(void)
     if (time_now - last_task_check_time > TASK_CHECK_INTERVAL)
       enqueue_threadpool_request(check_tasks, NULL);
 
-    get_svr_attr_l(SRV_ATR_tcp_timeout, &timeout);
-
-    /* don't allow timeouts to go below 300 seconds - this is a safety
-     * net for an extremely rare error */
-    if (timeout < 300)
+    if (disable_timeout_check == FALSE)
       {
-      snprintf(log_buf, sizeof(log_buf), "tcp timeout was %ld resetting to 300",
-        timeout);
-      timeout = 300;
-      log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, msg_daemonname, log_buf);
-      }
+      get_svr_attr_l(SRV_ATR_tcp_timeout, &timeout);
 
-    DIS_tcp_settimeout(timeout);
+      /* don't allow timeouts to go below 300 seconds - this is a safety
+       * net for an extremely rare error */
+      if (timeout < 300)
+        {
+        snprintf(log_buf, sizeof(log_buf), "tcp timeout was %ld resetting to 300",
+          timeout);
+        timeout = 300;
+        log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, msg_daemonname, log_buf);
+        }
+      
+      DIS_tcp_settimeout(timeout);
+      }
 
     waittime = MAX(1, waittime);
 
@@ -1336,7 +1421,7 @@ void main_loop(void)
         }
 
       /* If more than _LIMIT seconds since start, stop */
-      if ((c == 0) || 
+      if ((c == 0) ||
           (time_now > server.sv_started + SVR_HOT_LIMIT))
         {
         server_init_type = RECOV_WARM;
@@ -1346,25 +1431,7 @@ void main_loop(void)
         }
       }
 
-    /* retry routing as needed */
-    if (time_now - last_routing_retry_time > RETRY_ROUTING_INTERVAL)
-      enqueue_threadpool_request(handle_queue_routing_retries, NULL);
-
-#ifdef NO_SIGCHLD
-    pthread_mutex_lock(server.sv_qs_mutex);
-
-    /* if we have any jobs, check if any child jobs have completed */
-
-    if (server.sv_qs.sv_numjobs > 0)
-      {
-      check_children();
-      }
-    
-    pthread_mutex_unlock(server.sv_qs_mutex);
-#endif
-
     /* wait for a request and process it */
-
     if (wait_request(waittime, &state) != 0)
       {
       log_err(-1, msg_daemonname, "wait_request failed");
@@ -1401,13 +1468,13 @@ void main_loop(void)
       /* at this point kill the threadpool */
       destroy_request_pool();
       }
-    
+
     pthread_mutex_unlock(server.sv_jobstates_mutex);
 
     get_svr_attr_l(SRV_ATR_State, &state);
     usleep(100);
     }    /* END while (*state != SV_STATE_DOWN) */
-  
+
   pthread_cancel(accept_thread_id);
 
   svr_save(&server, SVR_SAVE_FULL); /* final recording of server */
@@ -1421,7 +1488,7 @@ void main_loop(void)
     {
     if (pjob->ji_modified)
       job_save(pjob, SAVEJOB_FULL, 0);
-    
+
     pthread_mutex_unlock(pjob->ji_mutex);
     }
 
@@ -1532,6 +1599,7 @@ int main(
   char         log_buf[LOCAL_LOG_BUF_SIZE];
   unsigned int server_name_file_port = 0;
 
+  extern char  pbs_server_name[];
   extern char *msg_svrdown; /* log message   */
   extern char *msg_startup1; /* log message   */
 
@@ -1602,6 +1670,8 @@ int main(
   get_port_from_server_name_file(&server_name_file_port);
   if (server_name_file_port != 0)
     pbs_server_port_dis = server_name_file_port;
+
+  strcpy(pbs_server_name, server_name);
   /* The following port numbers might have been initialized in set_globals_from_environment() above. */
 
   if (pbs_server_port_dis <= 0)
@@ -1644,7 +1714,7 @@ int main(
 
   if (recov_svr_attr(server_init_type) == -1)
     {
-    fprintf(stderr,"%s: failed to get server attributes\n", 
+    fprintf(stderr,"%s: failed to get server attributes\n",
       ProgName);
 
     return(1);
@@ -1652,10 +1722,10 @@ int main(
 
   /*
    * make sure no other server is running with this home directory.
-   * If server lockfile attribute has been set use it.
+   * If server lockfile pbs_attribute has been set use it.
    * If not use default location for it
    */
-  pthread_mutex_lock(server.sv_attr_mutex); 
+  pthread_mutex_lock(server.sv_attr_mutex);
   if ((server.sv_attr[SRV_ATR_lockfile].at_flags & ATR_VFLAG_SET) &&
                (server.sv_attr[SRV_ATR_lockfile].at_val.at_str))
     {
@@ -1665,7 +1735,7 @@ int main(
 
     if (LockfilePtr[0] == '/')
       {
-      snprintf(lockfile,sizeof(lockfile),"%s",LockfilePtr);     
+      snprintf(lockfile,sizeof(lockfile),"%s",LockfilePtr);
       }
     else
       {
@@ -1687,7 +1757,7 @@ int main(
   snprintf(HALockFile,MAXPATHLEN,"%s", lockfile);
   HALockCheckTime = server.sv_attr[SRV_ATR_LockfileCheckTime].at_val.at_long;
   HALockUpdateTime = server.sv_attr[SRV_ATR_LockfileUpdateTime].at_val.at_long;
-  pthread_mutex_unlock(server.sv_attr_mutex); 
+  pthread_mutex_unlock(server.sv_attr_mutex);
 
   /* apply HA defaults */
 
@@ -1761,16 +1831,14 @@ int main(
   /*
    * Open the log file so we can start recording events
    *
-   * set log_event_mask to point to the log_event attribute value so
+   * set log_event_mask to point to the log_event pbs_attribute value so
    * it controls which events are logged.
    */
   pthread_mutex_lock(server.sv_attr_mutex);
   log_event_mask = &server.sv_attr[SRV_ATR_log_events].at_val.at_long;
   pthread_mutex_unlock(server.sv_attr_mutex);
 
-  sprintf(path_log, "%s/%s",
-          path_home,
-          PBS_LOGFILES);
+  sprintf(path_log, "%s/%s", path_home, PBS_LOGFILES);
 
   pthread_mutex_lock(log_mutex);
   log_open(log_file, path_log);
@@ -1785,10 +1853,10 @@ int main(
     log_buf);
 
   set_localhost_name(server_localhost, localhost_len);
+
   /* initialize the server objects and perform specified recovery */
   /* will be left in the server's private directory  */
   /* NOTE:  env cleared in pbsd_init() */
-
   if (pbsd_init(server_init_type) != 0)
     {
     log_err(-1, msg_daemonname, "pbsd_init failed");
@@ -1811,6 +1879,15 @@ int main(
     log_buf);
 
 
+  if (check_network_port(pbs_server_port_dis) != 0)
+    {
+    perror("pbs_server: network");
+
+    log_err(-1, msg_daemonname, "pbs_server port already bound");
+
+    exit(3);
+    }
+
   if (init_network(0, start_process_pbs_server_port) != 0)
     {
     perror("pbs_server: unix domain socket");
@@ -1880,19 +1957,19 @@ void check_job_log(
     snprintf(log_buf,sizeof(log_buf),"checking for old job logs in dir '%s' (older than %ld days)",
       path_svrlog,
       keep_days);
-    
+
     log_event(
       PBSEVENT_SYSTEM | PBSEVENT_FORCE,
       PBS_EVENTCLASS_SERVER,
       msg_daemonname,
       log_buf);
-    
+
     if (log_remove_old(path_jobinfo_log, keep_days * SECS_PER_DAY) != 0)
       {
       log_err(-1,"check_job_log","failure occurred when checking for old job logs");
       }
     }
-  
+
   if (max_size != 0)
     {
     if ((job_log_size() >= max_size) &&
@@ -1905,7 +1982,7 @@ void check_job_log(
         PBS_EVENTCLASS_SERVER,
         msg_daemonname,
         "Rolling job log file");
-      
+
       if (roll_depth != -1)
         {
         depth = roll_depth;
@@ -1948,24 +2025,24 @@ void check_log(
     snprintf(log_buf,sizeof(log_buf),"checking for old pbs_server logs in dir '%s' (older than %ld days)",
       path_svrlog,
       keep_days);
-    
+
     log_event(
       PBSEVENT_SYSTEM | PBSEVENT_FORCE,
       PBS_EVENTCLASS_SERVER,
       msg_daemonname,
       log_buf);
-    
+
     if (log_remove_old(path_svrlog, keep_days * SECS_PER_DAY) != 0)
       {
       log_err(-1,"check_log","failure occurred when checking for old pbs_server logs");
       }
     }
-  
+
   if (get_svr_attr_l(SRV_ATR_LogFileMaxSize, &max_size) == PBSE_NONE)
     {
     long roll_depth = 1;
 
-    if ((log_size() >= max_size) && 
+    if ((log_size() >= max_size) &&
         (max_size > 0))
       {
       log_event(
@@ -1973,7 +2050,7 @@ void check_log(
         PBS_EVENTCLASS_SERVER,
         msg_daemonname,
         "Rolling log file");
-      
+
       get_svr_attr_l(SRV_ATR_LogFileRollDepth, &roll_depth);
 
       if ((roll_depth >= INT_MAX) || (roll_depth < 1))
@@ -1986,7 +2063,7 @@ void check_log(
         }
       }
     }
-  
+
 
   /* periodically record the version and loglevel */
   get_svr_attr_str(SRV_ATR_version, &version);
@@ -2023,13 +2100,13 @@ void check_acct_log(
       (keep_days >= 0))
     {
     sprintf(log_buf,"Checking accounting files - keep days = %ld", keep_days);
-    
+
     log_event(
       PBSEVENT_SYSTEM | PBSEVENT_FORCE,
       PBS_EVENTCLASS_SERVER,
       msg_daemonname,
       log_buf);
-     
+
     acct_cleanup(keep_days);
     }
 
@@ -2101,7 +2178,7 @@ static int get_port(
 
 
 /**
- * This function will extract the directory portion of 
+ * This function will extract the directory portion of
  * the given path and copy it into the Dir parameter
  *
  * @param FullPath (I)
@@ -2208,7 +2285,7 @@ int is_ha_lock_file_valid(
   }  /* END is_ha_lock_file_valid() */
 
 
-      
+
 /**
  * Try to release a lock on the given file
  *
@@ -2275,7 +2352,7 @@ int release_file_lock(
 
 
 
-/** 
+/**
  * Try to acquire a lock on the given file
  *
  * @param LockFile (I) the name of the file to lock.
@@ -2296,7 +2373,7 @@ int acquire_file_lock(
   char         log_buf[LOCAL_LOG_BUF_SIZE];
 
   if ((LockFile == NULL) ||
-      (LockFD == NULL) || 
+      (LockFD == NULL) ||
       (FileType == NULL))
     {
     return(FAILURE);
@@ -2396,15 +2473,15 @@ void *update_ha_lock_thread(
 
     rc = 0;
     LocalErrno = 0;
-    
+
     mutex_lock(&EUIDMutex);
-    
+
     errno = 0;
     if (stat(HALockFile,&statbuf) == 0)
       {
       /* check to make sure that no other process has modified this file
        * since the last time we did */
-      
+
       if ((LastModifyTime > 0) && (LastModifyTime != statbuf.st_mtime))
         {
         snprintf(EMsg,sizeof(EMsg),"update time changed unexpectedly");
@@ -2418,7 +2495,7 @@ void *update_ha_lock_thread(
         LastModifyTime  = time(NULL);
         timebuf.actime  = LastModifyTime;
         timebuf.modtime = LastModifyTime;
-        
+
         errno = 0;
         rc = utime(HALockFile,&timebuf);
         LocalErrno = errno;
@@ -2451,7 +2528,7 @@ void *update_ha_lock_thread(
           EMsg,
           LocalErrno,
           ErrorString);
-        
+
         log_err(LocalErrno, __func__, log_buf);
         }
       else
@@ -2459,7 +2536,7 @@ void *update_ha_lock_thread(
         sprintf(log_buf,"could not update HA lock file '%s' in heartbeat thread (%s)",
           HALockFile,
           EMsg);
-        
+
         log_err(-1, __func__, log_buf);
         }
 
@@ -2498,7 +2575,7 @@ int start_update_ha_lock_thread()
     return(FAILURE);
     }
 
-  snprintf(smallBuf,sizeof(smallBuf),"%ld\n",(long)sid); 
+  snprintf(smallBuf,sizeof(smallBuf),"%ld\n",(long)sid);
   if (write(fds,smallBuf,strlen(smallBuf)) != (ssize_t)strlen(smallBuf))
     {
     log_err(-1, __func__, "Couldn't write the pid to the lockfile\n");
@@ -2601,7 +2678,7 @@ static void lock_out_ha()
       usleep(DEF_USPERSECOND * HALockCheckTime);
 
       time_now = time(NULL);
-      
+
       UseFLock = FALSE;
       if (MutexLockFD > 0)
         close(MutexLockFD);
@@ -2618,11 +2695,11 @@ static void lock_out_ha()
       while (acquire_file_lock(MutexLockFile,&MutexLockFD,"HA") == FAILURE)
         {
         sprintf(log_buf,"Could not acquire HA flock--trying again in 1 second\n");
-        
+
         usleep(DEF_USPERSECOND);
         }
       }
-    
+
     /* check if file lock exists */
 
     if (stat(HALockFile,&StatBuf) == 0)
@@ -2641,21 +2718,21 @@ static void lock_out_ha()
       /* update the file to mark it as ours */
 
       utime(HALockFile,NULL);
-      
+
       FilePossession = TRUE;
       }
     else
       {
       /* file doesn't exist--wait required amount of time and check again */
-      
+
       if (FileIsMissing == FALSE)
         {
         FileIsMissing = TRUE;
-        
+
         /* if we don't have a mutex to protect file creation
          * race conditions, we need to wait and check again:
          * otherwise we can safely create it immediately */
-        
+
         if (UseFLock == FALSE)
           continue;
         }
@@ -2664,33 +2741,33 @@ static void lock_out_ha()
        * probably safe to create it */
 
       HALockFD = open(HALockFile,O_CREAT|O_EXCL|O_RDONLY,0600);
-      
+
       if (HALockFD < 0)
         {
         sprintf(log_buf,"could not create HA lock file '%s'--errno %d:%s",
           HALockFile,
           errno,
           strerror(errno));
-        
+
         continue;
         }
-      
+
       FilePossession = TRUE;
       }
-    
+
     if (FilePossession == TRUE)
       {
       /* start heartbeat thread */
-      
+
       start_update_ha_lock_thread();
       }
-    
+
     if (UseFLock == TRUE)
       close(MutexLockFD); /* unlock file mutex */
     } /* END while (!FilePossession) */
-  
+
   /* we have the file lock--go ahead and log this fact */
-  
+
   log_record(
     PBSEVENT_SYSTEM,
     PBS_EVENTCLASS_SERVER,
@@ -2715,35 +2792,35 @@ static int daemonize_server(
   int *sid)           /* O */
 
   {
-  int    pid;          
+  int    pid;
   FILE  *dummyfile;
-  
+
   if (!DoBackground)
-    {  
+    {
     /* handle foreground (i.e. debug mode) */
-    
+
     *sid = getpid();
-    
+
     setvbuf(stdout,NULL,_IOLBF,0);
     setvbuf(stderr,NULL,_IOLBF,0);
-    
+
     return(SUCCESS);
     }
-  
+
   /* run pbs_server in the background */
-  
+
   /* fork to disconnect from terminal */
-  
+
   if ((pid = fork()) == -1)
     {
     log_err(errno, __func__, "cannot fork into background");
-    
+
     return(FAILURE);
    }
-  
-  
+
+
   if (pid != 0)
-    {        
+    {
      /* exit if parent */
 
      log_event(
@@ -2751,41 +2828,41 @@ static int daemonize_server(
        PBS_EVENTCLASS_SERVER,
        __func__,
        "INFO:      parent is exiting");
-   
+
      exit(0);
     }
 
   /* NOTE: setsid() disconnects from controlling-terminal */
-  
+
   if ((*sid = setsid()) == -1)
     {
     log_err(errno, __func__, "Could not disconnect from controlling terminal");
 
     return(FAILURE);
-    }    
+    }
 
   /* disconnect stdin,stdout,stderr */
 
   fclose(stdin);
   fclose(stdout);
   fclose(stderr);
-    
+
   dummyfile = fopen("/dev/null","r");
   assert((dummyfile != 0) && (fileno(dummyfile) == 0));
-    
+
   dummyfile = fopen("/dev/null","w");
   assert((dummyfile != 0) && (fileno(dummyfile) == 1));
 
   dummyfile = fopen("/dev/null","w");
   assert((dummyfile != 0) && (fileno(dummyfile) == 2));
-    
+
   if ((pid = fork()) == -1)
     {
     log_err(errno, __func__, "cannot fork into background");
-      
+
     return(FAILURE);
     }
-    
+
   if (pid != 0)
     {
     /* exit if parent */
@@ -2795,10 +2872,10 @@ static int daemonize_server(
       PBS_EVENTCLASS_SERVER,
       __func__,
       "INFO:      parent is exiting");
-      
+
     exit(0);
     }
-  
+
   /* update the sid (pid written to the lock file) so that
    * the correct pid is present */
   *sid = getpid();
@@ -2808,7 +2885,7 @@ static int daemonize_server(
     PBS_EVENTCLASS_SERVER,
     __func__,
     "INFO:      child process in background");
-    
+
   return(SUCCESS);
   } /* END daemonize_server() */
 
@@ -2834,39 +2911,39 @@ int get_file_info(
   long          *FileSize,    /* O (optional */
   bool_t        *IsExe,       /* O (optional */
   bool_t        *IsDir)       /* O (optional */
-    
+
   {
   int          rc;
   char        *ptr;
   char         log_buf[LOCAL_LOG_BUF_SIZE];
-  
+
   struct stat  sbuf;
-  
+
   if (IsExe != NULL)
     *IsExe = FALSE;
 
   if (ModifyTime != NULL)
     *ModifyTime = 0;
-  
+
   if (FileSize != NULL)
     *FileSize = 0;
-  
+
   if (IsDir != NULL)
     *IsDir = FALSE;
-  
+
   if ((FileName == NULL) || (FileName[0] == '\0'))
     {
-    return(FAILURE);   
+    return(FAILURE);
     }
 
   /* FORMAT:   <FILENAME>[ <ARG>]... */
-  
+
   /* NOTE:  mask off, then restore possible args */
   ptr = strchr(FileName,' ');
-  
+
   if (ptr != NULL)
     *ptr = '\0';
-  
+
   rc = stat(FileName,&sbuf);
 
   if (rc == -1)
@@ -2875,22 +2952,22 @@ int get_file_info(
       FileName,
       errno,
       strerror(errno));
-    
+
     log_err(errno, __func__, log_buf);
-    
+
     return(FAILURE);
     }
-  
+
   if (ModifyTime != NULL)
     {
     *ModifyTime = (unsigned long)sbuf.st_mtime;
     }
-  
+
   if (FileSize != NULL)
     {
     *FileSize = (long)sbuf.st_size;
     }
-  
+
   if (IsExe != NULL)
     {
     if (sbuf.st_mode & S_IXUSR)
@@ -2898,7 +2975,7 @@ int get_file_info(
     else
       *IsExe = FALSE;
     }
-  
+
   if (IsDir != NULL)
    {
    if (sbuf.st_mode & S_IFDIR)
@@ -2906,7 +2983,7 @@ int get_file_info(
    else
      *IsDir = FALSE;
    }
-  
+
   return(SUCCESS);
   } /* end get_file_info() */
 
@@ -2932,16 +3009,16 @@ int get_full_path(
   char    tmpPath[MAX_LINE];
   bool_t  IsExe = FALSE;
   bool_t  IsDir = FALSE;
-  
+
   if (Cmd[0] == '/')
     {
     /* absolute path specified */
-    
+
     if (get_file_info(Cmd,NULL,NULL,&IsExe,&IsDir) == FAILURE)
       {
       return(FAILURE);
       }
-    
+
     if ((IsExe == FALSE) && (IsDir == FALSE))
       {
       return(FAILURE);
@@ -2951,7 +3028,7 @@ int get_full_path(
 
     return(SUCCESS);
     }
-    
+
   PathLocation = strtok_r(OriginalPath,Delims,&TokPtr);
 
   while (PathLocation != NULL)
@@ -2959,10 +3036,10 @@ int get_full_path(
     if (strlen(PathLocation) <= 0)
       {
       PathLocation = strtok_r(NULL,Delims,&TokPtr);
-      
+
       continue;
       }
-    
+
     if (PathLocation[strlen(PathLocation) - 1] == '/')
       {
       sprintf(tmpPath,"%s%s",
@@ -2975,26 +3052,26 @@ int get_full_path(
         PathLocation,
         Cmd);
       }
-    
+
     if (get_file_info(tmpPath,NULL,NULL,&IsExe,NULL) == FAILURE)
       {
       PathLocation = strtok_r(NULL,Delims,&TokPtr);
-      
+
       continue;
      }
-    
+
     if (IsExe == FALSE)
       {
       PathLocation = strtok_r(NULL,Delims,&TokPtr);
-      
+
       continue;
       }
-    
+
     snprintf(GoodCmd,GoodCmdLen,"%s",tmpPath);
-    
+
     return(SUCCESS);
     } /* END while (PathLocation != NULL) */
-  
+
   return(FAILURE);
   } /* END get_full_path() */
 
@@ -3009,65 +3086,65 @@ int svr_restart()
 
   {
   int   rc;
-  
+
   char  FullCmd[MAX_LINE];
   char  log_buf[LOCAL_LOG_BUF_SIZE];
-  
+
   if (get_full_path(
         ArgV[0],
         FullCmd,
         sizeof(FullCmd)) == FAILURE)
     {
     sprintf(log_buf, "ALERT:      cannot locate full path for '%s'\n", ArgV[0]);
-    
+
     log_err(-1, __func__, log_buf);
 
     exit(-10);
     }
-  
+
   /* shut down network connections */
   net_close(-1);   /* close all network connections */
-  
-  /* copying FullCmd to ArV[0] is necessary for multiple restarts because 
+
+  /* copying FullCmd to ArV[0] is necessary for multiple restarts because
    * the path changes when we run pbs_server in the background. */
-  
+
   if (strcmp(FullCmd,ArgV[0]) != 0)
     {
     free(ArgV[0]);
-    
+
     ArgV[0] = calloc(sizeof(char), (strlen(FullCmd) + 1));
-    
+
     if (ArgV[0] == NULL)
       {
       /* could not calloc */
-      
+
       log_err(errno, __func__, "ERROR:     cannot allocate memory for full command, cannot restart\n");
-      
+
       exit(-10);
       }
-    
+
     strcpy(ArgV[0],FullCmd);
     }
 
   sprintf(log_buf, "INFO:     about to exec '%s'\n", ArgV[0]);
 
   log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, __func__, log_buf);
-  
+
   pthread_mutex_lock(log_mutex);
   log_close(1);
   pthread_mutex_unlock(log_mutex);
-  
+
   if ((rc = execv(FullCmd,ArgV)) == -1)
     {
     /* exec failed */
-    
+
     exit(-10);
     }
-  
+
   /* NOT REACHED */
-  
+
   exit(0);
-  
+
   return(SUCCESS);
   }  /* END svr_restart() */
 
@@ -3075,12 +3152,12 @@ int svr_restart()
 
 /**
  *
- * restores this attribute to its default where supported/possible
+ * restores this pbs_attribute to its default where supported/possible
  */
 
 void restore_attr_default(
 
-  struct attribute *attr) /* I */
+  pbs_attribute *attr) /* I */
 
   {
   int index;
@@ -3119,11 +3196,11 @@ void restore_attr_default(
 
       server.sv_attr[SRV_ATR_LogLevel].at_val.at_long = 0;
 
-      break; 
+      break;
 
     default:
 
-      /* should never get here, but if we do then reset the flags so the user knows 
+      /* should never get here, but if we do then reset the flags so the user knows
        * that the value hasn't been cleared */
 
       attr->at_flags |= ATR_VFLAG_SET;
@@ -3133,6 +3210,35 @@ void restore_attr_default(
 
   } /* END restore_attr_default() */
 
+int lock_sv_qs_mutex(pthread_mutex_t *sv_qs_mutex, const char *msg_string)
+  {
+  int rc;
+  char log_buf[LOCAL_LOG_BUF_SIZE + 1];
+
+  if (LOGLEVEL >= 7)
+    {
+    snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "%s: locking sv_qs_mutex", msg_string);
+    log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, __func__, log_buf);
+    }
+
+  rc = pthread_mutex_lock(sv_qs_mutex);
+  return(rc);
+  }
+
+int unlock_sv_qs_mutex(pthread_mutex_t *sv_qs_mutex, const char *msg_string)
+  {
+  int rc;
+  char log_buf[LOCAL_LOG_BUF_SIZE + 1];
+
+  if (LOGLEVEL >= 7)
+    {
+    snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "%s: unlocking sv_qs_mutex", msg_string);
+    log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, __func__, log_buf);
+    }
+
+  rc = pthread_mutex_unlock(sv_qs_mutex);
+  return(rc);
+  }
 
 /* END pbsd_main.c */
 
