@@ -82,9 +82,11 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include <sys/time.h>
 #include <memory.h>
 #include <time.h>
+#include <pwd.h>
 #include <sys/param.h>
 #if HAVE_SYS_UCRED_H
 #include <sys/ucred.h>
@@ -305,6 +307,8 @@ int get_creds(
  *    That function MUST free the request by calling free_br()
  *
  * NOTE: the caller functions hold the mutex for the connection
+ * NOTE: The socket connection must be returned in an open state.
+ *       The connection will be closed by start_process_pbs_server_port.
  */
 
 int process_request(
@@ -326,7 +330,7 @@ int process_request(
   unsigned short        conn_socktype;
   unsigned short        conn_authen;
   unsigned long         conn_addr;
-  int sfds = chan->sock;
+  int                   sfds = chan->sock;
 
   pthread_mutex_lock(svr_conn[sfds].cn_mutex);
   conn_active = svr_conn[sfds].cn_active;
@@ -444,7 +448,6 @@ int process_request(
     }
 
   /* is the request from a host acceptable to the server */
-
   if (conn_socktype & PBS_SOCK_UNIX)
     {
     strcpy(request->rq_host, server_name);
@@ -546,7 +549,9 @@ int process_request(
       /* If munge_on is true we will validate the connection now */
       if (request->rq_type == PBS_BATCH_AltAuthenUser)
         {
-        rc = req_altauthenuser(request); 
+        rc = req_altauthenuser(request);
+        free_request = FALSE;
+        goto process_request_cleanup;
         }
       else
         {
@@ -654,13 +659,10 @@ int process_request(
 
   rc = dispatch_request(sfds, request);
 
-
   return(rc);
 
 process_request_cleanup:
 
-  /*close_conn(sfds, FALSE);*/
-
   if (free_request == TRUE)
     free_br(request);
 
@@ -679,12 +681,11 @@ process_request_cleanup:
 
 int dispatch_request(
 
-  int          sfds,    /* I */
+  int                   sfds,    /* I */
   struct batch_request *request) /* I */
 
   {
-  int  rc = PBSE_NONE;
-  char *id = "dispatch_request";
+  int   rc = PBSE_NONE;
   char  log_buf[LOCAL_LOG_BUF_SIZE];
   char *job_id = NULL;
 
@@ -694,7 +695,7 @@ int dispatch_request(
       reqtype_to_txt(request->rq_type),
       sfds);
 
-    log_record(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,id,log_buf);
+    log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
     }
 
   switch (request->rq_type)
@@ -970,7 +971,7 @@ int dispatch_request(
       break;
     }  /* END switch (request->rq_type) */
 
-  return rc;
+  return(rc);
   }  /* END dispatch_request() */
 
 
@@ -987,22 +988,14 @@ struct batch_request *alloc_br(
   {
 
   struct batch_request *req = NULL;
-  int rc = PBSE_NONE;
 
-  memmgr *mm = NULL;
-  if ((rc = memmgr_init(&mm, 0)) != PBSE_NONE)
+  if ((req = (struct batch_request *)calloc(1, sizeof(struct batch_request))) == NULL)
     {
-    printf("rc = %d\n", rc);
-    log_err(errno, "alloc_br", "memmgr_init in alloc_br failed");
-    }
-  else if ((req = (struct batch_request *)memmgr_calloc(&mm, 1, sizeof(struct batch_request))) == NULL)
-    {
-    log_err(errno, "alloc_br", msg_err_malloc);
+    log_err(errno, __func__, msg_err_malloc);
     }
   else
     {
 
-    req->mm = mm;
     req->rq_type = type;
 
     req->rq_conn = -1;  /* indicate not connected */
@@ -1017,9 +1010,12 @@ struct batch_request *alloc_br(
 
 
 
-int close_quejob_by_jobid(char *job_id)
+int close_quejob_by_jobid(
+    
+  char *job_id)
+
   {
-  int rc = PBSE_NONE;
+  int  rc = PBSE_NONE;
   job *pjob = NULL;
 
   if ((pjob = find_job(job_id)) == NULL)
@@ -1038,17 +1034,21 @@ int close_quejob_by_jobid(char *job_id)
     pjob->ji_qs.ji_state = JOB_STATE_QUEUED;
     pjob->ji_qs.ji_substate = JOB_SUBSTATE_QUEUED;
     rc = svr_enquejob(pjob, FALSE, -1);
-    if (rc == PBSE_JOBNOTFOUND)
+
+    if ((rc == PBSE_JOBNOTFOUND) ||
+        (rc == PBSE_JOB_RECYCLED))
       {
       pjob = NULL;
       }
     else if (rc != PBSE_NONE)
       job_abt(&pjob, msg_err_noqueue);
     }
+
   if (pjob != NULL)
     pthread_mutex_unlock(pjob->ji_mutex);
-  return rc;
-  }
+
+  return(rc);
+  } /* close_quejob_by_jobid() */
 
 
 
@@ -1062,8 +1062,12 @@ void free_br(
   struct batch_request *preq)
 
   {
-  memmgr *mm = preq->mm;
-
+  if (preq->rq_id != NULL)
+    {
+    remove_batch_request(preq->rq_id);
+    free(preq->rq_id);
+    preq->rq_id = NULL;
+    }
 
   reply_free(&preq->rq_reply);
 
@@ -1190,7 +1194,7 @@ void free_br(
       break;
     }  /* END switch (preq->rq_type) */
 
-  memmgr_destroy(&mm);
+  free(preq);
 
   return;
   }  /* END free_br() */
