@@ -282,7 +282,6 @@ int execute_job_delete(
   struct batch_request *preq)            /* I */
 
   {
-  static char      *id = "execute_job_delete";
   struct work_task *pwtnew;
 
   int               rc;
@@ -451,8 +450,9 @@ jump:
      * Send signal request to MOM.  The server will automagically
      * pick up and "finish" off the client request when MOM replies.
      */
+    get_batch_request_id(preq);
 
-    if ((rc = issue_signal(&pjob, sigt, post_delete_mom1, preq)))
+    if ((rc = issue_signal(&pjob, sigt, post_delete_mom1, strdup(preq->rq_id))))
       {
       /* cant send to MOM */
 
@@ -537,7 +537,7 @@ jump:
 
       if (LOGLEVEL >= 7)
         {
-        sprintf(log_buf, "%s: unlocking ai_mutex", id);
+        sprintf(log_buf, "%s: unlocking ai_mutex", __func__);
         log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
         }
       pthread_mutex_unlock(pa->ai_mutex);
@@ -547,21 +547,17 @@ jump:
   if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_CHECKPOINT_FILE) != 0)
     {
     /* job has restart file at mom, do end job processing */
-    
     svr_setjobstate(pjob, JOB_STATE_EXITING, JOB_SUBSTATE_EXITING, FALSE);
 
+    /* force new connection */
     pjob->ji_momhandle = -1;
 
-    /* force new connection */
-    if(LOGLEVEL >= 7)
+    if (LOGLEVEL >= 7)
       {
-      sprintf(log_buf, "calling on_job_exit from %s", id);
-      log_event(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
-        log_buf);
+      sprintf(log_buf, "calling on_job_exit from %s", __func__);
+      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
       }
+
     set_task(WORK_Immed, 0, on_job_exit, strdup(pjob->ji_qs.ji_jobid), FALSE);
     }
   else if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_StagedIn) != 0)
@@ -590,11 +586,11 @@ jump:
       {
       pque->qu_numcompleted++;
 
-      unlock_queue(pque, id, NULL, LOGLEVEL);
+      unlock_queue(pque, __func__, NULL, LOGLEVEL);
       
       if (LOGLEVEL >= 7)
         {
-        sprintf(log_buf, "calling on_job_exit from %s", id);
+        sprintf(log_buf, "calling on_job_exit from %s", __func__);
         log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
         }
     
@@ -641,6 +637,8 @@ struct batch_request *duplicate_request(
     preq->rq_ind.rq_manager.rq_objname, PBS_MAXSVRJOBID + 1);
   memcpy(preq_tmp->rq_user, preq->rq_user, PBS_MAXUSER + 1);
   memcpy(preq_tmp->rq_host, preq->rq_host, PBS_MAXHOSTNAME + 1);
+  if (preq->rq_extend != NULL)
+    preq_tmp->rq_extend = strdup(preq->rq_extend);
 
   return(preq_tmp);
   } /* END duplicate_request() */
@@ -661,7 +659,7 @@ int handle_delete_all(
   int                   iter = -1;
   int                   failed_deletes = 0;
   int                   total_jobs = 0;
-  int                   rc;
+  int                   rc = PBSE_NONE;
   char                  tmpLine[MAXLINE];
 
   preq_dup->rq_noreply = TRUE;
@@ -672,8 +670,13 @@ int handle_delete_all(
     preq->rq_noreply = TRUE; /* set for no more replies */
     }
   
-  while ((pjob = next_job(&alljobs,&iter)) != NULL)
+  while ((pjob = next_job(&alljobs, &iter)) != NULL)
     {
+    if ((rc = forced_jobpurge(pjob, preq_dup)) == PURGE_SUCCESS)
+      {
+      continue;
+      }
+
     if (pjob->ji_qs.ji_state >= JOB_STATE_EXITING)
       {
       pthread_mutex_unlock(pjob->ji_mutex);
@@ -684,8 +687,14 @@ int handle_delete_all(
     total_jobs++;
     
     /* mutex is freed below */
-    if ((rc = forced_jobpurge(pjob,preq_dup)) == PBSE_NONE)
-      rc = execute_job_delete(pjob,Msg,preq_dup);
+    if (rc == PBSE_NONE)
+      {
+      if ((rc = execute_job_delete(pjob, Msg, preq_dup)) == PBSE_NONE)
+        reply_ack(preq_dup);
+       
+      /* mark this as NULL because it has been freed */
+      preq_dup = NULL;
+      }
     
     if (rc != PURGE_SUCCESS)
       {
@@ -700,7 +709,17 @@ int handle_delete_all(
     }
   
   if (failed_deletes == 0)
+    {
     reply_ack(preq);
+
+    /* PURGE SUCCESS means this was qdel -p all. In this case no reply_*() 
+     * functions have been called */
+    if (rc == PURGE_SUCCESS)
+      {
+      free_br(preq_dup);
+      preq_dup = NULL;
+      }
+    }
   else
     {
     snprintf(tmpLine,sizeof(tmpLine),"Deletes failed for %d of %d jobs",
@@ -709,6 +728,11 @@ int handle_delete_all(
     
     req_reject(PBSE_SYSTEM, 0, preq, NULL, tmpLine);
     }
+    
+  /* preq_dup happens at the end of the loop, so free the extra one if
+   * it is there */
+  if (preq_dup != NULL)
+    free_br(preq_dup);
 
   return(PBSE_NONE);
   } /* END handle_delete_all() */
@@ -742,8 +766,8 @@ int handle_single_delete(
       }
     
     /* mutex is freed below */
-    if ((rc = forced_jobpurge(pjob,preq)) == PBSE_NONE)
-      rc = execute_job_delete(pjob,Msg,preq);
+    if ((rc = forced_jobpurge(pjob, preq)) == PBSE_NONE)
+      rc = execute_job_delete(pjob, Msg, preq);
     }
   
   if ((rc == PBSE_NONE) ||
@@ -924,7 +948,10 @@ static void post_delete_route(
   struct work_task *pwt)
 
   {
-  req_deletejob((struct batch_request *)pwt->wt_parm1);
+  batch_request *preq = get_remove_batch_request((char *)pwt->wt_parm1);
+
+  if (preq != NULL)
+    req_deletejob(preq);
 
   free(pwt->wt_mutex);
   free(pwt);
@@ -952,17 +979,35 @@ static void post_delete_mom1(
 
   pbs_queue            *pque;
 
-  struct batch_request *preq_sig;  /* signal request to MOM */
+  char                 *preq_clt_id;
+  struct batch_request *preq_sig;         /* signal request to MOM */
 
-  struct batch_request *preq_clt;  /* original client request */
+  struct batch_request *preq_clt = NULL;  /* original client request */
   int                   rc;
   time_t                time_now = time(NULL);
 
-  preq_sig = pwt->wt_parm1;
-  rc       = preq_sig->rq_reply.brp_code;
-  preq_clt = preq_sig->rq_extra;
+  preq_sig = get_remove_batch_request((char *)pwt->wt_parm1);
+  
+  free(pwt->wt_mutex);
+  free(pwt);
 
-  release_req(pwt);
+  if (preq_sig == NULL)
+    return;
+
+  rc          = preq_sig->rq_reply.brp_code;
+  preq_clt_id = preq_sig->rq_extra;
+
+  free_br(preq_sig);
+
+  if (preq_clt_id != NULL)
+    {
+    preq_clt = get_remove_batch_request(preq_clt_id);
+    free(preq_clt_id);
+    }
+
+  /* the client request has been handled another way, nothing left to do */
+  if (preq_clt == NULL)
+    return;
 
   pjob = find_job(preq_clt->rq_ind.rq_delete.rq_objname);
 
@@ -1030,7 +1075,7 @@ static void post_delete_mom1(
                              &server.sv_attr[SRV_ATR_KillDelay],
                              2);
       pthread_mutex_unlock(server.sv_attr_mutex);
-      unlock_queue(pque, "post_delete_mom1", NULL, LOGLEVEL);
+      unlock_queue(pque, __func__, NULL, LOGLEVEL);
       }
     else if (pjob != NULL)
       return;
@@ -1056,7 +1101,6 @@ static void post_delete_mom2(
   struct work_task *pwt)
 
   {
-  static char *id = "post_delete_mom2";
   char        *jobid;
   char        *sigk = "SIGKILL";
   char         log_buf[LOCAL_LOG_BUF_SIZE];
@@ -1068,7 +1112,7 @@ static void post_delete_mom2(
   
   if (jobid == NULL)
     {
-    log_err(ENOMEM,id,"Cannot allocate memory");
+    log_err(ENOMEM, __func__, "Cannot allocate memory");
     return;
     }
 
@@ -1079,7 +1123,7 @@ static void post_delete_mom2(
     {
     if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING)
       {
-      issue_signal(&pjob, sigk, release_req, 0);
+      issue_signal(&pjob, sigk, release_req, NULL);
       
       if (pjob != NULL)
         {
@@ -1156,7 +1200,6 @@ int apply_job_delete_nanny(
   int         delay)  /* I */
 
   {
-  static char      *id = "apply_job_delete_nanny";
   enum work_type    tasktype;
   long              nanny = FALSE;
 
@@ -1178,7 +1221,7 @@ int apply_job_delete_nanny(
     }
   else
     {
-    log_err(-1, id, "negative delay requested for nanny");
+    log_err(-1, __func__, "negative delay requested for nanny");
 
     return(-1);
     }
@@ -1218,7 +1261,6 @@ static void job_delete_nanny(
   struct work_task *pwt)
 
   {
-  static char          *id = "job_delete_nanny";
   job                  *pjob;
   char                 *sigk = "SIGKILL";
   char                 *jobid;
@@ -1262,7 +1304,7 @@ static void job_delete_nanny(
       }
     else
       {
-      log_err(ENOMEM,id,"Cannot allocate memory");
+      log_err(ENOMEM, __func__, "Cannot allocate memory");
       }
     }
   
@@ -1295,14 +1337,21 @@ static void post_job_delete_nanny(
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
   long                  nanny = 0;
 
-  preq_sig = pwt->wt_parm1;
+  preq_sig = get_remove_batch_request((char *)pwt->wt_parm1);
+  
+  free(pwt->wt_mutex);
+  free(pwt);
+
+  if (preq_sig == NULL)    
+    return;
+
   rc       = preq_sig->rq_reply.brp_code;
 
   get_svr_attr_l(SRV_ATR_JobNanny, &nanny);
   if (!nanny)
     {
     /* the admin disabled nanny within the last minute or so */
-    release_req(pwt);
+    free_br(preq_sig);
 
     return;
     }
@@ -1326,7 +1375,7 @@ static void post_job_delete_nanny(
 
     set_resc_assigned(pjob, DECR);
   
-    release_req(pwt);
+    free_br(preq_sig);
 
     job_purge(pjob);
 
@@ -1336,7 +1385,7 @@ static void post_job_delete_nanny(
   pthread_mutex_unlock(pjob->ji_mutex);
 
   /* free task */
-  release_req(pwt);
+  free_br(preq_sig);
 
   return;
   } /* END post_job_delete_nanny() */
@@ -1354,7 +1403,6 @@ void purge_completed_jobs(
   struct batch_request *preq)  /* I */
 
   {
-  char         *id = "purge_completed_jobs";
   job          *pjob;
   char         *time_str;
   time_t        purge_time = 0;
@@ -1386,7 +1434,7 @@ void purge_completed_jobs(
     sprintf(log_buf,"Received purge completed jobs command, purge time is %ld (%s)",
       (long)purge_time, preq->rq_extend);
 
-    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_REQUEST,id,log_buf);
+    log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_REQUEST, __func__, log_buf);
     }
 
   iter = -1;
