@@ -5,7 +5,7 @@
 #include "batch_request.h" /* batch_request */
 #include "work_task.h" /* work_task */
 
-void *req_gpuctrl_svr(void *vp);
+int req_gpuctrl_svr(struct batch_request *preq);
 
 #ifdef NVIDIA_GPUS
 void process_gpu_request_reply(struct work_task *pwt);
