@@ -302,9 +302,9 @@ static int is_joined(
   enum job_atr  ati)   /* I */
 
   {
-  char       key;
-  attribute *pattr;
-  char     *pd;
+  char           key;
+  pbs_attribute *pattr;
+  char          *pd;
 
   if (ati == JOB_ATR_outpath)
     key = 'o';
@@ -394,15 +394,15 @@ static struct batch_request *cpy_stdfile(
   enum job_atr         ati) /* JOB_ATR_ output or error path */
 
   {
-  char *from;
-  char  key;
+  char          *from;
+  char           key;
 
-  attribute *jkpattr;
-  attribute *pathattr = &pjob->ji_wattr[ati];
+  pbs_attribute *jkpattr;
+  pbs_attribute *pathattr = &pjob->ji_wattr[ati];
 
-  char *suffix;
-  char *to = NULL;
-  char  log_buf[LOCAL_LOG_BUF_SIZE+1];
+  char          *suffix;
+  char          *to = NULL;
+  char           log_buf[LOCAL_LOG_BUF_SIZE];
 
   if ((pjob->ji_wattr[JOB_ATR_interactive].at_flags) &&
       (pjob->ji_wattr[JOB_ATR_interactive].at_val.at_long))
@@ -448,7 +448,7 @@ static struct batch_request *cpy_stdfile(
     }
 
   /*
-   * If the job has a keep file attribute, and the specified file is in
+   * If the job has a keep file pbs_attribute, and the specified file is in
    * the keep list, MOM has already placed the file in the user's HOME
    * directory.  It doesn't need to be copied.
    */
@@ -534,7 +534,7 @@ struct batch_request *cpy_stage(
   {
   int                   i;
   char                 *from;
-  attribute            *pattr;
+  pbs_attribute        *pattr;
 
   struct array_strings *parst;
   char                 *plocal;
@@ -822,24 +822,22 @@ int handle_returnstd(
   int                   type)
 
   {
-  int rc = PBSE_NONE;
-  int         KeepSeconds = 0;
-  int         IsFaked = 0;
-  char       *namebuf2;
-  char        namebuf[MAXPATHLEN + 1];
-  char        log_buf[LOCAL_LOG_BUF_SIZE+1];
-  pbs_queue  *pque;
-  int handle = -1;
-  char job_id[PBS_MAXSVRJOBID+1];
-  char job_fileprefix[PBS_JOBBASE+1];
-  unsigned long job_momaddr;
-  unsigned short job_momport;
-  char *job_momname = NULL;
+  int            rc = PBSE_NONE;
+  int            KeepSeconds = 0;
+  int            IsFaked = 0;
+  char          *namebuf2;
+  char           namebuf[MAXPATHLEN + 1];
+  char           log_buf[LOCAL_LOG_BUF_SIZE+1];
+  pbs_queue     *pque;
+  int            handle = -1;
+  char           job_id[PBS_MAXSVRJOBID+1];
+  char           job_fileprefix[PBS_JOBBASE+1];
+  unsigned long  job_momaddr;
+  char          *job_momname = NULL;
 
   strcpy(job_id, pjob->ji_qs.ji_jobid);
   strcpy(job_fileprefix, pjob->ji_qs.ji_fileprefix);
   job_momaddr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
-  job_momport = pjob->ji_qs.ji_un.ji_exect.ji_momport;
   if (pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str == NULL)
     {
     pthread_mutex_unlock(pjob->ji_mutex);
@@ -1404,14 +1402,16 @@ handle_stagedel_cleanup:
 
 
 int handle_exited(
-  job          *pjob)
+
+  job *pjob)
+
   {
   struct batch_request *preq;
   pbs_queue            *pque;
   char                  log_buf[LOCAL_LOG_BUF_SIZE+1];
   int                   rc;
-  int handle = -1;
-  char job_id[PBS_MAXSVRJOBID+1];
+  int                   handle = -1;
+  char                  job_id[PBS_MAXSVRJOBID+1];
 
   strcpy(job_id, pjob->ji_qs.ji_jobid);
 
@@ -1433,7 +1433,7 @@ int handle_exited(
     else
       {
       pthread_mutex_unlock(pjob->ji_mutex);
-      if ((rc = issue_Drequest(handle, preq, NULL, 0)) != PBSE_NONE)
+      if ((rc = issue_Drequest(handle, preq, release_req, 0)) != PBSE_NONE)
         {
         snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "DeleteJob issue_Drequest failure, rc = %d", rc);
 
@@ -1448,12 +1448,12 @@ int handle_exited(
     }
   else
     pthread_mutex_unlock(pjob->ji_mutex);
-  free_br(preq);
+
   preq = NULL;
   
   if ((pjob = find_job(job_id)) == NULL)
     return PBSE_JOBNOTFOUND;
-  /* NOTE: we never check if MOM actually deleted the job */
+
   rel_resc(pjob); /* free any resc assigned to the job */
   
   if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
@@ -1475,15 +1475,17 @@ int handle_exited(
     {
     pque->qu_numcompleted++;
 
-    unlock_queue(pque, "handle_exited", NULL, LOGLEVEL);
+    unlock_queue(pque, __func__, NULL, LOGLEVEL);
     }
   else if (pjob == NULL)
     {
     log_err(PBSE_JOBNOTFOUND, __func__, "Job lost while acquiring queue");
     return(PBSE_JOBNOTFOUND);
     }
+
   pthread_mutex_unlock(pjob->ji_mutex);
-  return PBSE_NONE;
+
+  return(PBSE_NONE);
   } /* END handle_exited() */
 
 
@@ -1627,34 +1629,30 @@ int handle_complete_second_time(
   job *pjob)
 
   {
-  int rc = PBSE_NONE;
   char         log_buf[LOCAL_LOG_BUF_SIZE+1];
   time_t       time_now = time(NULL);
-  char job_id[PBS_MAXSVRJOBID+1];
+  int          rc = PBSE_NONE;
+  char         job_id[PBS_MAXSVRJOBID+1];
 
   strcpy(job_id, pjob->ji_qs.ji_jobid);
   
   if (((pjob->ji_wattr[JOB_ATR_reported].at_flags & ATR_VFLAG_SET) != 0) &&
       (pjob->ji_wattr[JOB_ATR_reported].at_val.at_long == 0))
     {
-    /* the job must report attribute but hasn't:
+    /* the job must report pbs_attribute but hasn't:
      * skip the job if it has not yet reported to the scheduler. */
     if (LOGLEVEL >= 7)
       {
       sprintf(log_buf, "Bypassing job %s waiting for purge completed command",
         job_id);
       
-      log_record(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,job_id,log_buf);
+      log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
       }
     
-    if(LOGLEVEL >= 7)
+    if (LOGLEVEL >= 7)
       {
       sprintf(log_buf, "calling on_job_exit from %s", __func__);
-      log_event(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      job_id,
-      log_buf);
+      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
       }
 
     set_task(WORK_Timed, time_now + JOBMUSTREPORTDEFAULTKEEP, on_job_exit, strdup(job_id), FALSE);
@@ -1667,7 +1665,7 @@ int handle_complete_second_time(
       pthread_mutex_unlock(pjob->ji_mutex);
     
     }
-  return rc;
+  return(rc); 
   } /* END handle_complete_second_time() */
 
 
@@ -1696,26 +1694,32 @@ void on_job_exit(
   struct work_task *ptask)  /* I */
 
   {
-  int rc = PBSE_NONE;
+  int                   rc = PBSE_NONE;
   job                  *pjob;
   struct batch_request *preq;
   char                 *job_id;
   int                   type = ptask->wt_type;
-  char                  log_buf[LOCAL_LOG_BUF_SIZE+1];
-
-  extern void remove_job_delete_nanny(struct job *);
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   if (ptask->wt_type != WORK_Deferred_Reply)
     {
     preq = NULL;
 
     job_id = strdup((char *)ptask->wt_parm1);
+
     free(ptask->wt_parm1);
+    free(ptask->wt_mutex);
+    free(ptask);
     }
   else
     {
-    preq = (struct batch_request *)ptask->wt_parm1;
-    if (preq->rq_extra != NULL)
+    preq = get_remove_batch_request(ptask->wt_parm1);
+      
+    free(ptask->wt_mutex);
+    free(ptask);
+
+    if ((preq != NULL) &&
+        (preq->rq_extra != NULL))
       {
       job_id = strdup((char *)preq->rq_extra);
       free(preq->rq_extra);
@@ -1723,20 +1727,14 @@ void on_job_exit(
       }
     else
       {
-      /* Something is broken. */
-      free(ptask->wt_mutex);
-      free(ptask);
       return;
       }
     }
-  
-  free(ptask->wt_mutex);
-  free(ptask);
 
   /* check for calloc errors */
   if (job_id == NULL)
     {
-    log_err(ENOMEM,__func__,"Cannot allocate memory!");
+    log_err(ENOMEM, __func__, "Cannot allocate memory!");
     return;
     }
 
@@ -1747,21 +1745,21 @@ void on_job_exit(
   if (pjob == NULL)
     {
     sprintf(log_buf, "%s called with INVALID jobid: %s", __func__, job_id);
-    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,"NULL",log_buf);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, "NULL", log_buf);
     free(job_id);
     return;
     }
   else
     {
-    sprintf(log_buf, "%s valid pjob: %p (substate=%d)",
+    sprintf(log_buf, "%s valid pjob: %s (substate=%d)",
       __func__,
-      (void *)pjob,
+      job_id,
       pjob->ji_qs.ji_substate);
     
-    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,job_id,log_buf);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
     }
 
-/* pjob is unlocked in all error cases */  
+  /* NOTE: pjob is unlocked in all error cases */
   /* MOM has killed everything it can kill, so we can stop the nanny */
   switch (pjob->ji_qs.ji_substate)
     {
@@ -1865,7 +1863,6 @@ void on_job_rerun(
   struct work_task *ptask)
 
   {
-  static char          *id = "on_job_rerun";
   int                   handle;
   int                   newstate;
   int                   newsubst;
@@ -1885,14 +1882,19 @@ void on_job_rerun(
     }
   else
     {
-    preq = (struct batch_request *)ptask->wt_parm1;
+    if ((preq = get_remove_batch_request(ptask->wt_parm1)) == NULL)
+      {
+      free(ptask->wt_mutex);
+      free(ptask);
+      }
+
     jobid = (char *)preq->rq_extra;
     }
 
   /* check for memory allocation */
   if (jobid == NULL)
     {
-    log_err(ENOMEM,id,"Cannot allocate memory");
+    log_err(ENOMEM, __func__, "Cannot allocate memory");
     if (preq != NULL)
       free_br(preq);
     free(ptask->wt_mutex);
@@ -1903,6 +1905,18 @@ void on_job_rerun(
   pjob = find_job(jobid);
   free(jobid);
 
+  /* the job has already exited */
+  if (pjob == NULL)
+    {
+    if (preq != NULL)
+      free_br(preq);
+
+    free(ptask->wt_mutex);
+    free(ptask);
+
+    return;
+    }
+
   if ((handle = mom_comm(pjob, on_job_rerun)) < 0)
     {
     pthread_mutex_unlock(pjob->ji_mutex);
@@ -1955,16 +1969,33 @@ void on_job_rerun(
         strcpy(preq->rq_ind.rq_rerun, pjob->ji_qs.ji_jobid);
 
         preq->rq_extra = strdup(pjob->ji_qs.ji_jobid);
+        jobid = strdup(pjob->ji_qs.ji_jobid); 
+        pthread_mutex_unlock(pjob->ji_mutex);
 
         if (issue_Drequest(handle, preq, on_job_rerun, 0) == 0)
           {
           /* request ok, will come back when its done */
-          pthread_mutex_unlock(pjob->ji_mutex);
+          free(jobid);
           free(ptask->wt_mutex);
           free(ptask);
 
           return;
           }
+        pjob = find_job(jobid);
+        if(pjob == NULL)
+          {
+          snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "Job %s removed during call to issue_Drequest", jobid );
+          log_event(
+              PBSEVENT_JOB,
+              PBS_EVENTCLASS_JOB,
+              __func__,
+              log_buf);
+          free(jobid);
+          free(ptask->wt_mutex);
+          free(ptask);
+          return;
+          }
+        free(jobid);
 
         /* cannot issue request to mom, set up as if mom returned error */
 
@@ -2229,8 +2260,8 @@ void on_job_rerun(
       if (preq != NULL)
         {
         strcpy(preq->rq_ind.rq_delete.rq_objname, pjob->ji_qs.ji_jobid);
-
-        preq->rq_extra = (void *)pjob;
+        jobid = strdup(pjob->ji_qs.ji_jobid); 
+        pthread_mutex_unlock(pjob->ji_mutex);
 
         rc = issue_Drequest(handle, preq, release_req, 0);
 
@@ -2245,6 +2276,22 @@ void on_job_rerun(
             pjob->ji_qs.ji_jobid,
             log_buf);
           }
+        pjob = find_job(jobid);
+        if(pjob == NULL)
+          {
+          snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "Job %s removed during call to issue_Drequest", jobid );
+          log_event(
+              PBSEVENT_JOB,
+              PBS_EVENTCLASS_JOB,
+              __func__,
+              log_buf);
+          free(jobid);
+          free(ptask->wt_mutex);
+          free(ptask);
+          return;
+          }
+        free(jobid);
+
 
         /* release_req will free preq and close connection */
         }
@@ -2281,7 +2328,6 @@ void on_job_rerun(
     }  /* END switch (pjob->ji_qs.ji_substate) */
 
   pthread_mutex_unlock(pjob->ji_mutex);
-    
   free(ptask->wt_mutex);
   free(ptask);
 
@@ -2303,7 +2349,10 @@ static void wait_for_send(
   struct work_task *ptask)
 
   {
-  req_jobobit((struct batch_request *)ptask->wt_parm1);
+  batch_request *preq = get_remove_batch_request(ptask->wt_parm1);
+
+  if (preq != NULL)
+    req_jobobit(preq);
 
   return;
   }
@@ -2410,9 +2459,9 @@ void encode_job_used(
   tlist_head *phead)  /* O */
 
   {
-  attribute  *at;
+  pbs_attribute  *at;
   attribute_def  *ad;
-  resource  *rs;
+  resource       *rs;
 
   at = &pjob->ji_wattr[JOB_ATR_resc_used];
   ad = &job_attr_def[JOB_ATR_resc_used];
@@ -2427,18 +2476,18 @@ void encode_job_used(
        rs = (resource *)GET_NEXT(rs->rs_link))
     {
     resource_def *rd = rs->rs_defin;
-    attribute     val;
+    pbs_attribute val;
     int           rc;
 
-    val = rs->rs_value; /* copy resource attribute */
+    val = rs->rs_value; /* copy resource pbs_attribute */
 
     rc = rd->rs_encode(
-
            &val,
            phead,
            ad->at_name,
            rd->rs_name,
-           ATR_ENCODE_CLIENT);
+           ATR_ENCODE_CLIENT,
+           ATR_DFLAG_ACCESS);
 
     if (rc < 0)
       break;
@@ -2475,7 +2524,7 @@ int req_jobobit(
   int                   newsubst;
   int                   local_errno = 0;
   char                 *pc;
-  char                 *tmp;
+  char                 *tmp = NULL;
   job                  *pjob;
   char                  job_id[PBS_MAXSVRJOBID+1];
   long  job_atr_hold;
@@ -2487,14 +2536,16 @@ int req_jobobit(
   char                  log_buf[LOCAL_LOG_BUF_SIZE+1];
   time_t                time_now = time(NULL);
   long                  events = 0;
+  pbs_net_t             mom_addr;
 
   strcpy(job_id, preq->rq_ind.rq_jobobit.rq_jid);  /* This will be needed later for logging after preq is freed. */
-  pjob = find_job(job_id);
 
   tmp = parse_servername(preq->rq_host, &dummy);
+  mom_addr = get_hostaddr(&local_errno, tmp);
 
+  pjob = find_job(job_id);
   if ((pjob == NULL) ||
-      (pjob->ji_qs.ji_un.ji_exect.ji_momaddr != get_hostaddr(&local_errno, tmp)))
+      (pjob->ji_qs.ji_un.ji_exect.ji_momaddr != mom_addr))
     {
     /* not found or from wrong node */
 
@@ -2588,7 +2639,7 @@ int req_jobobit(
    * save exit state, update the resources used, and reply to MOM.
    *
    * Note, must make the log/mail message about resources used BEFORE
-   * replying to MOM.  The reply will free the attribute list so it
+   * replying to MOM.  The reply will free the pbs_attribute list so it
    * cannot be used after the call to reply_ack();
    */
 
