@@ -97,7 +97,7 @@
  *      mgr_node_delete()       - top level function for effectively deleting a node
  *      mgr_node_create()       - top level function for creating a node
  * req_manager()  - process manager request (top level)
- * disallowed_types_chk() - at action for queue disallowed_types attribute
+ * disallowed_types_chk() - at action for queue disallowed_types pbs_attribute
  */
 #include <pbs_config.h>   /* the master config generated by configure */
 #include "req_manager.h"
@@ -143,20 +143,21 @@
 
 /* Global Data Items: */
 
-extern int LOGLEVEL;
-extern struct server server;
+extern int            LOGLEVEL;
+extern struct server  server;
 extern all_queues     svr_queues;
-extern attribute_def que_attr_def[];
-extern attribute_def svr_attr_def[];
+extern attribute_def  que_attr_def[];
+extern attribute_def  svr_attr_def[];
 extern attribute_def  node_attr_def[];   /* node attributes defs */
-extern int  svr_chngNodesfile;
-extern char *msg_attrtype;
-extern char *msg_daemonname;
-extern char *msg_manager;
-extern char *msg_man_cre;
-extern char *msg_man_del;
-extern char *msg_man_set;
-extern char *msg_man_uns;
+extern int            svr_chngNodesfile;
+extern char          *msg_attrtype;
+extern char          *msg_daemonname;
+extern char          *msg_manager;
+extern char          *msg_man_cre;
+extern char          *msg_man_del;
+extern char          *msg_man_set;
+extern char          *msg_man_uns;
+extern int            disable_timeout_check;
 
 
 extern int que_purge(pbs_queue *);
@@ -177,7 +178,7 @@ static char *all_nodes_str = "_All_";
  * the current queue type.  This is called when creating or setting
  * the attributes of a queue.
  *
- * Returns: NULL if all ok or name of bad attribute if not ok
+ * Returns: NULL if all ok or name of bad pbs_attribute if not ok
  */
 
 static char *check_que_attr(
@@ -225,12 +226,12 @@ static char *check_que_attr(
 
 int check_que_enable(
 
-  attribute *pattr,
-  void *pque, /* actually a queue* */
-  int mode)
+  pbs_attribute *pattr,
+  void          *pque, /* actually a queue* */
+  int            mode)
 
   {
-  attribute *datr;
+  pbs_attribute *datr;
 
   if (pattr->at_val.at_long != 0)
     {
@@ -259,16 +260,16 @@ int check_que_enable(
 
 
 /*
- * set_queue_type - check the requested value of the queue type attribute
+ * set_queue_type - check the requested value of the queue type pbs_attribute
  * and set qu_type accordingly.
  * This is the at_action() routine for QA_ATR_QType
  */
 
 int set_queue_type(
     
-  attribute *pattr,
-  void *pque,
-  int mode)
+  pbs_attribute *pattr,
+  void          *pque,
+  int            mode)
 
   {
   int   i;
@@ -311,7 +312,7 @@ int set_queue_type(
         }
       }
 
-    if (spectype != -1)   /* set up the attribute */
+    if (spectype != -1)   /* set up the pbs_attribute */
       {
       ((pbs_queue *)pque)->qu_qs.qu_type = spectype;
       free(pattr->at_val.at_str);
@@ -333,7 +334,7 @@ int set_queue_type(
 
 
 /*
- * mgr_log_attr - log the change of an attribute
+ * mgr_log_attr - log the change of an pbs_attribute
  */
 
 static void mgr_log_attr(
@@ -389,37 +390,37 @@ static void mgr_log_attr(
 
 static int mgr_set_attr(
 
-  attribute *pattr,  /* current attributes */
+  pbs_attribute *pattr,  /* current attributes */
   attribute_def *pdef,
-  int   limit,
-  svrattrl *plist,
-  int   privil,
-  int  *bad,
-  void  *parent,
-  int   mode)
+  int            limit,
+  svrattrl      *plist,
+  int            privil,
+  int           *bad,
+  void          *parent,
+  int            mode)
 
   {
-  int   index;
-  attribute *new;
-  attribute *pnew;
-  attribute *pold;
-  int   rc;
+  int            index;
+  pbs_attribute *new;
+  pbs_attribute *pnew;
+  pbs_attribute *pold;
+  int            rc;
 
   if (plist == (struct svrattrl *)0)
     {
     return(0);
     }
 
-  new = (attribute *)calloc((unsigned int)limit, sizeof(attribute));
+  new = (pbs_attribute *)calloc((unsigned int)limit, sizeof(pbs_attribute));
 
-  if (new == (attribute *)0)
+  if (new == NULL)
     {
     return(PBSE_SYSTEM);
     }
 
   /*
-   * decode the new attribute values which are in the request,
-   * copy the corresponding current attribute into a holding array
+   * decode the new pbs_attribute values which are in the request,
+   * copy the corresponding current pbs_attribute into a holding array
    * and update it with the newly decoded value
    */
 
@@ -439,8 +440,8 @@ static int mgr_set_attr(
     if (pnew->at_flags & ATR_VFLAG_MODIFY)
       {
       /*
-       * for each attribute which is to be modified, call the
-       * at_action routine for the attribute, if one exists, with the
+       * for each pbs_attribute which is to be modified, call the
+       * at_action routine for the pbs_attribute, if one exists, with the
        *  new value.  If the action fails, undo everything.
        */
 
@@ -458,6 +459,10 @@ static int mgr_set_attr(
           }
         }
 
+      if ((index == SRV_ATR_tcp_timeout) &&
+          (pnew->at_val.at_long < 300))
+        disable_timeout_check = TRUE;
+
       /* now replace the old values with any modified new values */
 
       (pdef + index)->at_free(pold);
@@ -499,12 +504,12 @@ static int mgr_set_attr(
 
 int mgr_unset_attr(
 
-  attribute *pattr,  /* current attributes */
+  pbs_attribute *pattr,  /* current attributes */
   attribute_def *pdef,
-  int   limit,
-  svrattrl *plist,
-  int   privil,
-  int  *bad)
+  int            limit,
+  svrattrl      *plist,
+  int            privil,
+  int           *bad)
 
   {
   int   index;
@@ -513,7 +518,7 @@ int mgr_unset_attr(
   resource_def *prsdef;
   resource *presc;
 
-  /* first check the attribute exists and we have privilege to set */
+  /* first check the pbs_attribute exists and we have privilege to set */
 
   ord = 0;
   pl = plist;
@@ -531,7 +536,7 @@ int mgr_unset_attr(
       return(PBSE_NOATTR);
       }
 
-    /* have we privilege to unset the attribute ? */
+    /* have we privilege to unset the pbs_attribute ? */
 
     if (((pdef + index)->at_flags & privil & ATR_DFLAG_WRACC) == 0)
       {
@@ -577,7 +582,7 @@ int mgr_unset_attr(
     if (((pdef + index)->at_type == ATR_TYPE_RESC) &&
         (plist->al_resc != NULL))
       {
-      /* free resource member, not the attribute */
+      /* free resource member, not the pbs_attribute */
 
       prsdef = find_resc_def(
                  svr_resc_def,
@@ -599,7 +604,7 @@ int mgr_unset_attr(
       }
     else
       {
-      /* free the whole attribute */
+      /* free the whole pbs_attribute */
 
       (pdef + index)->at_free(pattr + index);
 
@@ -627,7 +632,7 @@ int mgr_unset_attr(
  * otherwise updated.
  */
 
-static void update_subnode(
+void update_subnode(
 
   struct pbsnode *pnode)
 
@@ -663,44 +668,41 @@ static void update_subnode(
  * of its structure members, we will proceed as though it did by using the
  *      node to derive a temporary array of attributes.  We use the "action"
  *      function of our artificial attribute_def array with a mode value of NEW
- *      in constructing this temporary node attribute array. Once this temporary
+ *      in constructing this temporary node pbs_attribute array. Once this temporary
  *      array is constucted, the "svrattrl" request list is processed against it
- * to produce a new "updated" attribute array.  When all the attribute
- *      processing completes, assuming it's successful, the updated attribute
+ * to produce a new "updated" pbs_attribute array.  When all the pbs_attribute
+ *      processing completes, assuming it's successful, the updated pbs_attribute
  *      array is used to modify the node itself-- this can be done via each
- *      attribute's "action" function with the value of mode set to "ALTER".
- * Although it's a little twisted to use the idea of an attribute's
- * "action function" to move between node an attribute,it pushes the
+ *      pbs_attribute's "action" function with the value of mode set to "ALTER".
+ * Although it's a little twisted to use the idea of an pbs_attribute's
+ * "action function" to move between node an pbs_attribute,it pushes the
  *      details down a level, below that which is taking place here-- a good
  *      thing.
  */
 
 int mgr_set_node_attr(
 
-  struct pbsnode *pnode, /*do some kind of "set" action on this node */
-  attribute_def  *pdef,  /*pbsnode doesn't have attributes*/
-  /*but in the future it might  so,*/
-  /*make  code similar to existing */
-
-  int   limit,  /*length of the *pdef array      */
-  svrattrl *plist,  /*batch request's svrattrl list  */
-  /*begins here                    */
-  int   privil, /*requester's privilege          */
-  int  *bad,  /*if there is a "bad attribute"  */
-  /*pass back position via this loc*/
-  void  *parent, /*may go unused in this function */
-  int   mode)  /*passed to attrib's action func */
-/*not used by this func at this time*/
+  struct pbsnode *pnode,  /* do some kind of "set" action on this node */
+  attribute_def  *pdef,   /* pbsnode doesn't have attributes but in the
+                             future it might  so, make  code similar to existing */
+  int             limit,  /* length of the *pdef array      */
+  svrattrl       *plist,  /* batch request's svrattrl list begins here */
+  int             privil, /* requester's privilege          */
+  int            *bad,    /* if there is a "bad pbs_attribute" pass back 
+                             position via this loc */
+  void           *parent, /*may go unused in this function */
+  int             mode)  /*passed to attrib's action func not used by 
+                           this func at this time*/
 
   {
-  int   i;
-  int   index;
-  int   nstatus = 0;
-  int   nprops = 0;
-  int   rc;
-  attribute *new;
-  attribute *unused = (attribute *)0;
-  attribute *pnew;
+  int              i;
+  int              index;
+  int              nstatus = 0;
+  int              nprops = 0;
+  int              rc;
+  pbs_attribute   *new;
+  pbs_attribute   *unused = NULL;
+  pbs_attribute   *pnew;
 
   struct pbsnode   tnode;  /*temporary node*/
 
@@ -718,13 +720,13 @@ int mgr_set_node_attr(
     return(0);  /* nothing to do, return success */
     }
 
-  /* Get heap space for a temporary node-attribute array and use the
+  /* Get heap space for a temporary node-pbs_attribute array and use the
    * various "node-attribute action" functions defined in the file
    * "node_attr_def.c" to set the current values of each entry's
    * structure members
    */
 
-  new = (attribute *)calloc((unsigned int)limit, sizeof(attribute));
+  new = (pbs_attribute *)calloc((unsigned int)limit, sizeof(pbs_attribute));
 
   if (new == NULL)
     {
@@ -750,12 +752,12 @@ int mgr_set_node_attr(
 
   /*
    * The function "attr_atomic_node_set" does the following:
-   * successively decodes the new attribute modifications carried in the
-   * the request list, "plist", into a temporary attribute (on stack).
-   * Updates each attribute in the derived node-attribute array using the
+   * successively decodes the new pbs_attribute modifications carried in the
+   * the request list, "plist", into a temporary pbs_attribute (on stack).
+   * Updates each pbs_attribute in the derived node-attribute array using the
    * decoded values.  If a failure of some sort occurs, as evidenced by
    * a non-zero return code (rc), call attr_atomic_kill() to undo
-   * everything-- calls upon the "at_free" function for each attribute
+   * everything-- calls upon the "at_free" function for each pbs_attribute
    * to free hanging structures, then frees up the "new" array
    * return code (rc) shapes caller's reply
    */
@@ -768,11 +770,11 @@ int mgr_set_node_attr(
     }
 
   /*
-   * Use the updated attribute values in attribute array, "new", to update a
+   * Use the updated pbs_attribute values in pbs_attribute array, "new", to update a
    * temporary (on stack) pbsnode, which is a copy of the one pointed to by
    * pnode.  If all goes well, we are home free-- just copy the information
    * in the temporary pbsnode, after it has gotten modified, back to the
-   * node pointed to by "pnode" and free the attribute array "new".
+   * node pointed to by "pnode" and free the pbs_attribute array "new".
    */
 
   tnode = *pnode;
@@ -790,12 +792,12 @@ int mgr_set_node_attr(
     if (pnew->at_flags & ATR_VFLAG_MODIFY)
       {
       /*
-       * for each attribute which is to cause modification to the node, "tnode",
-       * call the specified "at_action" routine for that attribute passing it
-       * the address of the temporary attribute and the address of the temporary
+       * for each pbs_attribute which is to cause modification to the node, "tnode",
+       * call the specified "at_action" routine for that pbs_attribute passing it
+       * the address of the temporary pbs_attribute and the address of the temporary
        * pbsnode object.  If any of the update actions fail along the way, undo
        * everything.   Any calloc'd substructures should be hanging off the
-       * appropriate attribute of the attribute array "new".
+       * appropriate pbs_attribute of the pbs_attribute array "new".
        */
 
       if ((pdef + index)->at_action)
@@ -826,7 +828,7 @@ int mgr_set_node_attr(
     pnode->nd_prop = NULL;
     }
 
-  /* NOTE:  nd_status properly freed during attribute alter */
+  /* NOTE:  nd_status properly freed during pbs_attribute alter */
 
   if ((pnode->nd_state != tnode.nd_state))
     {
@@ -848,7 +850,7 @@ int mgr_set_node_attr(
     log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_NODE,pnode->nd_name,log_buf);
     }
 
-  /* NOTE:  nd_status properly freed during attribute alter */
+  /* NOTE:  nd_status properly freed during pbs_attribute alter */
 
   /*
     if ((pnode->nd_status != NULL) && (pnode->nd_status != tnode.nd_status))
@@ -874,7 +876,7 @@ int mgr_set_node_attr(
 
   free(new);  /*any new  prop list has been put on pnode*/
 
-  /*dispense with the attribute array itself*/
+  /*dispense with the pbs_attribute array itself*/
 
   /* update prop list based on new prop array */
 
@@ -1210,11 +1212,11 @@ void mgr_server_set(
 
   if (rc == PBSE_BADACLHOST)
     {
-    char     *bad_host;
-    char     *host_entry;
-    attribute temp;
-    int       index;
-    int       i;
+    char          *bad_host;
+    char          *host_entry;
+    pbs_attribute  temp;
+    int            index;
+    int            i;
 
     struct    array_strings *pstr;
     int       bhstrlen;
@@ -2232,9 +2234,9 @@ int req_manager(
 
 int manager_oper_chk(
 
-  attribute *pattr,
-  void      *pobject,
-  int      actmode)
+  pbs_attribute *pattr,
+  void          *pobject,
+  int            actmode)
 
   {
   char   *entry;
@@ -2300,9 +2302,9 @@ int manager_oper_chk(
 
 int servername_chk(
 
-  attribute *pattr,   /* I */
-  void      *pobject, /* I */
-  int      actmode) /* I */
+  pbs_attribute *pattr,   /* I */
+  void          *pobject, /* I */
+  int           actmode) /* I */
 
   {
   int    err = 0;
@@ -2358,7 +2360,7 @@ int servername_chk(
 
 
 /*
- * extra_resc_chk() called when extra_resc server attribute is changed
+ * extra_resc_chk() called when extra_resc server pbs_attribute is changed
  *     It's purpose is to re-init the resource definitions by calling
  *     init_resc_defs().  Unfortunately, it gets call before the
  *     change is actually committed; so we setup a work task to call
@@ -2377,9 +2379,9 @@ void on_extra_resc(
 
 int extra_resc_chk(
 
-  attribute *pattr,   /* I */
-  void      *pobject, /* I */
-  int      actmode) /* I */
+  pbs_attribute *pattr,   /* I */
+  void          *pobject, /* I */
+  int            actmode) /* I */
 
   {
   /* Is there anything to validate?  Maybe check for all alphanum? */
@@ -2395,7 +2397,7 @@ int extra_resc_chk(
  */
 void free_extraresc(
 
-  struct attribute *attr)
+  pbs_attribute *attr)
 
   {
   set_task(WORK_Immed, 0, on_extra_resc, NULL, FALSE);
@@ -2406,14 +2408,14 @@ void free_extraresc(
 
 /*
  * disallowed_types_chk -
- *      This is the at_action() routine for the queue disallowed_type attribute
+ *      This is the at_action() routine for the queue disallowed_type pbs_attribute
  */
 
 int disallowed_types_chk(
 
-  attribute *pattr,
-  void      *pobject,
-  int        actmode)
+  pbs_attribute *pattr,
+  void          *pobject,
+  int            actmode)
 
   {
   int i;
@@ -2472,13 +2474,12 @@ int disallowed_types_chk(
 
 static int mgr_long_action_helper(
 
-  attribute *pattr,
-  int        actmode,
-  long       minimum,
-  long       maximum)
+  pbs_attribute *pattr,
+  int            actmode,
+  long           minimum,
+  long           maximum)
 
   {
-
   int err = PBSE_NONE;
 
   if (pattr->at_val.at_long < minimum)
@@ -2506,15 +2507,15 @@ static int mgr_long_action_helper(
 
 /*
  * schiter_chk - make sure values are sane
- * This is the at_action() routine for the server attribute
+ * This is the at_action() routine for the server pbs_attribute
  * "scheduler_iteration"
  */
 
 int schiter_chk(
 
-  attribute *pattr,
-  void      *pobject,
-  int        actmode)
+  pbs_attribute *pattr,
+  void          *pobject,
+  int            actmode)
 
   {
   return(mgr_long_action_helper(pattr, actmode, 1, PBS_SCHEDULE_CYCLE));
@@ -2524,14 +2525,14 @@ int schiter_chk(
 
 /* nextjobnum_action - makes sure value is sane (>=0)
    Note that there must be special code in svr_recov
-   to prevent the attribute value from clobbering
+   to prevent the pbs_attribute value from clobbering
    the internal counter server.sv_qs.sv_jobidnumber.
  */
 int nextjobnum_chk(
 
-  attribute *pattr,
-  void      *pobject,
-  int        actmode)
+  pbs_attribute *pattr,
+  void          *pobject,
+  int            actmode)
 
   {
   if (pattr->at_val.at_long > PBS_SEQNUMTOP)
@@ -2550,11 +2551,12 @@ int nextjobnum_chk(
 
 int set_nextjobnum(
 
-  struct attribute *attr,
-  struct attribute *new,
-  enum batch_op op)
+  pbs_attribute *attr,
+  pbs_attribute *new,
+  enum batch_op  op)
 
   {
+  char log_buf[LOCAL_LOG_BUF_SIZE + 1];
   /* this is almost identical to set_l, but we need to grab the
      current value of server.sv_qs.sv_jobidnumber before we INCR/DECR the value of
      attr->at_val.at_long.  In fact, it probably should be moved to Libattr/ */
@@ -2564,7 +2566,7 @@ int set_nextjobnum(
    * Otherwise, if the number is in the default range, it sets it to the new range.
    */
 
-  pthread_mutex_lock(server.sv_qs_mutex);
+  lock_sv_qs_mutex(server.sv_qs_mutex, __func__);
   switch (op)
     {
 
@@ -2591,7 +2593,8 @@ int set_nextjobnum(
 
     default:
      
-      pthread_mutex_unlock(server.sv_qs_mutex);
+      sprintf(log_buf, "%s:1", __func__);
+      unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
       return(PBSE_SYSTEM);
     }
 
@@ -2599,7 +2602,8 @@ int set_nextjobnum(
 
   server.sv_qs.sv_jobidnumber = attr->at_val.at_long;
 
-  pthread_mutex_unlock(server.sv_qs_mutex);
+  sprintf(log_buf, "%s:2", __func__);
+  unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
 
   return(PBSE_NONE);
   } /* END set_nextjobnum() */
