@@ -153,12 +153,18 @@ static void post_modify_req(
   {
 
   struct batch_request *preq;
-  job  *pjob;
+  job                  *pjob;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   svr_disconnect(pwt->wt_event);  /* close connection to MOM */
 
-  preq = pwt->wt_parm1;
+  preq = get_remove_batch_request(pwt->wt_parm1);
+
+  free(pwt->wt_mutex);
+  free(pwt);
+
+  if (preq == NULL)
+    return;
 
   preq->rq_conn = preq->rq_orgconn;  /* restore socket to client */
 
@@ -181,8 +187,6 @@ static void post_modify_req(
       if ((pjob = find_job(preq->rq_ind.rq_modify.rq_objname)) == NULL)
         {
         req_reject(preq->rq_reply.brp_code, 0, preq, NULL, NULL);
-        free(pwt->wt_mutex);
-        free(pwt);
         return;
         }
       else
@@ -205,9 +209,6 @@ static void post_modify_req(
     reply_ack(preq);
     }
 
-  free(pwt->wt_mutex);
-  free(pwt);
-
   return;
   }  /* END post_modify_req() */
 
@@ -225,8 +226,6 @@ void mom_cleanup_checkpoint_hold(
   struct work_task *ptask)
 
   {
-  static char          *id = "mom_cleanup_checkpoint_hold";
-
   int                   rc = 0;
   job                  *pjob;
   char                 *jobid;
@@ -241,7 +240,7 @@ void mom_cleanup_checkpoint_hold(
 
   if (jobid == NULL)
     {
-    log_err(ENOMEM,id,"Cannot allocate memory");
+    log_err(ENOMEM, __func__, "Cannot allocate memory");
     return;
     }
 
@@ -252,7 +251,7 @@ void mom_cleanup_checkpoint_hold(
       {
       sprintf(log_buf,
         "%s:failed to find job\n",
-        id);
+        __func__);
 
       log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,jobid,log_buf);
       }
@@ -280,7 +279,7 @@ void mom_cleanup_checkpoint_hold(
     {
     if ((preq = alloc_br(PBS_BATCH_DeleteJob)) == NULL)
       {
-      log_err(-1, id, "unable to allocate DeleteJob request - big trouble!");
+      log_err(-1, __func__, "unable to allocate DeleteJob request - big trouble!");
       }
     else
       {
@@ -295,7 +294,7 @@ void mom_cleanup_checkpoint_hold(
             "Unable to relay information to mom for job '%s'\n",
             pjob->ji_qs.ji_jobid);
           
-          log_err(rc,id,log_buf);
+          log_err(rc, __func__, log_buf);
           
           pthread_mutex_unlock(pjob->ji_mutex);
           }
@@ -335,15 +334,22 @@ void chkpt_xfr_hold(
   struct work_task *ptask)
 
   {
-  job       *pjob;
+  job                  *pjob;
 
   struct batch_request *preq;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
-  preq = (struct batch_request *)ptask->wt_parm1;
-  pjob = (job *)preq->rq_extra;
+  preq = get_remove_batch_request(ptask->wt_parm1);
 
-  pthread_mutex_lock(pjob->ji_mutex);
+  free(ptask->wt_mutex);
+  free(ptask);
+
+  if ((preq == NULL) ||
+      (preq->rq_extra == NULL))
+    return;
+
+  if ((pjob = find_job(preq->rq_extra)) == NULL)
+    return;
 
   if (LOGLEVEL >= 7)
     {
@@ -352,10 +358,10 @@ void chkpt_xfr_hold(
       PJobState[pjob->ji_qs.ji_state],
       PJobSubState[pjob->ji_qs.ji_substate]);
 
-    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
     }
   
-  release_req(ptask);
+  free_br(preq);
 
   set_task(WORK_Immed, 0, mom_cleanup_checkpoint_hold, strdup(pjob->ji_qs.ji_jobid), FALSE);
 
@@ -424,8 +430,8 @@ int modify_job(
   
   if (pjob == NULL)
     {
-    sprintf(log_buffer, "job structure is NULL");
-    log_err(PBSE_IVALREQ, __func__, log_buffer);
+    sprintf(log_buf, "job structure is NULL");
+    log_err(PBSE_IVALREQ, __func__, log_buf);
     return(PBSE_IVALREQ);
     }
 
@@ -491,7 +497,7 @@ int modify_job(
     {
     while (plist != NULL)
       {
-      /* is the attribute modifiable in RUN state ? */
+      /* is the pbs_attribute modifiable in RUN state ? */
 
       i = find_attr(job_attr_def, plist->al_name, JOB_ATR_LAST);
 
@@ -635,8 +641,7 @@ int modify_job(
     if (momreq != NULL)
       {
       /* have files to copy */
-
-      momreq->rq_extra = (void *)pjob;
+      momreq->rq_extra = strdup(pjob->ji_qs.ji_jobid);
 
       /* The momreq is freed in relay_to_mom (failure)
        * or in issue_Drequest (success) */
@@ -841,7 +846,6 @@ int modify_whole_array(
   int        checkpoint_req)  /* I */
 
   {
-  char  id[] = "modify_whole_array";
   int   i;
   int   rc = 0;
   int   mom_relay = 0;
@@ -890,7 +894,7 @@ int modify_whole_array(
             snprintf(log_buf,sizeof(log_buf),
               "Unable to relay information to mom for job '%s'\n",
               pjob->ji_qs.ji_jobid);
-            log_err(rc,id,log_buf);
+            log_err(rc, __func__, log_buf);
             pthread_mutex_unlock(pjob->ji_mutex);
             }
 
@@ -929,8 +933,7 @@ void *req_modifyarray(
   void *vp) /* I */
 
   {
-  char                 id[] = "req_modifyarray";
-  char                 log_buf[LOCAL_LOG_BUF_SIZE];
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
   job_array            *pa;
   job                  *pjob = NULL;
   svrattrl             *plist;
@@ -1004,10 +1007,10 @@ void *req_modifyarray(
        (rc != PBSE_RELAYED_TO_MOM))
       {
       pthread_mutex_unlock(pa->ai_mutex);
-      if(LOGLEVEL >= 7)
+      if (LOGLEVEL >= 7)
         {
-        sprintf(log_buf, "%s: unlocked ai_mutex", id);
-        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, id, log_buf);
+        sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
         }
 
       req_reject(PBSE_IVALREQ,0,preq,NULL,"Error reading array range");
@@ -1021,10 +1024,10 @@ void *req_modifyarray(
     if (rc == PBSE_RELAYED_TO_MOM)
       {
       pthread_mutex_unlock(pa->ai_mutex);
-      if(LOGLEVEL >= 7)
+      if (LOGLEVEL >= 7)
         {
-        sprintf(log_buf, "%s: unlocked ai_mutex", id);
-        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, id, log_buf);
+        sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
         }
 
       return(NULL);
@@ -1038,10 +1041,10 @@ void *req_modifyarray(
         (rc != PBSE_RELAYED_TO_MOM))
       {
       pthread_mutex_unlock(pa->ai_mutex);
-      if(LOGLEVEL >= 7)
+      if (LOGLEVEL >= 7)
         {
-        sprintf(log_buf, "%s: unlocked ai_mutex", id);
-        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, id, log_buf);
+        sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
         }
 
       req_reject(PBSE_IVALREQ,0,preq,NULL,"Error altering the array");
@@ -1063,7 +1066,7 @@ void *req_modifyarray(
       pthread_mutex_unlock(pa->ai_mutex);
       if (LOGLEVEL >= 7)
         {
-        sprintf(log_buf, "%s: unlocked ai_mutex", id);
+        sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
         log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
         }
       pthread_mutex_unlock(pjob->ji_mutex);
@@ -1075,9 +1078,9 @@ void *req_modifyarray(
     if (rc == PBSE_RELAYED_TO_MOM)
       {
       pthread_mutex_unlock(pa->ai_mutex);
-      if(LOGLEVEL >= 7)
+      if (LOGLEVEL >= 7)
         {
-        sprintf(log_buf, "%s: unlocked ai_mutex", id);
+        sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
         log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
         }
       pthread_mutex_unlock(pjob->ji_mutex);
@@ -1091,7 +1094,7 @@ void *req_modifyarray(
   /* SUCCESS */
   if (LOGLEVEL >= 7)
     {
-    sprintf(log_buf, "%s: unlocked ai_mutex", id);
+    sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
     log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pa->ai_qs.parent_id, log_buf);
     }
   pthread_mutex_unlock(pa->ai_mutex);
@@ -1207,13 +1210,13 @@ int modify_job_attr(
   int      *bad)   /* O */
 
   {
-  int        allow_unkn = -1;
-  long       i;
-  attribute  newattr[JOB_ATR_LAST];
-  attribute *pattr;
-  int        rc;
-  char       log_buf[LOCAL_LOG_BUF_SIZE];
-  pbs_queue *pque;
+  int            allow_unkn = -1;
+  long           i;
+  pbs_attribute  newattr[JOB_ATR_LAST];
+  pbs_attribute *pattr;
+  int            rc;
+  char           log_buf[LOCAL_LOG_BUF_SIZE];
+  pbs_queue     *pque;
 
   if ((pque = get_jobs_queue(&pjob)) != NULL)
     {
@@ -1253,7 +1256,7 @@ int modify_job_attr(
 
       if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING)
         {
-        long lim = 0;
+        long lim = TRUE;
         int comp_resc_lt;
        
         get_svr_attr_l(SRV_ATR_QCQLimits, &lim);
@@ -1455,7 +1458,14 @@ void post_modify_arrayreq(
 
   svr_disconnect(pwt->wt_event);  /* close connection to MOM */
 
-  preq = pwt->wt_parm1;
+  preq = get_remove_batch_request(pwt->wt_parm1);
+
+  free(pwt->wt_mutex);
+  free(pwt);
+
+  if (preq == NULL)
+    return;
+
   parent_req = preq->rq_extra; /* This is the original batch_request allocated by process_request */
 
   preq->rq_conn = preq->rq_orgconn;  /* restore socket to client */
@@ -1483,9 +1493,6 @@ void post_modify_arrayreq(
         {
         parent_req->rq_refcount--;
 
-        free(pwt->wt_mutex);
-        free(pwt);
-
         if (parent_req->rq_refcount == 0)
           {
           free_br(preq);
@@ -1526,9 +1533,6 @@ void post_modify_arrayreq(
       free_br(preq);
     }
 
-  free(pwt->wt_mutex);
-  free(pwt);
-
   return;
   }  /* END post_modify_arrayreq() */
 
