@@ -122,9 +122,6 @@ int req_movejob(
   void *vp) /* I */
 
   {
-#ifndef NDEBUG
-  char *id = "req_movejob";
-#endif
   job *jobp;
   struct batch_request *req = (struct batch_request *)vp;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
@@ -142,7 +139,7 @@ int req_movejob(
       (jobp->ji_qs.ji_state != JOB_STATE_WAITING))
     {
 #ifndef NDEBUG
-    sprintf(log_buf, "%s %d %s", pbse_to_txt(PBSE_BADSTATE), jobp->ji_qs.ji_state, id);
+    sprintf(log_buf, "%s %d %s", pbse_to_txt(PBSE_BADSTATE), jobp->ji_qs.ji_state, __func__);
 
     log_event(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,jobp->ji_qs.ji_jobid,log_buf);
 #endif /* NDEBUG */
@@ -216,9 +213,6 @@ int req_orderjob(
   void *vp) /* I */
 
   {
-#ifndef NDEBUG
-  static char          *id = "req_orderjob";
-#endif
   job                  *pjob;
   job                  *pjob1;
   job                  *pjob2;
@@ -250,7 +244,7 @@ int req_orderjob(
             pbse_to_txt(PBSE_BADSTATE),
             pjob->ji_qs.ji_state);
 
-    strcat(log_buf, id);
+    strcat(log_buf, __func__);
 
     log_event(
       PBSEVENT_DEBUG,
@@ -330,17 +324,15 @@ int req_orderjob(
     strcpy(job_id2, pjob2->ji_qs.ji_jobid);
     pthread_mutex_unlock(pjob1->ji_mutex);
     pthread_mutex_unlock(pjob2->ji_mutex);
-    pjob1 = NULL;
-    pjob2 = NULL;
 
     svr_dequejob(job_id1, FALSE);
     svr_dequejob(job_id2, FALSE);
 
-    if ((pjob1 = find_job(job_id1)) != NULL)
-      svr_enquejob(pjob1, FALSE, -1);
+    if (svr_enquejob(pjob1, FALSE, -1) == PBSE_JOB_RECYCLED)
+      pjob1 = NULL;
 
-    if ((pjob2 = find_job(job_id2)) != NULL)
-      svr_enquejob(pjob2, FALSE, -1);
+    if (svr_enquejob(pjob2, FALSE, -1) == PBSE_JOB_RECYCLED)
+      pjob2 = NULL;
     }
   else
     {
@@ -365,7 +357,9 @@ int req_orderjob(
     pthread_mutex_unlock(pjob2->ji_mutex);
     }
 
+  /* SUCCESS */
   reply_ack(req);
+
   return(PBSE_NONE);
   }  /* END req_orderjob() */
 
