@@ -121,8 +121,11 @@
 #include "array.h"
 #include "queue_func.h" /* get_dfltque, find_queuebyname */
 #include "svr_func.h" /* get_svr_attr_* */
+#include "array_func.h" /* setup_array_struct */
 #include "threadpool.h"
 #include "job_func.h" /* job_purge */
+#include "pbs_nodes.h"
+#include "../lib/Libutils/u_lock_ctl.h" /* lock_node, unlock_node */
 
 
 #include "work_task.h"
@@ -136,7 +139,6 @@ extern int   svr_chkque(job *, pbs_queue *, char *, int, char *);
 extern int   job_route(job *);
 extern int   node_avail_complex(char *, int *, int *, int *, int*);
 extern void  set_chkpt_deflt(job *, pbs_queue *);
-int          setup_array_struct(job *pjob);
 void        *job_clone_wt(void *);
 
 /* Global Data Items: */
@@ -494,14 +496,13 @@ int req_quejob(
   char                  buf[256];
   char                  EMsg[MAXPATHLEN];
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
-
   time_t                time_now = time(NULL);
 
   job                  *pj;
   attribute_def        *pdef;
   svrattrl             *psatl;
   pbs_queue            *pque;
-  attribute             tempattr;
+  pbs_attribute         tempattr;
   char                 *alias = NULL;
   int                   jobid_number;
   
@@ -543,7 +544,7 @@ int req_quejob(
 
     get_svr_attr_l(SRV_ATR_display_job_server_suffix, &server_suffix);
 
-    pthread_mutex_lock(server.sv_qs_mutex);
+    lock_sv_qs_mutex(server.sv_qs_mutex, __func__);
 
     if ((alias != NULL) &&
         (server_suffix == TRUE))
@@ -594,7 +595,7 @@ int req_quejob(
       server.sv_qs.sv_jobidnumber = 0; /* wrap it */
 
     jobid_number = server.sv_qs.sv_jobidnumber;
-    pthread_mutex_unlock(server.sv_qs_mutex);
+    unlock_sv_qs_mutex(server.sv_qs_mutex, __func__);
 
     /* Make the current job number visible in qmgr print server commnad. */
     pthread_mutex_lock(server.sv_attr_mutex);
@@ -602,14 +603,8 @@ int req_quejob(
     server.sv_attr[SRV_ATR_NextJobNumber].at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
     pthread_mutex_unlock(server.sv_attr_mutex);
 
-
-    if (svr_save(&server, SVR_SAVE_QUICK))
-      {
-      /* FAILURE */
-      rc = PBSE_CAN_NOT_SAVE_FILE;
-      req_reject(rc, 0, preq, NULL, "Server file save failed");
-      return rc;
-      }
+    /* Change - do not fail if the server can't be saved - the job file is saved */
+    svr_save(&server, SVR_SAVE_QUICK);
     }
 
   /* does job already exist, check both old and new jobs */
@@ -667,6 +662,11 @@ int req_quejob(
     return rc;
     }
 
+  /* unlock the queue. We validated that it was there, now let someone else
+     use it until we need it */
+  sprintf(log_buf, "Just validated queue");
+  unlock_queue(pque, __func__, log_buf, LOGLEVEL);
+
   /*
    * make up job file name, it is based on the jobid, however the
    * minimun acceptable file name limit is only 14 character in POSIX,
@@ -695,12 +695,11 @@ int req_quejob(
             /* FAILURE */
             rc = PBSE_JOB_FILE_CORRUPT;
             snprintf(log_buf, LOCAL_LOG_BUF_SIZE,
-                "job file %s corrupt (%d - %s)",
-                namebuf, errno, strerror(errno));
-            unlock_queue(pque, __func__, log_buf, LOGLEVEL);
+              "job file %s corrupt (%d - %s)",
+              namebuf, errno, strerror(errno));
             log_err(rc, __func__, log_buf);
             req_reject(rc, 0, preq, NULL, log_buf);
-            return rc;
+            return(rc);
             }
           }
 
@@ -711,12 +710,11 @@ int req_quejob(
         /* FAILURE */
         rc = PBSE_CAN_NOT_SAVE_FILE;
         snprintf(log_buf, LOCAL_LOG_BUF_SIZE,
-            "cannot create job file %s (%d - %s)",
-            namebuf, errno, strerror(errno));
-        unlock_queue(pque, __func__, log_buf, LOGLEVEL);
+          "cannot create job file %s (%d - %s)",
+          namebuf, errno, strerror(errno));
         log_err(rc, __func__, log_buf);
         req_reject(PBSE_SYSTEM, 0, preq, NULL, log_buf);
-        return rc;
+        return(rc);
         }
       }
     }
@@ -733,11 +731,10 @@ int req_quejob(
     snprintf(log_buf, LOCAL_LOG_BUF_SIZE,
         "can not alloc memory for new job %s - (%d - %s)",
         namebuf, errno, strerror(errno));
-    unlock_queue(pque, __func__, log_buf, LOGLEVEL);
     log_err(rc, __func__, log_buf);
     unlink(namebuf);
     req_reject(PBSE_SYSTEM, 0, preq, NULL, log_buf);
-    return rc;
+    return(rc);
     }
 
   strcpy(pj->ji_qs.ji_jobid, jid);
@@ -763,7 +760,7 @@ int req_quejob(
         }
       }
 
-    /* identify the attribute by name */
+    /* identify the pbs_attribute by name */
     attr_index = find_attr(job_attr_def, psatl->al_name, JOB_ATR_LAST);
 
     if (attr_index < 0)
@@ -777,16 +774,15 @@ int req_quejob(
 
     pdef = &job_attr_def[attr_index];
 
-    /* Is attribute not writeable by manager or by a server? */
+    /* Is pbs_attribute not writeable by manager or by a server? */
 
     if ((pdef->at_flags & resc_access_perm) == 0)
       {
       /* FAILURE */
       rc = PBSE_ATTRRO;
-      unlock_queue(pque, __func__, "attr ro", LOGLEVEL);
       job_purge(pj);
       reply_badattr(rc, 1, psatl, preq);
-      return rc;
+      return(rc);
       }
 
     /* special gpu case
@@ -818,17 +814,16 @@ int req_quejob(
         if (rc != 0)
           {
           /* FAILURE */
-          unlock_queue(pque, __func__, "gpus attribute error", LOGLEVEL);
           /* any other error is fatal */
           job_purge(pj);
           reply_badattr(rc, 1, psatl, preq);
-          return rc;
+          return(rc);
           }
         }
       pdef = &job_attr_def[attr_index];
       }
 
-    /* decode attribute */
+    /* decode pbs_attribute */
 
     rc = pdef->at_decode(
            &pj->ji_wattr[attr_index],
@@ -850,20 +845,18 @@ int req_quejob(
         if (pque->qu_qs.qu_type == QTYPE_Execution)
           {
           /* FAILURE */
-          unlock_queue(pque, __func__, "bad attr", LOGLEVEL);
           job_purge(pj);
           reply_badattr(rc, 1, psatl, preq);
-          return rc;
+          return(rc);
           }
         }
       else
         {
         /* FAILURE */
-        unlock_queue(pque, __func__, "random error", LOGLEVEL);
         /* any other error is fatal */
         job_purge(pj);
         reply_badattr(rc, 1, psatl, preq);
-        return rc;
+        return(rc);
         }
       }    /* END if (rc != 0) */
 
@@ -891,10 +884,9 @@ int req_quejob(
 
       if (rc)
         {
-        unlock_queue(pque, __func__, "attr exec fail", LOGLEVEL);
         job_purge(pj);
         req_reject(rc, i, preq, NULL, "cannot execute attribute action");
-        return rc;
+        return(rc);
         }
       }
     }    /* END for (i) */
@@ -933,7 +925,6 @@ int req_quejob(
           (pj->ji_wattr[JOB_ATR_priority].at_val.at_long > 1024))
         {
         rc = PBSE_BADATVAL;
-        unlock_queue(pque, __func__, "invalid job priority", LOGLEVEL);
         job_purge(pj);
         req_reject(rc, 0, preq, NULL, "invalid job priority");
         return rc;
@@ -962,7 +953,6 @@ int req_quejob(
         {
         pthread_mutex_unlock(tmpjob->ji_mutex);
 
-        unlock_queue(pque, __func__, "job duplication", LOGLEVEL);
         /* not unique, reject job */
         job_purge(pj);
        
@@ -1194,7 +1184,6 @@ int req_quejob(
         (pj->ji_wattr[JOB_ATR_errpath].at_val.at_str == NULL))
       {
       rc = PBSE_NOATTR;
-      unlock_queue(pque, __func__, "no output/error file", LOGLEVEL);
       job_purge(pj);
       req_reject(rc, 0, preq, NULL, "no output/error file specified");
       return rc;
@@ -1210,8 +1199,8 @@ int req_quejob(
 
     if (!(pj->ji_wattr[JOB_ATR_checkpoint_dir].at_flags & ATR_VFLAG_SET))
       {
-      attribute *pattr;
-      char *vp;
+      pbs_attribute *pattr;
+      char          *vp;
 
       pattr = &pj->ji_wattr[JOB_ATR_checkpoint];
 
@@ -1270,7 +1259,6 @@ int req_quejob(
             pj->ji_wattr[JOB_ATR_account].at_val.at_str) == 0)
         {
         rc = PBSE_BADACCT;
-        unlock_queue(pque, __func__, "invalid account", LOGLEVEL);
         job_purge(pj);
         req_reject(rc, 0, preq, NULL, "invalid account");
         return rc;
@@ -1291,7 +1279,6 @@ int req_quejob(
         {
         /* no default found */
         rc = PBSE_BADACCT;
-        unlock_queue(pque, __func__, "no default account", LOGLEVEL);
         job_purge(pj);
         req_reject(rc, 0, preq, NULL, "no default account available");
         return rc;
@@ -1310,7 +1297,6 @@ int req_quejob(
       {
       rc = PBSE_IVALREQ;
       snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "no job owner specified");
-      unlock_queue(pque, __func__, log_buf, LOGLEVEL);
       job_purge(pj);
       log_err(rc, __func__, log_buf);
       req_reject(rc, 0, preq, NULL, log_buf);
@@ -1322,14 +1308,13 @@ int req_quejob(
     if (++pj->ji_wattr[JOB_ATR_hopcount].at_val.at_long > PBS_MAX_HOPCOUNT)
       {
       rc = PBSE_HOPCOUNT;
-      unlock_queue(pque, __func__, "max job hop", LOGLEVEL);
       job_purge(pj);
       req_reject(rc, 0, preq, NULL, "max job hop reached");
       return rc;
       }
     }
 
-  /* set up at_server attribute for status */
+  /* set up at_server pbs_attribute for status */
 
   job_attr_def[JOB_ATR_at_server].at_decode(
     &pj->ji_wattr[JOB_ATR_at_server],
@@ -1368,18 +1353,17 @@ int req_quejob(
     *(hostname++) = '\0';
 
     snprintf(pj->ji_qs.ji_jobid, PBS_MAXSVRJOBID, "%s[].%s",
-             oldid,
-             hostname);
+      oldid,
+      hostname);
 
     free(oldid);    
     }
 
   if ((rc = svr_chkque(pj, pque, preq->rq_host, MOVE_TYPE_Move, EMsg)))
     {
-    unlock_queue(pque, __func__, "can not move", LOGLEVEL);
     job_purge(pj);
     req_reject(rc, 0, preq, NULL, EMsg);
-    return rc;
+    return(rc);
     }
 
   /* FIXME: if EMsg[0] != '\0', send a warning email to the user */
@@ -1414,8 +1398,6 @@ int req_quejob(
     {
     /* reply failed, purge the job and close the connection */
     rc = PBSE_SOCKET_WRITE; /* Re-write reply_jobid to return the error */
-    unlock_queue(pque, __func__, "reply fail", LOGLEVEL);
-    /*close_conn(sock,FALSE);*/
     job_purge(pj);
     return rc;
     }
@@ -1423,7 +1405,6 @@ int req_quejob(
   /* link job into server's new jobs list request  */
   insert_job(&newjobs,pj);
 
-  unlock_queue(pque, __func__, "success", LOGLEVEL);
   *pjob_id = strdup(pj->ji_qs.ji_jobid);
   pthread_mutex_unlock(pj->ji_mutex);
 
@@ -1503,7 +1484,7 @@ int req_jobscript(
         preq->rq_ind.rq_jobfile.rq_jobid, errno, strerror(errno));
     log_err(rc, __func__, log_buf);
     req_reject(rc, 0, preq, NULL, log_buf);
-    return rc;
+    return(rc);
     }
 
   /* what is the difference between JOB_SUBSTATE_TRANSIN and TRANSICM? */
@@ -1530,7 +1511,7 @@ int req_jobscript(
     log_err(rc, __func__, log_buf);
     req_reject(rc, 0, preq, NULL, log_buf);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   if (svr_authorize_jobreq(preq, pj) == -1)
@@ -1604,7 +1585,7 @@ int req_jobscript(
 
   reply_ack(preq);
 
-  return rc;
+  return(rc);
   }  /* END req_jobscript() */
 
 
@@ -1626,10 +1607,10 @@ int req_mvjobfile(
   struct batch_request *preq) /* ptr to the decoded request   */
 
   {
-  int                   fds;
-  char                  namebuf[MAXPATHLEN];
-  job                  *pj;
-  char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  int   fds;
+  char  namebuf[MAXPATHLEN];
+  job  *pj;
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
   int   rc = PBSE_NONE;
 
   pj = locate_new_job(preq->rq_conn, NULL);
@@ -1646,7 +1627,7 @@ int req_mvjobfile(
     req_reject(rc, 0, preq, NULL, NULL);
     if (pj != NULL)
       pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   snprintf(namebuf, sizeof(namebuf), "%s%s", path_spool, pj->ji_qs.ji_fileprefix);
@@ -1680,7 +1661,7 @@ int req_mvjobfile(
       log_err(rc, __func__, log_buf);
       req_reject(rc, 0, preq, NULL, log_buf);
       pthread_mutex_unlock(pj->ji_mutex);
-      return rc;
+      return(rc);
       break;
     }
 
@@ -1701,7 +1682,7 @@ int req_mvjobfile(
     log_err(errno, __func__, log_buf);
     req_reject(rc, 0, preq, NULL, log_buf);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   if (write(
@@ -1717,7 +1698,7 @@ int req_mvjobfile(
     req_reject(PBSE_SYSTEM, 0, preq, NULL, log_buf);
     close(fds);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   close(fds);
@@ -1740,7 +1721,7 @@ int req_mvjobfile(
 
   pthread_mutex_unlock(pj->ji_mutex);
 
-  return rc;
+  return(rc);
   }  /* END req_mvjobfile() */
 
 
@@ -1790,7 +1771,7 @@ int req_rdytocommit(
         preq->rq_ind.rq_rdytocommit, errno, strerror(errno));
     log_err(rc, __func__, log_buf);
     req_reject(rc, 0, preq, NULL, log_buf);
-    return rc;
+    return(rc);
     }
 
   if (pj->ji_qs.ji_substate != JOB_SUBSTATE_TRANSIN)
@@ -1802,7 +1783,7 @@ int req_rdytocommit(
     log_err(rc, __func__, log_buf);
     req_reject(rc, 0, preq, NULL, log_buf);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   if (svr_authorize_jobreq(preq, pj) == -1)
@@ -1812,7 +1793,7 @@ int req_rdytocommit(
         preq->rq_ind.rq_rdytocommit);
     req_reject(rc, 0, preq, NULL, log_buf);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   OrigState  = pj->ji_qs.ji_state;
@@ -1860,7 +1841,7 @@ int req_rdytocommit(
 
     req_reject(rc, 0, preq, NULL, log_buf);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   /* acknowledge the request with the job id */
@@ -1880,10 +1861,9 @@ int req_rdytocommit(
       errno,
       strerror(errno));
     log_err(rc, __func__, log_buf);
-    /*close_conn(sock,FALSE);*/
     if ((pj = find_job(jobid)) != NULL)
       job_purge(pj);
-    return rc;
+    return(rc);
     }
 
   if (LOGLEVEL >= 6)
@@ -1894,11 +1874,68 @@ int req_rdytocommit(
       (jobid != NULL) ? jobid : "NULL",
       "ready to commit job completed");
     }
-  return rc;
+
+  return(rc);
   }  /* END req_rdytocommit() */
 
 
 
+int set_interactive_job_roaming_policy(
+
+  job *pjob)
+
+  {
+  long            interactive_roaming = FALSE;
+  long            cray_enabled = FALSE;
+  struct pbsnode *pnode;
+  char           *submit_node_id;
+  char           *dot;
+  char            log_buf[LOCAL_LOG_BUF_SIZE];
+  int             rc = PBSE_NONE;
+  
+  get_svr_attr_l(SRV_ATR_InteractiveJobsCanRoam, &interactive_roaming);
+  get_svr_attr_l(SRV_ATR_CrayEnabled, &cray_enabled);
+
+  if (cray_enabled == TRUE)
+    {
+    if (pjob->ji_wattr[JOB_ATR_interactive].at_val.at_long)
+      {
+      if (interactive_roaming == FALSE)
+        {
+        submit_node_id = strdup(pjob->ji_wattr[JOB_ATR_submit_host].at_val.at_str);
+        if ((pnode = find_nodebyname(submit_node_id)) == NULL)
+          {
+          if ((dot = strchr(submit_node_id, '.')) != NULL)
+            {
+            *dot = '\0';
+            pnode = find_nodebyname(submit_node_id);
+            }
+          }
+        
+        if (pnode != NULL)
+          {
+          pjob->ji_wattr[JOB_ATR_login_prop].at_flags |= ATR_VFLAG_SET;
+          pjob->ji_wattr[JOB_ATR_login_prop].at_val.at_str = submit_node_id;
+          
+          unlock_node(pnode, __func__, NULL, 0);
+          }
+        else
+          {
+          snprintf(log_buf, sizeof(log_buf),
+            "Couldn't determine which login node is %s",
+            pjob->ji_wattr[JOB_ATR_submit_host].at_val.at_str);
+          log_err(PBSE_UNKNODE, __func__, log_buf);
+          rc = -1;
+          }
+        }
+      }
+    }
+  
+  return(rc);
+  } /* END set_interactive_job_roaming_policy() */
+
+
+
 
 /*
  * req_commit - commit ownership of job
@@ -1923,20 +1960,20 @@ int req_commit(
 #ifdef AUTORUN_JOBS
 
   struct batch_request *preq_run = '\0';
-  attribute *pattr;
-  int nodes_avail = -1;
-  int dummy;
-  char *spec = NULL;
-  char *rq_destin = NULL;
+  pbs_attribute        *pattr;
+  int                   nodes_avail = -1;
+  int                   dummy;
+  char                 *spec = NULL;
+  char                  *rq_destin = NULL;
 #endif /* AUTORUN_JOBS */
 
 #ifdef QUICKCOMMIT
-  int  OrigState;
-  int  OrigSState;
-  char OrigSChar;
-  long OrigFlags;
+  int                    OrigState;
+  int                    OrigSState;
+  char                   OrigSChar;
+  long                   OrigFlags;
 
-  char namebuf[MAXPATHLEN+1];
+  char                   namebuf[MAXPATHLEN+1];
 #endif /* QUICKCOMMIT */
 
   pj = locate_new_job(preq->rq_conn, preq->rq_ind.rq_commit);
@@ -1954,7 +1991,7 @@ int req_commit(
     {
     rc = PBSE_UNKJOBID;
     req_reject(rc, 0, preq, NULL, NULL);
-    return rc;
+    return(rc);
     }
 
 #ifdef QUICKCOMMIT
@@ -1967,7 +2004,7 @@ int req_commit(
     log_err(rc, __func__, log_buf);
     req_reject(PBSE_IVALREQ, 0, preq, NULL, log_buf);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   OrigState  = pj->ji_qs.ji_state;
@@ -2000,7 +2037,7 @@ int req_commit(
     log_err(rc, __func__, "cannot commit job in unexpected state");
     req_reject(rc, 0, preq, NULL, NULL);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   if (svr_authorize_jobreq(preq, pj) == -1)
@@ -2012,7 +2049,7 @@ int req_commit(
     if (LOGLEVEL >= 6)
       log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pj->ji_qs.ji_jobid, log_buf);
     pthread_mutex_unlock(pj->ji_mutex);
-    return rc;
+    return(rc);
     }
 
   /* remove job from the server new job list, set state, and enqueue it */
@@ -2052,41 +2089,54 @@ int req_commit(
         }
 
       pthread_mutex_unlock(pj->ji_mutex);
-      return rc;
+
+      return(rc);
       }
     }  /* end if (pj->ji_wattr[JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET) */
 
   svr_evaljobstate(pj, &newstate, &newsub, 1);
-
   svr_setjobstate(pj, newstate, newsub, FALSE);
 
+  set_interactive_job_roaming_policy(pj);
+
   /* set the queue rank attribute */
-
   pj->ji_wattr[JOB_ATR_qrank].at_val.at_long = ++queue_rank;
-
   pj->ji_wattr[JOB_ATR_qrank].at_flags |= ATR_VFLAG_SET;
 
   if ((rc = svr_enquejob(pj, FALSE, -1)) != PBSE_NONE)
     {
-    snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "can not queue job %s",
-        pj->ji_qs.ji_jobid);
-    if (LOGLEVEL >= 6)
-      log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pj->ji_qs.ji_jobid, log_buf);
-    job_purge(pj);
+    if (rc != PBSE_JOB_RECYCLED)
+      {
+      if (LOGLEVEL >= 6)
+        {
+        snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "can not queue job %s",
+          pj->ji_qs.ji_jobid);
+        
+        log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pj->ji_qs.ji_jobid, log_buf);
+        }
+
+      job_purge(pj);
+      }
+
     req_reject(rc, 0, preq, NULL, log_buf);
-    return rc;
+
+    return(rc);
     }
 
   if (job_save(pj, SAVEJOB_FULL, 0) != 0)
     {
     rc = PBSE_CAN_NOT_SAVE_FILE;
-    snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "can not save job %s",
-        pj->ji_qs.ji_jobid);
     if (LOGLEVEL >= 6)
+      {
+      snprintf(log_buf, LOCAL_LOG_BUF_SIZE, "can not save job %s",
+        pj->ji_qs.ji_jobid);
+
       log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pj->ji_qs.ji_jobid, log_buf);
+      }
+
     job_purge(pj);
     req_reject(rc, 0, preq, NULL, log_buf);
-    return rc;
+    return(rc);
     }
 
   /*
@@ -2114,7 +2164,7 @@ int req_commit(
           }
         job_purge(pj);
         req_reject(rc, 0, preq, NULL, log_buf);
-        return rc;
+        return(rc);
         }
       }
 
@@ -2204,7 +2254,7 @@ int req_commit(
 
 #endif /* AUTORUN_JOBS */
 
-  return rc;
+  return(rc);
   }  /* END req_commit() */
 
 
