@@ -120,21 +120,21 @@ extern long calc_job_cost(job *);
 
 /* Local Private Functions */
 
-void set_depend_hold(job *, attribute *);
+void set_depend_hold(job *, pbs_attribute *);
 static int register_sync(struct depend *,  char *child, char *host, long);
-static int register_dep(attribute *, struct batch_request *, int, int *);
-static int unregister_dep(attribute *, struct batch_request *);
-static int unregister_sync(attribute *, struct batch_request *);
+static int register_dep(pbs_attribute *, struct batch_request *, int, int *);
+static int unregister_dep(pbs_attribute *, struct batch_request *);
+static int unregister_sync(pbs_attribute *, struct batch_request *);
 
-static struct depend *find_depend(int type, attribute *pattr);
+static struct depend *find_depend(int type, pbs_attribute *pattr);
 
-static struct depend *make_depend(int type, attribute *pattr);
+static struct depend *make_depend(int type, pbs_attribute *pattr);
 
 static struct depend_job *find_dependjob(struct depend *, char *name);
 
 static struct depend_job *make_dependjob(struct depend *, char *jobid, char *host);
 static void   del_depend_job(struct depend_job *pdj);
-static int    build_depend(attribute *, char *);
+static int    build_depend(pbs_attribute *, char *);
 static void   clear_depend(struct depend *, int type, int exists);
 static void   del_depend(struct depend *);
 static void   release_cheapest(job *, struct depend *);
@@ -174,7 +174,7 @@ int req_register(
 
   {
   int                   made;
-  attribute            *pattr;
+  pbs_attribute        *pattr;
 
   struct depend        *pdep;
 
@@ -998,18 +998,23 @@ static void post_doq(
   struct work_task *pwt)
 
   {
-  struct batch_request *preq = (struct batch_request *)pwt->wt_parm1;
+  struct batch_request *preq = get_remove_batch_request(pwt->wt_parm1);
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  char                 *msg;
+  job                  *pjob;
+  pbs_attribute        *pattr;
 
-  char *jobid = preq->rq_ind.rq_register.rq_child;
+  struct depend        *pdp;
+  struct depend_job    *pdjb;
+  char                 *jobid;
 
-  char *msg;
-  job  *pjob;
-  attribute       *pattr;
+  free(pwt->wt_mutex);
+  free(pwt);
 
-  struct depend     *pdp;
+  if (preq == NULL)
+    return;
 
-  struct depend_job *pdjb;
+  jobid = preq->rq_ind.rq_register.rq_child;
 
   if (preq->rq_reply.brp_code)
     {
@@ -1056,7 +1061,7 @@ static void post_doq(
       }
     }
 
-  release_req(pwt);
+  free_br(preq);
 
   return;
   }  /* END post_doq() */
@@ -1066,16 +1071,16 @@ static void post_doq(
 
 
 /*
- * alter_unreg - if required, unregister dependencies on alter of attribute
+ * alter_unreg - if required, unregister dependencies on alter of pbs_attribute
  * This is called from depend_on_que() when is is acting as the at_action
- * routine for the dependency attribute.
+ * routine for the dependency pbs_attribute.
  */
 
 static void alter_unreg(
 
-  job     *pjob,
-  attribute *old,  /* current job dependency attribure */
-  attribute *new)  /* job dependency attribute after alter */
+  job           *pjob,
+  pbs_attribute *old,  /* current job dependency attribure */
+  pbs_attribute *new)  /* job dependency pbs_attribute after alter */
 
   {
   struct depend     *poldd;
@@ -1132,14 +1137,14 @@ static void alter_unreg(
  * that needs action when the job is queued into an execution queue.
  *
  * Called from svr_enquejob() when a job enters an
- * execution queue.  Also  the at_action routine for the attribute.
+ * execution queue.  Also  the at_action routine for the pbs_attribute.
  */
 
 int depend_on_que(
 
-  attribute *pattr,
-  void       *pj,
-  int        mode)
+  pbs_attribute *pattr,
+  void          *pj,
+  int            mode)
 
   {
   long               cost;
@@ -1246,14 +1251,22 @@ static void post_doe(
   struct work_task *pwt)
 
   {
-  struct batch_request *preq = pwt->wt_parm1;
-  char       *jobid = preq->rq_ind.rq_register.rq_child;
-  attribute      *pattr;
+  struct batch_request *preq = get_remove_batch_request(pwt->wt_parm1);
+  pbs_attribute        *pattr;
 
-  struct depend      *pdep;
+  struct depend        *pdep;
 
-  struct depend_job  *pdj;
-  job       *pjob;
+  struct depend_job    *pdj;
+  job                  *pjob;
+  char                 *jobid;
+ 
+  free(pwt->wt_mutex);
+  free(pwt);
+
+  if (preq == NULL)
+    return;
+
+  jobid = preq->rq_ind.rq_register.rq_child;
 
   pjob = find_job(jobid);
 
@@ -1276,7 +1289,7 @@ static void post_doe(
     pthread_mutex_unlock(pjob->ji_mutex);
     }
 
-  release_req(pwt);
+  free_br(preq);
 
   return;
   }  /* END post_doe() */
@@ -1385,7 +1398,7 @@ int depend_on_exec(
  * appropriate.
  *
  * This function is invoked from on_job_exit() in req_jobobit.c.
- * When there are no depends to deal with, free the attribute and
+ * When there are no depends to deal with, free the pbs_attribute and
  * recall on_job_exit().
  */
 
@@ -1394,21 +1407,23 @@ int depend_on_term(
   job *pjob)
 
   {
-  int        exitstat = pjob->ji_qs.ji_un.ji_exect.ji_exitstat;
-  int        op;
-  attribute     *pattr;
+  int                exitstat = pjob->ji_qs.ji_un.ji_exect.ji_exitstat;
+  int                op;
+  pbs_attribute     *pattr;
 
-  struct depend *pdep;
+  struct depend     *pdep;
 
   struct depend_job *pparent;
-  int        rc;
-  int        shouldkill = 0;
-  int        type;
-  char job_id[PBS_MAXSVRJOBID+1];
-  int job_unlocked = 0;
-
+  int                rc;
+  int                shouldkill = 0;
+  int                type;
+  char               job_id[PBS_MAXSVRJOBID+1];
+  int                job_unlocked = 0;
+ 
   strcpy(job_id, pjob->ji_qs.ji_jobid);
+
   pattr = &pjob->ji_wattr[JOB_ATR_depend];
+
   pdep = (struct depend *)GET_NEXT(pattr->at_val.at_list);
 
   while (pdep != NULL)
@@ -1479,13 +1494,15 @@ int depend_on_term(
 
           while (pparent)
             {
-            if ((pjob == NULL) && ((pjob = find_job(job_id)) == NULL))
-              return PBSE_JOBNOTFOUND;
+            if ((pjob == NULL) && 
+                ((pjob = find_job(job_id)) == NULL))
+              return(PBSE_JOBNOTFOUND);
 
             rc = send_depend_req(pjob, pparent, type,
                             JOB_DEPEND_OP_DELETE,
                             SYNC_SCHED_HINT_NULL,
                             release_req);
+
             pjob = NULL;
             job_unlocked = 1;
 
@@ -1503,8 +1520,10 @@ int depend_on_term(
 
       while (pparent)
         {
-        if ((pjob == NULL) && ((pjob = find_job(job_id)) == NULL))
-          return PBSE_JOBNOTFOUND;
+        if ((pjob == NULL) && 
+            ((pjob = find_job(job_id)) == NULL))
+          return(PBSE_JOBNOTFOUND);
+
         /* "release" the job to execute */
         if ((rc = send_depend_req(pjob, pparent, type, op, SYNC_SCHED_HINT_NULL,
                 release_req)) != PBSE_NONE)
@@ -1604,8 +1623,8 @@ static void release_cheapest(
 
 void set_depend_hold(
 
-  job       *pjob,
-  attribute *pattr)
+  job           *pjob,
+  pbs_attribute *pattr)
 
   {
   int  loop = 1;
@@ -1745,7 +1764,7 @@ void set_depend_hold(
 
 
 /*
- * depend_clrrdy - clear state ready flags in job dependency attribute
+ * depend_clrrdy - clear state ready flags in job dependency pbs_attribute
  */
 
 void depend_clrrdy(
@@ -1787,8 +1806,8 @@ void depend_clrrdy(
 
 static struct depend *find_depend(
 
-  int        type,
-  attribute *pattr)
+  int            type,
+  pbs_attribute *pattr)
 
   {
 
@@ -1820,8 +1839,8 @@ static struct depend *find_depend(
 
 static struct depend *make_depend(
 
-  int        type,
-  attribute *pattr)
+  int            type,
+  pbs_attribute *pattr)
 
   {
 
@@ -1905,15 +1924,13 @@ static int register_sync(
 
 static int register_dep(
 
-  attribute        *pattr,
+  pbs_attribute        *pattr,
   struct batch_request *preq,
-  int          type,
-  int         *made) /* RETURN */
+  int                   type,
+  int                  *made) /* RETURN */
 
   {
-
   struct depend     *pdep;
-
   struct depend_job *pdj;
 
   /* change into the mirror image type */
@@ -1968,7 +1985,7 @@ static int register_dep(
 
 static int unregister_dep(
 
-  attribute            *pattr,
+  pbs_attribute        *pattr,
   struct batch_request *preq)
 
   {
@@ -2004,7 +2021,7 @@ static int unregister_dep(
 
 static int unregister_sync(
 
-  attribute            *pattr,
+  pbs_attribute        *pattr,
   struct batch_request *preq)
 
   {
@@ -2118,10 +2135,11 @@ static int send_depend_req(
   void               (*postfunc)(struct work_task *))
 
   {
-  int rc = 0;
+  int                   rc = 0;
   int                   i;
   char                 *myid = "send_depend_req";
   char                  job_id[PBS_MAXSVRJOBID + 1];
+  char                  br_id[MAXLINE];
 
   struct batch_request *preq;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
@@ -2132,7 +2150,7 @@ static int send_depend_req(
     {
     log_err(errno, myid, msg_err_malloc);
     pthread_mutex_unlock(pjob->ji_mutex);
-    return PBSE_SYSTEM;
+    return(PBSE_SYSTEM);
     }
 
   for (i = 0;i < PBS_MAXUSER;++i)
@@ -2185,12 +2203,20 @@ static int send_depend_req(
   strcpy(job_id, pjob->ji_qs.ji_jobid);
   pthread_mutex_unlock(pjob->ji_mutex);
 
+  get_batch_request_id(preq);
+  strcpy(br_id, preq->rq_id);
+
   if ((rc = issue_to_svr(pparent->dc_svr, preq, postfunc)) != PBSE_NONE)
     {
     sprintf(log_buf, "Unable to perform dependency with job %s\n", pparent->dc_child);
-    return rc;
+
+    if ((preq = get_remove_batch_request(br_id)) != NULL)
+      free_br(preq);
+
+    return(rc);
     }
-  return PBSE_NONE;
+
+  return(PBSE_NONE);
   }  /* END send_depend_req() */
 
 
@@ -2200,17 +2226,17 @@ static int send_depend_req(
 /*
  * This section contains general function for dependency attributes
  *
- * Each attribute has functions for:
+ * Each pbs_attribute has functions for:
  * Decoding the value string to the machine representation.
- * Encoding the internal representation of the attribute to external
+ * Encoding the internal representation of the pbs_attribute to external
  * Setting the value by =, + or - operators.
- * Comparing a (decoded) value with the attribute value.
- * Freeing the space calloc-ed to the attribute value.
+ * Comparing a (decoded) value with the pbs_attribute value.
+ * Freeing the space calloc-ed to the pbs_attribute value.
  *
  * The prototypes are declared in "attribute.h"
  *
  * ----------------------------------------------------------------------------
- * Attribute functions for attributes of type "dependency".
+ * pbs_Attribute functions for attributes of type "dependency".
  *
  * The "encoded" or external form of the value is a string with sub-strings
  * separated by commas and terminated by a null.
@@ -2266,11 +2292,11 @@ struct dependnames
 
 int decode_depend(
 
-  attribute *patr,
-  char      *name,  /* attribute name */
-  char      *rescn, /* resource name, unused here */
-  char      *val,   /* attribute value */
-  int        perm)  /* only used for resources */
+  pbs_attribute *patr,
+  char          *name,  /* attribute name */
+  char          *rescn, /* resource name, unused here */
+  char          *val,   /* attribute value */
+  int            perm)  /* only used for resources */
 
   {
   int  rc;
@@ -2420,7 +2446,7 @@ static void fast_strcat(
 
 static int dup_depend(
 
-  attribute     *pattr,
+  pbs_attribute *pattr,
   struct depend *pd)
 
   {
@@ -2476,12 +2502,12 @@ static int dup_depend(
 
 int encode_depend(
 
-  attribute    *attr,   /* ptr to attribute to encode */
-  tlist_head   *phead,  /* ptr to head of attrlist list */
-  char	       *atname, /* attribute name */
-  char	       *rsname, /* resource name or null */
-  int           mode,   /* encode mode, unused here */
-  int           perm)   /* only used for resources */
+  pbs_attribute *attr,   /* ptr to attribute to encode */
+  tlist_head    *phead,  /* ptr to head of attrlist list */
+  char	        *atname, /* attribute name */
+  char	        *rsname, /* resource name or null */
+  int            mode,   /* encode mode, unused here */
+  int            perm)   /* only used for resources */
 
   {
   int      ct = 0;
@@ -2607,7 +2633,7 @@ int encode_depend(
 
 
 /*
- * set_depend - set value of attribute of dependency type to another
+ * set_depend - set value of pbs_attribute of dependency type to another
  *
  * A=B --> set of dependencies in A replaced by set in B
  * A+B --> dependencies in B added to list in A
@@ -2619,9 +2645,9 @@ int encode_depend(
 
 int set_depend(
 
-  struct attribute *attr,
-  struct attribute *new,
-  enum batch_op     op)
+  pbs_attribute *attr,
+  pbs_attribute *new,
+  enum batch_op  op)
 
   {
 
@@ -2695,23 +2721,19 @@ int set_depend(
 
 int comp_depend(
     
-  struct attribute *attr, 
-  struct attribute *with)
+  pbs_attribute *attr, 
+  pbs_attribute *with)
 
   {
-
   return (-1);
-
   }
 
 void free_depend(
 
-  struct attribute *attr)
+  pbs_attribute *attr)
 
   {
-
-  struct depend   *pdp;
-
+  struct depend     *pdp;
   struct depend_job *pdjb;
 
   while ((pdp = (struct depend *)GET_NEXT(attr->at_val.at_list)))
@@ -2746,8 +2768,8 @@ void free_depend(
 
 static int build_depend(
 
-  attribute *pattr,
-  char      *value)
+  pbs_attribute *pattr,
+  char          *value)
 
   {
 
