@@ -129,7 +129,7 @@ extern void  set_resc_assigned(job *, enum batch_op);
 
 extern struct batch_request *cpy_stage(struct batch_request *, job *, enum job_atr, int);
 void                         stream_eof(int, u_long, uint16_t, int);
-extern int                   job_set_wait(attribute *, void *, int);
+extern int                   job_set_wait(pbs_attribute *, void *, int);
 
 extern int LOGLEVEL;
 
@@ -187,17 +187,17 @@ int req_runjob(
   struct batch_request *preq)  /* I (modified) */
 
   {
-  job   *pjob;
-  int   rc;
-  void  *bp;
-  int   setneednodes;
-  char  failhost[MAXLINE];
-  char  emsg[MAXLINE];
-  char  log_buf[LOCAL_LOG_BUF_SIZE + 1];
-  char  job_id[PBS_MAXSVRJOBID+1];
-  long  job_atr_hold;
-  int   job_exit_status;
-  int   job_state;
+  job                  *pjob;
+  int                   rc;
+  void                 *bp;
+  int                   setneednodes;
+  char                  failhost[MAXLINE];
+  char                  emsg[MAXLINE];
+  char                  log_buf[LOCAL_LOG_BUF_SIZE + 1];
+  char                  job_id[PBS_MAXSVRJOBID+1];
+  long                  job_atr_hold;
+  int                   job_exit_status;
+  int                   job_state;
 
 
   /* chk_job_torun will extract job id and assign hostlist if specified */
@@ -224,15 +224,6 @@ int req_runjob(
     return(PBSE_IVALREQ);
     }
 
-  /* we don't currently allow running of an entire job array */
-
-  if (strstr(pjob->ji_qs.ji_jobid,"[]") != NULL)
-    {
-    req_reject(PBSE_IVALREQ, 0, preq, NULL, "cannot run a job array");
-
-    return(PBSE_IVALREQ);
-    }
-
   pthread_mutex_lock(scheduler_sock_jobct_mutex);
   if (preq->rq_conn == scheduler_sock)
     ++scheduler_jobct; /* see scheduler_close() */
@@ -385,11 +376,18 @@ static void post_checkpointsend(
   job                  *pjob;
 
   struct batch_request *preq;
-  attribute            *pwait;
+  pbs_attribute            *pwait;
   char                  log_buf[LOCAL_LOG_BUF_SIZE];
   time_t                time_now = time(NULL);
 
-  preq = pwt->wt_parm1;
+  preq = get_remove_batch_request(pwt->wt_parm1);
+
+  free(pwt->wt_mutex);
+  free(pwt);
+
+  if (preq == NULL)
+    return;
+
   code = preq->rq_reply.brp_code;
   pjob = find_job(preq->rq_extra);
 
@@ -438,7 +436,7 @@ static void post_checkpointsend(
 
       pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHECKPOINT_COPIED;
       
-      /* set restart_name attribute to the checkpoint_name we just copied */
+      /* set restart_name pbs_attribute to the checkpoint_name we just copied */
       
       job_attr_def[JOB_ATR_restart_name].at_set(
         &pjob->ji_wattr[JOB_ATR_restart_name],
@@ -458,7 +456,7 @@ static void post_checkpointsend(
       pthread_mutex_unlock(pjob->ji_mutex);
     }    /* END if (pjob != NULL) */
 
-  release_req(pwt); /* close connection and release request */
+  free_br(preq); /* close connection and release request */
 
   return;
   }  /* END post_checkpointsend() */
@@ -607,10 +605,18 @@ static void post_stagein(
   job                  *pjob;
 
   struct batch_request *preq;
-  attribute            *pwait;
+  pbs_attribute        *pwait;
   time_t                time_now = time(NULL);
 
-  preq = pwt->wt_parm1;
+  preq = get_remove_batch_request(pwt->wt_parm1);
+    
+  free(pwt->wt_mutex);
+  free(pwt);
+
+  /* preq handled previously */
+  if (preq == NULL)
+    return;
+
   code = preq->rq_reply.brp_code;
   pjob = find_job(preq->rq_extra);
 
@@ -686,7 +692,7 @@ static void post_stagein(
       pthread_mutex_unlock(pjob->ji_mutex);
     }    /* END if (pjob != NULL) */
 
-  release_req(pwt); /* close connection and release request */
+  free_br(preq); /* close connection and release request */
 
   return;
   }  /* END post_stagein() */
@@ -783,6 +789,7 @@ int verify_moms_up(
   char               *hostlist;
   char               *hostlist_ptr;
 
+  struct sockaddr_in *sai;
   struct sockaddr_in  saddr;
 
   badplace           *bp;
@@ -811,9 +818,9 @@ int verify_moms_up(
       }
 
     /* Lookup IP address of host. */
-    if (getaddrinfo(nodestr, NULL, NULL, &addr_info) != 0)
+    if ((sai = get_cached_addrinfo(nodestr)) == NULL)
       {
-      sprintf(log_buf, "could not contact %s (getaddrinfo() failed, errno: %d (%s))",
+      sprintf(log_buf, "could not contact %s (no address info, errno: %d (%s))",
         nodestr,
         errno,
         pbs_strerror(errno));
@@ -880,10 +887,8 @@ int verify_moms_up(
       }
 
     /* Set the host information. */
-    memset(&saddr, '\0', sizeof(saddr));
+    memcpy(&saddr, sai, sizeof(saddr));
     saddr.sin_family = AF_INET;
-    saddr.sin_addr = ((struct sockaddr_in *)addr_info->ai_addr)->sin_addr;
-    freeaddrinfo(addr_info);
     saddr.sin_port = htons(pjob->ji_qs.ji_un.ji_exect.ji_mom_rmport);
 
     /* Connect to the host. */
@@ -960,7 +965,7 @@ int svr_startjob(
     EMsg[0] = '\0';
 
   /* if not already setup, transfer the control/script file basename */
-  /* into an attribute accessible by MOM */
+  /* into an pbs_attribute accessible by MOM */
 
   if (!(pjob->ji_wattr[JOB_ATR_hashname].at_flags & ATR_VFLAG_SET))
     {
@@ -1080,7 +1085,7 @@ static int svr_strtjob2(
   int              old_state;
   int              old_subst;
   int              my_err = 0;
-  attribute       *pattr;
+  pbs_attribute   *pattr;
   char             tmpLine[MAXLINE];
   struct timeval   start_time;
   struct timezone  tz;
@@ -1118,7 +1123,7 @@ static int svr_strtjob2(
 
   svr_setjobstate(pjob,JOB_STATE_RUNNING,JOB_SUBSTATE_PRERUN, FALSE);
 
-  /* if job start timeout attribute is set use its value */
+  /* if job start timeout pbs_attribute is set use its value */
   get_svr_attr_l(SRV_ATR_tcp_timeout, &tcp_timeout);
   
   if ((get_svr_attr_l(SRV_ATR_JobStartTimeout, &job_timeout) == PBSE_NONE) &&
@@ -1371,8 +1376,6 @@ static job *chk_job_torun(
   int                   setnn) /* I */
 
   {
-  static char *id = "chk_job_torun";
-
   job              *pjob;
 
   struct rq_runjob *prun;
@@ -1438,7 +1441,7 @@ static job *chk_job_torun(
     if (pque->qu_qs.qu_type != QTYPE_Execution)
       {
       /* FAILURE - job must be in execution queue */
-      log_err(-1, id, "attempt to start job in non-execution queue");
+      log_err(-1, __func__, "attempt to start job in non-execution queue");
   
       req_reject(PBSE_IVALREQ, 0, preq, NULL, "job not in execution queue");
   
@@ -1561,7 +1564,7 @@ static job *chk_job_torun(
 
     resource *JRes;      /* resource on job */
 
-    attribute *Attr;     /* 'neednodes' attribute */
+    pbs_attribute *Attr;     /* 'neednodes' pbs_attribute */
 
     Attr = &pjob->ji_wattr[JOB_ATR_resource];
 
@@ -1600,21 +1603,22 @@ static job *chk_job_torun(
 
 
 /* 
- * set_mother_superior_ports - The first host in list is the host
+ * set_job_exec_info - The first host in list is the host
  * of Mother Superior. Find the mom manager and service ports
  * from allnodes and then set the pjob mom ports accordingly 
  */
 
-int set_mother_superior_ports(
+int set_job_exec_info(
     
-  job *pjob,
+  job  *pjob,
   char *list)
 
   {
-  char ms[PBS_MAXHOSTNAME];
-  char *ptr;
-  int  i;
+  char            ms[PBS_MAXHOSTNAME];
+  char           *ptr;
+  int             i;
   struct pbsnode *pnode;
+  struct in_addr  hostaddr;
 
   if (list == NULL)
     {
@@ -1625,7 +1629,7 @@ int set_mother_superior_ports(
   ptr = list;
 
   /* get the first name in list. This is Mother Superior */
-  for(i = 0; ptr && (*ptr != '/') && (i < PBS_MAXHOSTNAME); i++)
+  for (i = 0; ptr && (*ptr != '/') && (i < PBS_MAXHOSTNAME); i++)
     {
     ms[i] = *ptr;
     ptr++;
@@ -1637,14 +1641,16 @@ int set_mother_superior_ports(
     {
     pjob->ji_qs.ji_un.ji_exect.ji_momport = pnode->nd_mom_port;
     pjob->ji_qs.ji_un.ji_exect.ji_mom_rmport = pnode->nd_mom_rm_port;
+    memcpy(&hostaddr, &pnode->nd_sock_addr.sin_addr, sizeof(hostaddr));
+    pjob->ji_qs.ji_un.ji_exect.ji_momaddr = ntohl(hostaddr.s_addr);
 
-    unlock_node(pnode, "set_mother_superior_ports", NULL, LOGLEVEL);
+    unlock_node(pnode, __func__, NULL, LOGLEVEL);
     
     return(PBSE_NONE);
     }
 
   return(PBSE_UNKNODEATR);
-  } /* END set_mother_superior_ports() */
+  } /* END set_job_exec_info() */
 
 
 
@@ -1656,17 +1662,16 @@ char *get_correct_spec_string(
   job  *pjob)
 
   {
-  static char   id[] = "get_correct_spec_string";
-  char     mode[20];
-  char    *mode_string;
-  char    *request;
-  char    *correct_spec;
-  char    *outer_plus;
-  char    *plus;
-  char    *one_req;
-  int      num_gpu_reqs;
-  char    *gpu_req;
-  int      len;
+  char      mode[20];
+  char     *mode_string;
+  char     *request;
+  char     *correct_spec;
+  char     *outer_plus;
+  char     *plus;
+  char     *one_req;
+  int       num_gpu_reqs;
+  char     *gpu_req;
+  int       len;
   resource *pres;
 
   /* check to see if there is a gpus request. If so moab
@@ -1706,27 +1711,52 @@ char *get_correct_spec_string(
           if (LOGLEVEL >= 7)
             {
             sprintf(log_buffer, "%s: job has %d gpu requests in node spec '%s'",
-                    id,
-                    num_gpu_reqs,
-                    given);
+              __func__,
+              num_gpu_reqs,
+              given);
 
-            log_event(
-              PBSEVENT_JOB,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
+            log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
             }
 
           if ((outer_plus = strchr(mode_string, '+')) != NULL)
             *outer_plus = '\0';
 
-          strcpy(mode, mode_string);
+          /* 
+           * The neednodes original value may have non gpu things in it, so we
+           * can not rely on the requested gpu mode being the first item in the
+           * the string after the gpus=x:.
+           */
+
+          if (strstr(mode_string, "exclusive_thread"))
+            {
+            strcpy(mode, ":exclusive_thread");
+            }
+          else if (strstr(mode_string, "exclusive_process"))
+            {
+            strcpy(mode, ":exclusive_process");
+            }
+          else if (strstr(mode_string, "exclusive"))
+            {
+            strcpy(mode, ":exclusive");
+            }
+          else if (strstr(mode_string, "default"))
+            {
+            strcpy(mode, ":default");
+            }
+          else if (strstr(mode_string, "shared"))
+            {
+            strcpy(mode, ":shared");
+            }
+          else
+            {
+            strcpy(mode, "");
+            }
 
           if (outer_plus != NULL)
             *outer_plus = '+';
 
-          /* 20 is a little more than the max length of gpu modes */
-          len = strlen(given) + 1 + (num_gpu_reqs * 20);
+          /* now using the actual length of requested gpu mode */
+          len = strlen(given) + 1 + (num_gpu_reqs * strlen(mode));
           if ((correct_spec = calloc(1, len)) != NULL)
             {
             one_req = given;
@@ -1754,14 +1784,10 @@ char *get_correct_spec_string(
           if ((LOGLEVEL >= 7) && (correct_spec != NULL) && (correct_spec[0] != '\0'))
             {
             sprintf(log_buffer, "%s: job gets adjusted gpu node spec of '%s'",
-                    id,
-                    correct_spec);
+              __func__,
+              correct_spec);
 
-            log_event(
-              PBSEVENT_JOB,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
+            log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
             }
           }
         else
@@ -1802,15 +1828,14 @@ static int assign_hosts(
   char         *list = NULL;
   char         *portlist = NULL;
   char         *hosttoalloc = NULL;
-  pbs_net_t     momaddr = 0;
   resource     *pres;
   int           rc = 0;
   int           procs=0;
   extern char  *mom_host;
   char          log_buf[LOCAL_LOG_BUF_SIZE];
-  int           local_errno = 0;
   char         *def_node = NULL;
   char         *to_free = NULL;
+  long          cray_enabled = FALSE;
 
   if (EMsg != NULL)
     EMsg[0] = '\0';
@@ -1889,7 +1914,6 @@ static int assign_hosts(
     else
       {
       hosttoalloc = mom_host;
-      momaddr = pbs_mom_addr;
       }
     }
   else if (def_node != NULL)
@@ -1962,45 +1986,41 @@ static int assign_hosts(
     else
       {
       /* leave exec_host alone and reuse old IP address */
-      momaddr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
-
       hosttoalloc = pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str;
       portlist = pjob->ji_wattr[JOB_ATR_exec_port].at_val.at_str;
       }
 
-    tmp = parse_servername(hosttoalloc, &dummy);
-
+    get_svr_attr_l(SRV_ATR_CrayEnabled, &cray_enabled);
+    if ((cray_enabled == TRUE) &&
+        (pjob->ji_wattr[JOB_ATR_login_node_id].at_val.at_str != NULL))
+      tmp = parse_servername(pjob->ji_wattr[JOB_ATR_login_node_id].at_val.at_str, &dummy);
+    else
+      tmp = parse_servername(hosttoalloc, &dummy);
+    
     snprintf(pjob->ji_qs.ji_destin, sizeof(pjob->ji_qs.ji_destin), "%s", tmp);
-
+    
     free(tmp);
 
-    if (momaddr == 0)
+    rc = set_job_exec_info(pjob, pjob->ji_qs.ji_destin);
+
+    if (rc != 0)
       {
-      momaddr = get_hostaddr(&local_errno, pjob->ji_qs.ji_destin);
-
-      if (momaddr == 0)
-        {
-        free_nodes(pjob);
-
-        if (list != NULL)
-          free(list);
-
-        sprintf(log_buf, "ALERT:  job cannot allocate node '%s' (could not determine IP address for node)",
-          pjob->ji_qs.ji_destin);
-
-        log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
-
-        if (to_free != NULL)
-          free(to_free);
-
-        return(PBSE_BADHOST);
-        }
+      free_nodes(pjob);
+      
+      if (list != NULL)
+        free(list);
+      
+      sprintf(log_buf, "ALERT:  job cannot allocate node '%s' (could not determine IP address for node)",
+        pjob->ji_qs.ji_destin);
+      
+      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
+      
+      if (to_free != NULL)
+        free(to_free);
+      
+      return(PBSE_BADHOST);
       }
 
-    pjob->ji_qs.ji_un.ji_exect.ji_momaddr = momaddr;
-
-    rc = set_mother_superior_ports(pjob, list);
-
     }  /* END if (rc == 0) */
 
   if (list != NULL)
