@@ -144,11 +144,11 @@ void svr_shutdown(
   int type) /* I */
 
   {
-  attribute    *pattr;
-  job          *pjob;
-  long          state = SV_STATE_DOWN;
-  int           iter;
-  char          log_buf[LOCAL_LOG_BUF_SIZE];
+  pbs_attribute *pattr;
+  job           *pjob;
+  long           state = SV_STATE_DOWN;
+  int            iter;
+  char           log_buf[LOCAL_LOG_BUF_SIZE];
 
   close(lockfds);
 
@@ -335,7 +335,7 @@ static int shutdown_checkpoint(
   {
   job                  *pjob = *pjob_ptr;
   struct batch_request *phold;
-  attribute             temp;
+  pbs_attribute         temp;
 
   phold = alloc_br(PBS_BATCH_HoldJob);
 
@@ -416,7 +416,14 @@ static void post_checkpoint(
 
   struct batch_request *preq;
 
-  preq = (struct batch_request *)ptask->wt_parm1;
+  preq = get_remove_batch_request((char *)ptask->wt_parm1);
+
+  free(ptask->wt_mutex);
+  free(ptask);
+
+  if (preq == NULL)
+    return;
+
   pjob = find_job(preq->rq_ind.rq_hold.rq_orig.rq_objname);
 
   if (preq->rq_reply.brp_code == 0)
@@ -444,7 +451,7 @@ static void post_checkpoint(
       }
     }
 
-  release_req(ptask);
+  free_br(preq);
 
   if (pjob != NULL)
     pthread_mutex_unlock(pjob->ji_mutex);
@@ -471,7 +478,7 @@ static void rerun_or_kill(
     {
     /* job is rerunable, mark it to be requeued */
 
-    issue_signal(&pjob, "SIGKILL", release_req, 0);
+    issue_signal(&pjob, "SIGKILL", release_req, NULL);
 
     if (pjob != NULL)
       {
