@@ -280,7 +280,6 @@ static void post_signal_req(
   struct work_task *pwt)
 
   {
-  static char          *id = "post_signal_req";
   char                 *jobid;
   job                  *pjob;
 
@@ -289,11 +288,15 @@ static void post_signal_req(
 
   svr_disconnect(pwt->wt_event); /* disconnect from MOM */
 
-  preq = pwt->wt_parm1;
+  preq = get_remove_batch_request((char *)pwt->wt_parm1);
  
   free(pwt->wt_mutex);
   free(pwt);
 
+  /* request has been handled elsewhere */
+  if (preq == NULL)
+    return;
+
   preq->rq_conn = preq->rq_orgconn;  /* restore client socket */
 
   if (preq->rq_reply.brp_code)
@@ -312,7 +315,7 @@ static void post_signal_req(
     {
     if ((jobid = preq->rq_extra) == NULL)
       {
-      log_err(ENOMEM,id,"Cannot allocate memory! FAILURE");
+      log_err(ENOMEM, __func__, "Cannot allocate memory! FAILURE");
       return;
       }
 
@@ -358,7 +361,7 @@ static void post_signal_req(
       snprintf(log_buf,sizeof(log_buf),
         "Cannot find job '%s', assuming success",
         jobid);
-      log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_JOB,id,log_buf);
+      log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_JOB, __func__, log_buf);
       }
 
     free(jobid);
