@@ -114,10 +114,11 @@
 #include "node_func.h" /* find_nodebyname */
 #include "issue_request.h" /* issue_Drequest */
 #include "../lib/Libutils/u_lock_ctl.h" /* lock_node, unlock_node */
-#include "svr_connect.h" /* svr_connect, svr_disconnect_sock */
+#include "svr_connect.h" /* svr_connect */
 #include "queue_func.h" /* find_queuebyname */
 #include "reply_send.h" /* reply_send_svr */
 #include "svr_func.h" /* get_svr_attr_* */
+#include "alps_functions.h"
 #include "node_manager.h" /* tfind_addr */
 
 
@@ -142,16 +143,16 @@ extern pthread_mutex_t *netrates_mutex;
 /* Extern Functions */
 
 int status_job(job *, struct batch_request *, svrattrl *, tlist_head *, int *);
-int status_attrib(svrattrl *, attribute_def *, attribute *, int, int, tlist_head *, int *, int);
+int status_attrib(svrattrl *, attribute_def *, pbs_attribute *, int, int, tlist_head *, int *, int);
 extern int  status_nodeattrib(svrattrl *, attribute_def *, struct pbsnode *, int, int, tlist_head *, int*);
 extern int  hasprop(struct pbsnode *, struct prop *);
 extern void rel_resc(job*);
 
 /* The following private support functions are included */
 
-static void update_state_ct(attribute *, int *, char *);
+static void update_state_ct(pbs_attribute *, int *, char *);
 static int  status_que(pbs_queue *, struct batch_request *, tlist_head *);
-static int  status_node(struct pbsnode *, struct batch_request *, int *, tlist_head *);
+int         status_node(struct pbsnode *, struct batch_request *, int *, tlist_head *);
 static void req_stat_job_step2(struct stat_cntl *);
 
 #ifndef TMAX_JOB
@@ -374,18 +375,19 @@ static void req_stat_job_step2(
   enum TJobStatTypeEnum  type;
   pbs_queue             *pque = NULL;
   int                    exec_only = 0;
+
   int                    bad = 0;
-  long                   DTime;  /* delta time - only report full attribute list if J->MTime > DTime */
+  long                   DTime;  /* delta time - only report full pbs_attribute list if J->MTime > DTime */
   static svrattrl       *dpal = NULL;
   int                    job_array_index = 0;
   job_array             *pa = NULL;
-  char                   log_buf[LOCAL_LOG_BUF_SIZE+1];
+  char                   log_buf[LOCAL_LOG_BUF_SIZE];
   int                    iter;
   time_t                 time_now = time(NULL);
   long                   poll_jobs = 0;
-  char job_id[PBS_MAXSVRJOBID+1];
-  int job_substate = -1;
-  time_t job_momstattime = -1;
+  char                   job_id[PBS_MAXSVRJOBID+1];
+  int                    job_substate = -1;
+  time_t                 job_momstattime = -1;
 
   preq   = cntl->sc_origrq;
   type   = (enum TJobStatTypeEnum)cntl->sc_type;
@@ -395,7 +397,7 @@ static void req_stat_job_step2(
 
   if (dpal == NULL)
     {
-    /* build 'delta' attribute list */
+    /* build 'delta' pbs_attribute list */
 
     svrattrl *tpal;
 
@@ -517,6 +519,7 @@ static void req_stat_job_step2(
           }
         else
           pjob = next_job(&alljobs,&iter);
+          
         }
 
       if (pjob == NULL)
@@ -534,22 +537,22 @@ static void req_stat_job_step2(
           ((time_now - job_momstattime) > JobStatRate))
         {
         /* go to MOM for status */
-
-
         if ((rc = stat_to_mom(job_id, cntl)) == PBSE_MEM_MALLOC)
           break;
 
         if (rc != 0)
           {
           pjob = find_job(job_id);
+
           rc = 0;
+
           continue;
           }
         
         if (LOGLEVEL >= 7)
           {
           sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
-          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
           }
 
         if (pa != NULL)
@@ -566,7 +569,7 @@ static void req_stat_job_step2(
         if(LOGLEVEL >= 7)
           {
           sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
-          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, job_id, log_buf);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
           }
 
       reply_free(preply);
@@ -775,7 +778,8 @@ static void req_stat_job_step2(
         {
         pque = get_jobs_queue(&pjob);
 
-        if (pjob == NULL)
+        if ((pjob == NULL) ||
+            (pque == NULL))
           goto nextjob;
         
         if (pque->qu_qs.qu_type != QTYPE_Execution)
@@ -821,7 +825,8 @@ static void req_stat_job_step2(
 
 nextjob:
 
-    pthread_mutex_unlock(pjob->ji_mutex);
+    if (pjob != NULL)
+      pthread_mutex_unlock(pjob->ji_mutex);
 
     if (type == tjstJob)
       break;
@@ -1036,15 +1041,7 @@ void stat_update(
         else
           {
           /* save so we can recover resources used */
-          job_save(pjob, SAVEJOB_FULL);
-          }
-#endif    /* USESAVEDRESOURCES */
-
-#ifdef USESAVEDRESOURCES
-        else
-          {
-          /* save so we can recover resources used */
-          job_save(pjob, SAVEJOB_FULL);
+          job_save(pjob, SAVEJOB_FULL, 0);
           }
 #endif    /* USESAVEDRESOURCES */
 
@@ -1411,12 +1408,16 @@ int get_numa_statuses(
  */
 
 int req_stat_node(
-    struct batch_request *preq)
+
+  struct batch_request *preq)
+
   {
-  int                   rc = PBSE_NONE;
   char                 *name;
+
+  int                   rc   = PBSE_NONE;
   int                   type = 0;
   int                   bad  = 0;
+
   struct pbsnode       *pnode = NULL;
   struct batch_reply   *preply;
   struct prop props;
@@ -1477,11 +1478,14 @@ int req_stat_node(
       {
       rc = PBSE_UNKNODE;
       req_reject(rc, 0, preq, NULL, "cannot locate specified node");
-      return rc;
+      return(rc);
       }
 
     /* get the status on all of the numa nodes */
-    rc = get_numa_statuses(pnode,preq,&bad,&preply->brp_un.brp_status);
+    if (pnode->nd_is_alps_reporter == TRUE)
+      rc = get_alps_statuses(pnode, preq, &bad, &preply->brp_un.brp_status);
+    else
+      rc = get_numa_statuses(pnode, preq, &bad, &preply->brp_un.brp_status);
 
     unlock_node(pnode, __func__, "type == 0", LOGLEVEL);
     }
@@ -1500,7 +1504,12 @@ int req_stat_node(
         }
 
       /* get the status on all of the numa nodes */
-      if ((rc = get_numa_statuses(pnode,preq,&bad,&preply->brp_un.brp_status)) != 0)
+      if (pnode->nd_is_alps_reporter == TRUE)
+        rc = get_alps_statuses(pnode, preq, &bad, &preply->brp_un.brp_status);
+      else
+        rc = get_numa_statuses(pnode, preq, &bad, &preply->brp_un.brp_status);
+      
+      if (rc != PBSE_NONE)
         {
         unlock_node(pnode, __func__, "type != 0, rc != 0, get_numa_statuses", LOGLEVEL);
         break;
@@ -1530,7 +1539,7 @@ int req_stat_node(
       }
     }
 
-  return rc;
+  return(rc);
   }  /* END req_stat_node() */
 
 
@@ -1541,7 +1550,7 @@ int req_stat_node(
  * status_node - Build the status reply for a single node.
  */
 
-static int status_node(
+int status_node(
 
   struct pbsnode       *pnode,    /* ptr to node receiving status query */
   struct batch_request *preq,
@@ -1623,15 +1632,17 @@ int req_stat_svr(
   struct batch_reply   *preply;
 
   struct brp_status    *pstat;
-  int                  *nc;
   int                   bad = 0;
   char                  nc_buf[128];
   int                   numjobs;
+  int                   netrates[3];
+
+  memset(netrates, 0, sizeof(netrates));
 
   /* update count and state counts from sv_numjobs and sv_jobstates */
-  pthread_mutex_lock(server.sv_qs_mutex);
+  lock_sv_qs_mutex(server.sv_qs_mutex, __func__);
   numjobs = server.sv_qs.sv_numjobs;
-  pthread_mutex_unlock(server.sv_qs_mutex);
+  unlock_sv_qs_mutex(server.sv_qs_mutex, __func__);
   
   pthread_mutex_lock(server.sv_attr_mutex);
   server.sv_attr[SRV_ATR_TotalJobs].at_val.at_long = numjobs;
@@ -1646,10 +1657,8 @@ int req_stat_svr(
   
   pthread_mutex_unlock(server.sv_jobstates_mutex);
 
-  nc = netcounter_get(); /* locks netrates mutex */
-  snprintf(nc_buf, 127, "%d %d %d", *nc, *(nc + 1), *(nc + 2));
-  
-  pthread_mutex_unlock(netrates_mutex);
+  netcounter_get(netrates);
+  snprintf(nc_buf, 127, "%d %d %d", netrates[0], netrates[1], netrates[2]);
 
   if (server.sv_attr[SRV_ATR_NetCounter].at_val.at_str != NULL)
     free(server.sv_attr[SRV_ATR_NetCounter].at_val.at_str);
@@ -1723,9 +1732,9 @@ int req_stat_svr(
 
 static void update_state_ct(
     
-  attribute *pattr,
-  int *ct_array,
-  char *buf)
+  pbs_attribute *pattr,
+  int           *ct_array,
+  char          *buf)
 
   {
   static char *statename[] = { "Transit", "Queued", "Held",
