@@ -77,8 +77,8 @@
 * without reference to its choice of law rules.
 */
 /*
- * svr_attr_def is the array of attribute definitions for the server.
- * Each legal server attribute is defined here
+ * svr_attr_def is the array of pbs_attribute definitions for the server.
+ * Each legal server pbs_attribute is defined here
  */
 
 #include <pbs_config.h>		/* the master config generated by configure */
@@ -89,38 +89,38 @@
 
 /* External Functions Referenced */
 
-extern int manager_oper_chk (attribute * pattr, void *pobject, int actmode);
-extern int servername_chk (attribute * pattr, void *pobject, int actmode);
-extern int schiter_chk (attribute * pattr, void *pobject, int actmode);
+extern int manager_oper_chk (pbs_attribute * pattr, void *pobject, int actmode);
+extern int servername_chk (pbs_attribute * pattr, void *pobject, int actmode);
+extern int schiter_chk (pbs_attribute * pattr, void *pobject, int actmode);
 
-extern int nextjobnum_chk (attribute * pattr, void *pobject, int actmode);
-extern int set_nextjobnum (attribute * attr, attribute * new,
+extern int nextjobnum_chk (pbs_attribute * pattr, void *pobject, int actmode);
+extern int set_nextjobnum (pbs_attribute * attr, pbs_attribute * new,
 			   enum batch_op op);
 
-extern int poke_scheduler (attribute * pattr, void *pobject, int actmode);
+extern int poke_scheduler (pbs_attribute * pattr, void *pobject, int actmode);
 
-extern int encode_svrstate (attribute * pattr, tlist_head * phead,
+extern int encode_svrstate (pbs_attribute * pattr, tlist_head * phead,
 			    char *aname, char *rsname, int mode, int perm);
 
-extern int decode_rcost (attribute * patr, char *name, char *rn, char *val, int perm);
-extern int encode_rcost (attribute * attr, tlist_head * phead, char *atname,
+extern int decode_rcost (pbs_attribute * patr, char *name, char *rn, char *val, int perm);
+extern int encode_rcost (pbs_attribute * attr, tlist_head * phead, char *atname,
 			 char *rsname, int mode, int perm);
-extern int set_rcost (attribute * attr, attribute * new, enum batch_op);
-extern void free_rcost (attribute * attr);
-extern int set_null (attribute * patr, attribute * new, enum batch_op op);
+extern int set_rcost (pbs_attribute * attr, pbs_attribute * new, enum batch_op);
+extern void free_rcost (pbs_attribute * attr);
+extern int set_null (pbs_attribute * patr, pbs_attribute * new, enum batch_op op);
 
-extern int token_chk (attribute * pattr, void *pobject, int actmode);
-extern int set_tokens (struct attribute *attr, struct attribute *new,
+extern int token_chk (pbs_attribute * pattr, void *pobject, int actmode);
+extern int set_tokens (struct pbs_attribute *attr, struct pbs_attribute *new,
 		       enum batch_op op);
 
-extern int extra_resc_chk (attribute * pattr, void *pobject, int actmode);
-extern void free_extraresc (attribute * attr);
-extern void restore_attr_default (struct attribute *);
+extern int extra_resc_chk (pbs_attribute * pattr, void *pobject, int actmode);
+extern void free_extraresc (pbs_attribute * attr);
+extern void restore_attr_default (struct pbs_attribute *);
 
 /* DIAGTODO: write diag_attr_def.c */
 
 /*
- * The entries for each attribute are (see attribute.h):
+ * The entries for each pbs_attribute are (see attribute.h):
  * name,
  * decode function,
  * encode function,
@@ -1295,7 +1295,31 @@ attribute_def svr_attr_def[] =
    ATR_TYPE_LONG,
    PARENT_TYPE_SERVER},
 
-  /* site supplied server attribute definitions if any, see site_svr_attr_*.h  */
+  /* SRV_ATR_CrayEnabled */
+  {ATTR_crayenabled,   /* "cray_enabled" */
+   decode_b,
+   encode_b,
+   set_b,
+   comp_b,
+   free_null,
+   NULL_FUNC,
+   MGR_ONLY_SET,
+   ATR_TYPE_LONG,
+   PARENT_TYPE_SERVER},
+
+  /* SRV_ATR_InteractiveJobsCanRoam */ /* NOTE: this only has effect if pbs_server is cray enabled */
+  {ATTR_interactivejobscanroam, /* "interactive_jobs_can_roam" */
+   decode_b,
+   encode_b,
+   set_b,
+   comp_b,
+   free_null,
+   NULL_FUNC,
+   MGR_ONLY_SET,
+   ATR_TYPE_LONG,
+   PARENT_TYPE_SERVER},
+
+  /* site supplied server pbs_attribute definitions if any, see site_svr_attr_*.h  */
 #include "site_svr_attr_def.h"
 
   };
