@@ -98,6 +98,7 @@
 #include "mcom.h"
 #include "utils.h"
 #include "svr_func.h" /* get_svr_attr_* */
+#include "net_cache.h"
 
 /* Global Data */
 
@@ -199,8 +200,8 @@ int svr_authorize_jobreq(
 int svr_authorize_req(
 
   struct batch_request *preq,
-  char *owner,
-  char *submit_host)
+  char                 *owner,
+  char                 *submit_host)
 
   {
   /* does requestor have special privileges? */
@@ -254,7 +255,6 @@ int svr_get_privilege(
   char *host)  /* I */
 
   {
-  char  id[] = "svr_get_privilege";
   int   is_root = 0;
   int   priv = (ATR_DFLAG_USRD | ATR_DFLAG_USWR);
   int   num_host_chars;
@@ -262,12 +262,14 @@ int svr_get_privilege(
   char  host_no_port[PBS_MAXHOSTNAME+1];
   char *colon_loc = NULL;
   char  log_buf[LOCAL_LOG_BUF_SIZE];
+  char *other_host;
+  int   other_priv = 0;
 
   if (!user)
     {
     sprintf(log_buf, "Invalid user: %s", "null");
 
-    log_record(PBSEVENT_SECURITY,PBS_EVENTCLASS_SERVER,id,log_buf);
+    log_record(PBSEVENT_SECURITY, PBS_EVENTCLASS_SERVER, __func__, log_buf);
 
     return(0);
     }
@@ -277,7 +279,7 @@ int svr_get_privilege(
     {
     sprintf(log_buf, "Invalid user: %s", user);
 
-    log_record(PBSEVENT_SECURITY,PBS_EVENTCLASS_SERVER,id,log_buf);
+    log_record(PBSEVENT_SECURITY, PBS_EVENTCLASS_SERVER, __func__, log_buf);
      
     return(0);
     }
@@ -311,7 +313,7 @@ int svr_get_privilege(
     {
     snprintf(log_buf, sizeof(log_buf), "Invalid host: %s", host);
 
-    log_record(PBSEVENT_SECURITY,PBS_EVENTCLASS_SERVER,id,log_buf);
+    log_record(PBSEVENT_SECURITY, PBS_EVENTCLASS_SERVER, __func__, log_buf);
 
     return(0);
     }
@@ -328,10 +330,9 @@ int svr_get_privilege(
     }
 #else /* __CYGWIN__ */
 
-/*  snprintf(log_buf, sizeof(log_buf), "%s - %s\n", user, server_host);
-  log_record(PBSEVENT_SECURITY, PBS_EVENTCLASS_SERVER, "svr_get_privilege", log_buf); */
   if ((strcmp(user, PBS_DEFAULT_ADMIN) == 0) &&
-       (!strcasecmp(host_no_port, server_host) || !strcasecmp(host_no_port, server_localhost)))
+       ((!strcasecmp(host_no_port, server_host)) || 
+        (!strcasecmp(host_no_port, server_localhost))))
     {
     is_root = 1;
 
@@ -363,6 +364,16 @@ int svr_get_privilege(
     }
   pthread_mutex_unlock(server.sv_attr_mutex);
 
+  /* resolve using the other hostname (if available) and give the higher privilege */
+  other_host = get_cached_fullhostname(host, NULL);
+  
+  if ((other_host != NULL) &&
+      (strcmp(host, other_host)))
+    other_priv = svr_get_privilege(user, other_host);
+
+  if (other_priv > priv)
+    priv = other_priv;
+
   return(priv);
   }  /* END svr_get_privilege() */
 
@@ -384,11 +395,10 @@ int authenticate_user(
   char   **autherr) /* O */
 
   {
-  char id[] = "authenticate_user";
   int    rc;
   char   uath[PBS_MAXUSER + PBS_MAXHOSTNAME + 1];
   time_t time_now = time(NULL);
-  char error_msg[1024];
+  char   error_msg[1024];
   long   acl_enabled = FALSE;
 
 #ifdef MUNGE_AUTH
@@ -407,11 +417,7 @@ int authenticate_user(
       *autherr = strdup("User not in authorized user list.");
       sprintf(error_msg, "%s Requested user %s: requested from host %s",
                      *autherr, preq->rq_user, preq->rq_host);
-      log_event(
-        PBSEVENT_ADMIN,
-        PBS_EVENTCLASS_SERVER,
-        id,
-        error_msg);
+      log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, __func__, error_msg);
       return(PBSE_BADCRED);
       }
     }
@@ -421,26 +427,57 @@ int authenticate_user(
     *autherr = strdup("Users do not match");
     sprintf(error_msg, "%s: Requested user %s: credential user %s: requested from host %s",
                    *autherr, preq->rq_user, pcred->username, preq->rq_host);
-    log_event(
-      PBSEVENT_ADMIN,
-      PBS_EVENTCLASS_SERVER,
-      id,
-      error_msg);
+    log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, __func__, error_msg);
     return(PBSE_BADCRED);
     }
 #endif
 
   if (strncmp(preq->rq_host, pcred->hostname, PBS_MAXHOSTNAME))
     {
-    *autherr = strdup("Hosts do not match");
-    sprintf(error_msg, "%s: Requested host %s: credential host: %s",
-                   *autherr, preq->rq_host, pcred->hostname);
-    log_event(
-      PBSEVENT_ADMIN,
-      PBS_EVENTCLASS_SERVER,
-      id,
-      error_msg);
-    return(PBSE_BADCRED);
+    struct sockaddr_in *sai1;
+    struct sockaddr_in *sai2;
+    struct addrinfo    *addr_info1;
+    struct addrinfo    *addr_info2;
+
+    sai1 = get_cached_addrinfo(preq->rq_host);
+    sai2 = get_cached_addrinfo(pcred->hostname);
+
+    if ((sai1 == NULL) &&
+        (getaddrinfo(preq->rq_host, NULL, NULL, &addr_info1) == PBSE_NONE))
+      {
+      sai1 = (struct sockaddr_in *)addr_info1->ai_addr;
+      insert_addr_name_info(preq->rq_host, addr_info1->ai_canonname, sai1);
+      }
+
+    if ((sai2 == NULL) &&
+        (getaddrinfo(pcred->hostname, NULL, NULL, &addr_info2) == PBSE_NONE))
+      {
+      sai2 = (struct sockaddr_in *)addr_info2->ai_addr;
+      insert_addr_name_info(pcred->hostname, addr_info2->ai_canonname, sai2);
+      }
+
+    if ((sai1 == NULL) ||
+        (sai2 == NULL) ||
+        (memcmp(sai1, sai2, sizeof(struct sockaddr_in))))
+      {
+      *autherr = strdup("Hosts do not match");
+      
+      sprintf(error_msg, "%s: Requested host %s: credential host: %s",
+        *autherr, preq->rq_host, pcred->hostname);
+      log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, __func__, error_msg);
+    
+      if (addr_info1 != NULL)
+        freeaddrinfo(addr_info1);
+      if (addr_info2 != NULL)
+        freeaddrinfo(addr_info2);
+      
+      return(PBSE_BADCRED);
+      }
+      
+    if (addr_info1 != NULL)
+      freeaddrinfo(addr_info1);
+    if (addr_info2 != NULL)
+      freeaddrinfo(addr_info2);
     }
 
   if (pcred->timestamp)
