@@ -91,7 +91,7 @@
  * prefix_std_file()  - build the fully prefixed default name for std e/o
  * get_jobowner()     - get job owner name without @host suffix
  * set_resc_deflt()   - set unspecified resource_limit to default values
- * set_statechar()    - set the job state attribute character value
+ * set_statechar()    - set the job state pbs_attribute character value
  *
  * Private functions
  * chk_svr_resc_limit() - check job requirements againt queue/server limits
@@ -142,7 +142,7 @@
 /* Private Functions */
 
 void default_std(job *, int key, dynamic_string *ds);
-void eval_checkpoint(attribute *j, attribute *q);
+void eval_checkpoint(pbs_attribute *j, pbs_attribute *q);
 
 /* Global Data Items: */
 
@@ -292,7 +292,7 @@ int svr_enquejob(
   int  prev_job_index)  /* I */
 
   {
-  attribute     *pattrjb;
+  pbs_attribute *pattrjb;
   attribute_def *pdef;
   job           *pjcur;
   pbs_queue     *pque;
@@ -314,10 +314,18 @@ int svr_enquejob(
     {
     return(PBSE_UNKQUE);
     }
+
   /* This is called when a job is not yet in the queue,
    * so find_job can not be used.... */
   pthread_mutex_lock(pjob->ji_mutex);
 
+  if (pjob->ji_being_recycled == TRUE)
+    {
+    unlock_queue(pque, __func__, NULL, 0);
+    pthread_mutex_unlock(pjob->ji_mutex);
+    return(PBSE_JOB_RECYCLED);
+    }
+
   /* add job to server's 'all job' list and update server counts */
 
 #ifndef NDEBUG
@@ -338,12 +346,12 @@ int svr_enquejob(
       insert_job_after_index(&alljobs, prev_job_index, pjob);
 
     if (has_sv_qs_mutex == FALSE)
-      pthread_mutex_lock(server.sv_qs_mutex);
+      lock_sv_qs_mutex(server.sv_qs_mutex, __func__);
 
     server.sv_qs.sv_numjobs++;
     
     if (has_sv_qs_mutex == FALSE)
-      pthread_mutex_unlock(server.sv_qs_mutex);
+      unlock_sv_qs_mutex(server.sv_qs_mutex, __func__);
 
     pthread_mutex_lock(server.sv_jobstates_mutex);
 
@@ -355,10 +363,12 @@ int svr_enquejob(
   /* place into array_summary if necessary */
   if (pjob->ji_is_array_template || pjob->ji_arraystruct == NULL)
     {
-    if (has_job(&array_summary, pjob) == FALSE)
+    if ((rc = has_job(&array_summary, pjob)) == FALSE)
       {
       insert_job(&array_summary, pjob);
       }
+    else if (rc == PBSE_JOB_RECYCLED)
+      return(rc);
     } /* END if (pjob->is_array_template) */
 
   /* place into queue in order of queue rank starting at end */
@@ -376,7 +386,10 @@ int svr_enquejob(
     while ((pjcur = next_job_from_back(pque->qu_jobs,&iter)) != NULL)
       {
       if (job_qrank > pjcur->ji_wattr[JOB_ATR_qrank].at_val.at_long)
+        {
+        pthread_mutex_unlock(pjcur->ji_mutex);
         break;
+        }
       if (strcmp(job_id, pjcur->ji_qs.ji_jobid) == 0)
         {
         pthread_mutex_unlock(pjcur->ji_mutex);
@@ -416,7 +429,10 @@ int svr_enquejob(
     while ((pjcur = next_job_from_back(pque->qu_jobs_array_sum,&iter)) != NULL)
       {
       if (job_qrank > pjcur->ji_wattr[JOB_ATR_qrank].at_val.at_long)
+        {
+        pthread_mutex_unlock(pjcur->ji_mutex);
         break;
+        }
       if (strcmp(job_id, pjcur->ji_qs.ji_jobid) == 0)
         {
         pthread_mutex_unlock(pjcur->ji_mutex);
@@ -444,7 +460,7 @@ int svr_enquejob(
       }
     }
 
-  /* update the current location and type attribute */
+  /* update the current location and type pbs_attribute */
 
   pdef    = &job_attr_def[JOB_ATR_in_queue];
 
@@ -556,59 +572,37 @@ int svr_enquejob(
 
 /*
  * svr_dequejob() - remove job from whatever queue its in and reduce counts
+ * Make sure job mutex is unlocked before exiting
  */
 
 int svr_dequejob(
 
-  char *job_id,
-  int  parent_queue_mutex_held) /* I */
+  char *job_id,                  /* I, M */
+  int   parent_queue_mutex_held) /* I */
 
   {
-  int          bad_ct = 0;
-  job         *pjob = NULL;
-  attribute   *pattr;
-  pbs_queue   *pque;
-  resource    *presc;
-  char         log_buf[LOCAL_LOG_BUF_SIZE];
+  int            bad_ct = 0;
+  int            rc = PBSE_NONE;
+  job           *pjob = NULL;
+  pbs_attribute *pattr;
+  pbs_queue     *pque;
+  resource      *presc;
+  char           log_buf[LOCAL_LOG_BUF_SIZE];
 
   /* remove job from server's all job list and reduce server counts */
 
   if ((pjob = find_job(job_id)) == NULL)
-    return PBSE_JOBNOTFOUND;
-
-  /* the only error is if the job isn't present */
-  if (remove_job(&alljobs, pjob) == PBSE_NONE)
-    {
-    if (!pjob->ji_is_array_template)
-      {
-      pthread_mutex_lock(server.sv_qs_mutex);
-
-      if (--server.sv_qs.sv_numjobs < 0)
-        {
-        bad_ct = 1;
-        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, __func__, "sv_numjobs < 0. Recount required.");
-        }
-      
-      pthread_mutex_unlock(server.sv_qs_mutex);
-      pthread_mutex_lock(server.sv_jobstates_mutex);
-
-      if (--server.sv_jobstates[pjob->ji_qs.ji_state] < 0)
-        {
-        bad_ct = 1;
-        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, __func__, "sv_jobstates < 0. Recount required.");
-        }
-      
-      pthread_mutex_unlock(server.sv_jobstates_mutex);
-      }
-    }
+    return(PBSE_JOBNOTFOUND);
 
   if (parent_queue_mutex_held == FALSE)
     {
     pque = get_jobs_queue(&pjob);
-    if (pjob == NULL)
+
+    if ((pjob == NULL) ||
+        (pque == NULL))
       {
       log_err(PBSE_JOBNOTFOUND, __func__, "Job lost while acquiring queue");
-      return PBSE_JOBNOTFOUND;
+      return(PBSE_JOBNOTFOUND);
       }
     }
   else
@@ -616,7 +610,7 @@ int svr_dequejob(
 
   if (pque != NULL)
     {
-    if (remove_job(pque->qu_jobs,pjob) == PBSE_NONE)
+    if ((rc = remove_job(pque->qu_jobs, pjob)) == PBSE_NONE)
       {
       if (--pque->qu_numjobs < 0)
         {
@@ -637,9 +631,15 @@ int svr_dequejob(
         bad_ct = 1;
         }
       }
+    else if (rc == PBSE_JOB_RECYCLED)
+      {
+      /* calling functions know this return code means the job is gone */
+      return(PBSE_JOBNOTFOUND);
+      }
 
-    /* just call remove job because nothing happens if it isn't there */
-    remove_job(pque->qu_jobs_array_sum,pjob);
+    /* the only reason to care about the error is if the job is gone */
+    if (remove_job(pque->qu_jobs_array_sum, pjob) == PBSE_JOB_RECYCLED)
+      return(PBSE_JOBNOTFOUND);
 
     pjob->ji_qhdr = NULL;
 
@@ -647,7 +647,7 @@ int svr_dequejob(
       unlock_queue(pque, __func__, NULL, LOGLEVEL);
     }
   else if (pjob == NULL)
-    return PBSE_JOBNOTFOUND;
+    return(PBSE_JOBNOTFOUND);
 
 #ifndef NDEBUG
 
@@ -667,7 +667,6 @@ int svr_dequejob(
   /* clear any default resource values.  */
 
   pattr = &pjob->ji_wattr[JOB_ATR_resource];
-  pthread_mutex_unlock(pjob->ji_mutex);
 
   if (pattr->at_flags & ATR_VFLAG_SET)
     {
@@ -682,6 +681,37 @@ int svr_dequejob(
       }
     }
 
+  /* the only error is if the job isn't present */
+  if ((rc = remove_job(&alljobs, pjob)) == PBSE_NONE)
+    {
+    if (!pjob->ji_is_array_template)
+      {
+      lock_sv_qs_mutex(server.sv_qs_mutex, __func__);
+
+      if (--server.sv_qs.sv_numjobs < 0)
+        {
+        bad_ct = 1;
+        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, __func__, "sv_numjobs < 0. Recount required.");
+        }
+      
+      unlock_sv_qs_mutex(server.sv_qs_mutex, __func__);
+      pthread_mutex_lock(server.sv_jobstates_mutex);
+
+      if (--server.sv_jobstates[pjob->ji_qs.ji_state] < 0)
+        {
+        bad_ct = 1;
+        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, __func__, "sv_jobstates < 0. Recount required.");
+        }
+      
+      pthread_mutex_unlock(server.sv_jobstates_mutex);
+      }
+    }
+  else if (rc == PBSE_JOB_RECYCLED)
+    {
+    /* calling functions know this return code means the job is gone */
+    return(PBSE_JOBNOTFOUND);
+    }
+
   /* notify scheduler a job has been removed */
 
   pthread_mutex_lock(svr_do_schedule_mutex);
@@ -691,7 +721,7 @@ int svr_dequejob(
   listener_command = SCH_SCHEDULE_TERM;
   pthread_mutex_unlock(listener_command_mutex);
 
-  return PBSE_NONE;
+  return(PBSE_NONE);
   }  /* END svr_dequejob() */
 
 
@@ -753,10 +783,14 @@ int svr_setjobstate(
       {
       changed = 1;
 
-      pthread_mutex_lock(server.sv_jobstates_mutex);
-      server.sv_jobstates[oldstate]--;
-      server.sv_jobstates[newstate]++;
-      pthread_mutex_unlock(server.sv_jobstates_mutex);
+      /* the array job isn't actually a job so don't count it here */
+      if (pjob->ji_is_array_template == FALSE)
+        {
+        pthread_mutex_lock(server.sv_jobstates_mutex);
+        server.sv_jobstates[oldstate]--;
+        server.sv_jobstates[newstate]++;
+        pthread_mutex_unlock(server.sv_jobstates_mutex);
+        }
 
       if (has_queue_mutex == FALSE)
         {
@@ -773,8 +807,12 @@ int svr_setjobstate(
 
       if (pque != NULL)
         {
-        pque->qu_njstate[oldstate]--;
-        pque->qu_njstate[newstate]++;
+        /* the array job isn't actually a job so don't count it here */
+        if (pjob->ji_is_array_template == FALSE)
+          {
+          pque->qu_njstate[oldstate]--;
+          pque->qu_njstate[newstate]++;
+          }
 
         /* if execution queue, and eligibility to run has improved, */
         /* notify the scheduler */
@@ -908,7 +946,7 @@ void svr_evaljobstate(
  * variable of a job
  *
  * Returns a pointer to the start of the value or NULL if the variable
- * is not found in the variable_list attribute.
+ * is not found in the variable_list pbs_attribute.
  */
 
 char *get_variable(
@@ -943,10 +981,10 @@ char *get_variable(
 
 resource *get_resource(
 
-  attribute    *p_queattr,  /* I */
-  attribute    *p_svrattr,  /* I */
-  resource_def *rscdf,      /* I */
-  int          *fromQueue)  /* O */
+  pbs_attribute *p_queattr,  /* I */
+  pbs_attribute *p_svrattr,  /* I */
+  resource_def  *rscdf,      /* I */
+  int           *fromQueue)  /* O */
 
   {
   resource *pr;
@@ -987,10 +1025,10 @@ resource *get_resource(
 
 int chk_svr_resc_limit(
 
-  attribute *jobatr, /* I */
-  pbs_queue *pque,   /* I */
-  int       qtype,   /* I */
-  char      *EMsg)   /* O (optional,minsize=1024) */
+  pbs_attribute *jobatr, /* I */
+  pbs_queue     *pque,   /* I */
+  int            qtype,   /* I */
+  char          *EMsg)   /* O (optional,minsize=1024) */
 
   {
   int           dummy;
@@ -1511,7 +1549,7 @@ int count_queued_jobs(
 
 
 /*
- * chk_resc_limits - check job Resource_Limits attribute against the queue
+ * chk_resc_limits - check job Resource_Limits pbs_attribute against the queue
  * and server maximum and mininum values.
  *
  * return 0 if within limits, or an pbs_error number if not.
@@ -1519,9 +1557,9 @@ int count_queued_jobs(
 
 int chk_resc_limits(
 
-  attribute *pattr,  /* I */
-  pbs_queue *pque,   /* I */
-  char      *EMsg)   /* O (optional,minsize=1024) */
+  pbs_attribute *pattr,  /* I */
+  pbs_queue     *pque,   /* I */
+  char          *EMsg)   /* O (optional,minsize=1024) */
 
   {
   int resc_lt;
@@ -1645,7 +1683,7 @@ int svr_chkque(
       return(PBSE_UNKRESC);
       }
 
-    /* 1d. cannot have an unknown attribute */
+    /* 1d. cannot have an unknown pbs_attribute */
 
     if (pjob->ji_wattr[JOB_ATR_UNKN].at_flags & ATR_VFLAG_SET)
       {
@@ -2113,7 +2151,7 @@ void job_wait_over(
 
     pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_HASWAIT;
     
-    /* clear the exectime attribute */
+    /* clear the exectime pbs_attribute */
     job_attr_def[JOB_ATR_exectime].at_free(
       &pjob->ji_wattr[JOB_ATR_exectime]);
     
@@ -2136,14 +2174,14 @@ void job_wait_over(
  * wait time of the job.
  *
  * This is called as the at_action (see attribute.h) function associated
- * with the execution-time job attribute.
+ * with the execution-time job pbs_attribute.
  */
 
 int job_set_wait(
 
-  attribute *pattr,
-  void      *j,    /* a (job *) cast to void * */
-  int        mode) /* unused, do it for all action modes */
+  pbs_attribute *pattr,
+  void          *j,    /* a (job *) cast to void * */
+  int            mode) /* unused, do it for all action modes */
 
   {
   long       when;
@@ -2337,6 +2375,16 @@ void get_jobowner(
   {
   int i;
 
+  if (to == NULL)
+    return;
+
+  if (from == NULL)
+    {
+    *to = 0;
+    return;
+    }
+
+
   for (i = 0;i < PBS_MAXUSER;++i)
     {
     if ((*(from + i) == '@') || (*(from + i) == '\0'))
@@ -2363,8 +2411,8 @@ void get_jobowner(
 
 void set_deflt_resc(
 
-  attribute *jb,
-  attribute *dflt)
+  pbs_attribute *jb,
+  pbs_attribute *dflt)
 
   {
   resource *prescjb;
@@ -2431,12 +2479,12 @@ void set_deflt_resc(
 
 void set_resc_deflt(
 
-  job       *pjob,            /* I (modified) */
-  attribute *ji_wattr,        /* I (optional) decoded attributes  */
-  int        has_queue_mutex) /* I */
+  job           *pjob,            /* I (modified) */
+  pbs_attribute *ji_wattr,        /* I (optional) decoded attributes  */
+  int            has_queue_mutex) /* I */
 
   {
-  attribute     *ja;
+  pbs_attribute *ja;
 
   pbs_queue     *pque;
   attribute_def *pdef;
@@ -2574,7 +2622,7 @@ void set_resc_deflt(
 
 
   /*
-   * set_statechar - set the job state attribute to the letter that correspondes
+   * set_statechar - set the job state pbs_attribute to the letter that correspondes
    * to its current state.
    */
 
@@ -2613,15 +2661,15 @@ void set_statechar(
 
 
 /*
- * eval_checkpoint - if the job's checkpoint attribute is "c=nnnn" and
+ * eval_checkpoint - if the job's checkpoint pbs_attribute is "c=nnnn" and
  *  nnnn is less than the queue' minimum checkpoint time, reset
  * to the queue min time.
  */
 
 void eval_checkpoint(
     
-  attribute *jobckp, /* job's checkpoint attribute */
-  attribute *queckp) /* queue's checkpoint attribute */
+  pbs_attribute *jobckp, /* job's checkpoint pbs_attribute */
+  pbs_attribute *queckp) /* queue's checkpoint pbs_attribute */
 
   {
   int   jobs;
@@ -2684,11 +2732,11 @@ void correct_ct()
   char          log_buf[LOCAL_LOG_BUF_SIZE];
   
   lock_startup();
-  pthread_mutex_lock(server.sv_qs_mutex);
+  lock_sv_qs_mutex(server.sv_qs_mutex, __func__);
   sprintf(log_buf, "Job state counts incorrect, server %d: ", server.sv_qs.sv_numjobs);
   
   server.sv_qs.sv_numjobs = 0;
-  pthread_mutex_unlock(server.sv_qs_mutex);
+  unlock_sv_qs_mutex(server.sv_qs_mutex, __func__);
   
   pthread_mutex_lock(server.sv_jobstates_mutex);
   
@@ -2743,9 +2791,10 @@ void correct_ct()
     unlock_queue(pque, "correct_ct", NULL, LOGLEVEL);
     } /* END for each queue */
   
-  pthread_mutex_lock(server.sv_qs_mutex);
+  sprintf(log_buf, "%s:2", __func__);
+  lock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
   server.sv_qs.sv_numjobs = num_jobs;
-  pthread_mutex_unlock(server.sv_qs_mutex);
+  unlock_sv_qs_mutex(server.sv_qs_mutex, log_buf);
   
   pthread_mutex_lock(server.sv_jobstates_mutex);
   for (i = 0; i < PBS_NUMJOBSTATE; i++)
