@@ -266,6 +266,7 @@ static int local_move(
   int        mtype;
   char       log_buf[LOCAL_LOG_BUF_SIZE];
   char       job_id[PBS_MAXSVRJOBID+1];
+  int        rc;
 
   /* search for destination queue */
   /* CAUTION!!! This code is very complex - be very careful editing */
@@ -348,9 +349,11 @@ static int local_move(
 
   strcpy(job_id, pjob->ji_qs.ji_jobid);
   pthread_mutex_unlock(pjob->ji_mutex);
-  svr_dequejob(job_id, TRUE);
-  if ((pjob = find_job(job_id)) == NULL)
-    return(PBSE_JOBNOTFOUND);
+  
+  /* if we come out of svr_dequejob successfully pjob->ji_mutex will be locked */
+  rc = svr_dequejob(job_id, TRUE); 
+  if (rc)
+    return(rc);
 
   strcpy(pjob->ji_qs.ji_queue, destination);
 
@@ -359,7 +362,8 @@ static int local_move(
   unlock_queue(dest_que, __func__, NULL, 0);
   unlock_queue(routing_que, __func__, "success", LOGLEVEL);
 
-  *my_err = svr_enquejob(pjob, FALSE, -1);
+  if ((*my_err = svr_enquejob(pjob, FALSE, -1)) == PBSE_JOB_RECYCLED)
+    return(-1);
 
   if (parent_queue_mutex_held == TRUE)
     {
@@ -420,6 +424,8 @@ void finish_routing_processing(
         {
         /* job delete in progress, just set to queued status */
         svr_setjobstate(pjob, JOB_STATE_QUEUED, JOB_SUBSTATE_ABORT, FALSE);
+        
+        svr_mailowner(pjob, 'a', TRUE, "Coudln't route job to remote server");
 
         pthread_mutex_unlock(pjob->ji_mutex);
 
@@ -431,6 +437,8 @@ void finish_routing_processing(
       /* fall through */
 
     default: /* try routing again */
+       
+      svr_mailowner(pjob, 'a', TRUE, "Coudln't route job to remote server");
 
       /* force re-eval of job state out of Transit */
 
@@ -625,6 +633,7 @@ int send_job_work(
 
   {
   int                   rc = LOCUTION_FAIL;
+  int                   ret = PBSE_NONE;
   int                   local_errno = 0;
   tlist_head            attrl;
   enum conn_type        cntype = ToServerDIS;
@@ -648,22 +657,23 @@ int send_job_work(
   unsigned char         has_job_script = FALSE;
   unsigned char         job_has_run = FALSE;
   job                  *pjob = NULL;
-  char                 job_destin[PBS_MAXROUTEDEST+1];
+  char                  job_destin[PBS_MAXROUTEDEST+1];
 
   struct attropl       *pqjatr;      /* list (single) of attropl for quejob */
-  attribute            *pattr;
+  pbs_attribute        *pattr;
   mbool_t               Timeout = FALSE;
   
   unsigned long         job_momaddr = -1;
   unsigned short        job_momport = -1;
 
   if ((pjob = find_job(job_id)) == NULL)
-    return PBSE_JOBNOTFOUND;
+    return(PBSE_JOBNOTFOUND);
 
   if (strlen(pjob->ji_qs.ji_destin) != 0)
     strcpy(job_destin, pjob->ji_qs.ji_destin);
   else
     job_destin[0] = '\0';
+
   job_momaddr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
   job_momport = pjob->ji_qs.ji_un.ji_exect.ji_momport;
 
@@ -709,9 +719,9 @@ int send_job_work(
 
     /* clear default resource settings */
     pthread_mutex_unlock(pjob->ji_mutex);
-    svr_dequejob(job_id, FALSE);
-    if ((pjob = find_job(job_id)) == NULL)
-      return PBSE_JOBNOTFOUND;
+    ret = svr_dequejob(job_id, FALSE);
+    if (ret)
+      return(ret);
     }
 
   pattr = pjob->ji_wattr;
